Function Pause {
    [CmdletBinding()]
    param (
        [Parameter(Mandatory=$true)][string]$Message='Press any key to continue...'
    )
    
    Write-Host -NoNewLine $Message
    $null = $Host.UI.RawUI.ReadKey('NoEcho,IncludeKeyDown')
    Write-Host ''
}

Function CreateInstanceNumber{
    [CmdletBinding()]
    param(
        [Parameter(Mandatory=$true)][int]$number
    )
    '{0:D2}' -f $number
}

Function AddNumberToPrefix{
    [CmdletBinding()]
    param(
        [Parameter(Mandatory=$true)][string]$prefix,
        [Parameter(Mandatory=$true)][int]$number
    )

    $fullnumber = CreateInstanceNumber -number $number
    if (-not $prefix -eq '') {
        $prefix + '_' + $fullnumber
    } else {
        Write-Error 'Instance prefix cannot be empty' -Category InvalidArgument
        Exit 1
    }
}

Function FuncStopService{
    [CmdletBinding()]
    param(
        [Parameter(Mandatory=$true)][string]$ServiceName,
        [string]$Part = '',
        [switch]$Force
    )

    if (-not $ServiceName -eq '') {
        $service = Get-Service -name $ServiceName -ErrorAction SilentlyContinue
        if (-not $service ) {
            Write-Host "Service with name $ServiceName could not be found."
        } else {
            if ($service.Status -eq 'Running'){
                Write-Host "Stop Service: $ServiceName"
                $result = Stop-Service $ServiceName -Force:$Force
                Show-Result $result
                if(-not ($Part -eq '')) {
                    Set-Key $Setup "$Part-Running" $ServiceName $true $true
                }
            } else {
                if(-not ($Part -eq '')) {
                    Set-Key $Setup "$Part-Running" $ServiceName $false $true
                }
                Write-Host "Service with name $ServiceName wasn't running."
            }
        }
    } else {
        Write-Error 'Stop Service: No service name provided.' -Category InvalidArgument
        Exit 1
    }
}

Function FuncStartService{
    [CmdletBinding()]
    param(
        [Parameter(Mandatory=$true)][string]$ServiceName,
        [string]$Part = ''
    )

    if (-not $ServiceName -eq '') {
        $service = Get-Service -name $ServiceName -ErrorAction SilentlyContinue
        if (-not $service ) {
            Write-Warning "Service with name $ServiceName could not be found."
        } else {
            if ($service.Status -ne 'Running'){
                if(((Have-Key $Setup "$Part-Running" $ServiceName) -and (Get-Key $Setup "$Part-Running" $ServiceName)) -or ($Part -eq '')){
                    Write-Host "Start named service: $ServiceName"
                    $result = Start-Service $ServiceName
                    Show-Result $result
                }
                else {
                    Write-Host "Service $ServiceName wasn't stopped by $Part"
                }
            } else {
                Write-Host "Service with name $ServiceName was already running."
            }
        }
    } else {
        Write-Error 'Start service: No service name provided.' -Category InvalidArgument
        Exit 1
    }
}

Function FuncRemoveService{
    [CmdletBinding()]
    param(
        [Parameter(Mandatory=$true)][string]$ServiceName,
        [Parameter(Mandatory=$true)][string]$WMIServiceName,
        [string]$Part = ''
    )

    if (-not $ServiceName -eq '') {
        $service = Get-Service -name $ServiceName -ErrorAction SilentlyContinue
        if (-not $service ) {
            Write-Host "Service with name $ServiceName could not be found."
        }
        else {
            (Get-WmiObject -Class Win32_Service -Filter "name='$WMIServiceName'").Delete()
            $service = Get-Service -name $ServiceName -ErrorAction SilentlyContinue
            if (-not $service ) {
                Write-Host "Service with name $WMIServiceName deleted."
            }
            else {
                 Write-Warning "Problem deleting service $WMIServiceName."
            }
        }
    }
    else {
        Write-Error 'Stop Service: No service name provided.' -Category InvalidArgument
        Exit 1
    }
}

Function Safe-Remove-Item {
    <#
    .Synopsis
        Remove files and folders in a recursive manner.
    .Description
        Remove files and folders in a recursive manner. work-around for the Remove-Item -recurse bug
    #>
    [CmdletBinding()]
    param(
        [Parameter(Mandatory=$true)][string]$Path
    )
    Write-Host "Removing: $Path"
    if (([string]::IsNullOrEmpty($Path)) -Or (-Not (Test-Path $Path))) {
        Write-Warning "Path not found: $Path"
    }
    else {
        Get-ChildItem -Path $Path -Recurse | Remove-Item -Force -Recurse | Out-Null
        if(Test-Path $Path) {
            Remove-Item $Path -Force -Recurse | Out-Null
        }
    }
}

Function CreateDirectory {
    [CmdletBinding()]
    param(
        [Parameter(Mandatory=$true)][string]$directory
    )

    If(-not (Test-Path -path $directory)){
        Write-Host "New directory: $directory"
        $result = New-Item -ItemType directory -Path $directory
        Show-Result $result
    } Else {
        Write-Host "Directory already exists: $directory"
    }
}

Function CopyDirectory {
    [CmdletBinding()]
    param(
        [Parameter(Mandatory=$true)][string]$source,
        [Parameter(Mandatory=$true)][string]$destination
    )
    Write-Host "copy `"$source`" `"$destination`""
    $result = copy-item $source $destination -force -recurse
    Show-Result $result
}

Function CreateRegistryKey {
    [CmdletBinding()]
    param(
        [Parameter(Mandatory=$true)][string]$Key
    )

    If(-not (Test-Path -path $Key)) {
        Write-Host "New Registry key: $Key"
        $result = New-Item -Path $Key
        Show-Result $result
    } Else {
        Write-Host "Registry key $Key already exists"
    }
}

Function CreateRegistryValue {
    [CmdletBinding()]
    param(
        [Parameter(Mandatory=$true)][string]$Key,
        [Parameter(Mandatory=$true)][string]$Name,
        [Parameter(Mandatory=$true)]$Value
    )

    $propertyType = '???'
    switch ($Value.GetType().Name) {
        "Int32" { $propertyType = 'DWord' }
        default { $propertyType = 'String' }
    }

    If (-not (Test-Path -path $Key )) {
        Write-Host "New Registry key: $Key : $Name = '$($propertyType):$Value'"
        $result = New-ItemProperty `
            -Path $Key `
            -Name $Name `
            -Value $Value `
            -PropertyType $propertyType
    } else {
        # Check if the property exists - as Test-Path does not work for this purpose we do it the ugly way
        $property = Get-ItemProperty -Path $Key -Name $Name -ErrorAction SilentlyContinue
        
        if ($property -eq $null) {
            Write-Host "New Registry key: $Key : $Name = '$($propertyType):$Value'"
            $result = New-ItemProperty `
                -Path $Key `
                -Name $Name `
                -Value $Value `
                -PropertyType $propertyType
        } else {
            Write-Host "Set Registry key: $Key : $Name = '$($propertyType):$Value'"
            $result = Set-ItemProperty `
                -Path $Key `
                -Name $Name `
                -Value $Value
        }
    }
    Show-Result $result
}

Function Get-IniContent {
    <#
    .Synopsis
        Gets the content of an INI file

    .Description
        Gets the content of an INI file and returns it as a hashtable
         
    .Notes 
        Author    : Oliver Lipkau <oliver@lipkau.net> 
        Blog      : http://oliver.lipkau.net/blog/ 
        Date      : 2010/03/12 
        Version   : 1.0 

        #Requires -Version 2.0

    .Inputs
        System.String

    .Outputs
        System.Collections.Hashtable

    .Parameter FilePath
        Specifies the path to the input file.

    .Example
        $FileContent = Get-IniContent "C:\myinifile.ini"
        -----------
        Description
        Saves the content of the c:\myinifile.ini in a hashtable called $FileContent

    .Example
        $inifilepath | $FileContent = Get-IniContent
        -----------
        Description
        Gets the content of the ini file passed through the pipe into a hashtable called $FileContent

    .Example
        C:\PS>$FileContent = Get-IniContent "c:\settings.ini"
        C:\PS>$FileContent["Section"]["Key"]
        -----------
        Description
        Returns the key "Key" of the section "Section" from the C:\settings.ini file

    .Link
        Out-IniFile
    #>

    [CmdletBinding()]
    Param(
        [ValidateNotNullOrEmpty()]
        [ValidateScript({(Test-Path $_) -and ((Get-Item $_).Extension -eq '.ini')})]
        [Parameter(ValueFromPipeline=$True,Mandatory=$True)]
        [string]$FilePath
    )

    Begin { Write-Verbose "$($MyInvocation.MyCommand.Name):: Function started" }

    Process {
        Write-Verbose "$($MyInvocation.MyCommand.Name):: Processing file: $Filepath"

        $ini = @{}
        switch -regex -file $FilePath
        {
            '^\[(.+)\]$' # Section
            {
                $section = $matches[1]
                $ini[$section] = @{}
                $CommentCount = 0
            }
            '^(;.*)$' # Comment
            {
            #    if (-not ($section)) {
            #        $section = 'No-Section'
            #        $ini[$section] = @{}
            #    }
            #    $value = $matches[1]
            #    $CommentCount = $CommentCount + 1
            #    $name = 'Comment' + $CommentCount
            #    $ini[$section][$name] = $value
            }
            '^[ `t]*([^;].*)=(.*)' # Key
            {
                if (-not ($section)) {
                    $section = 'No-Section'
                    $ini[$section] = @{}
                }
                $name,$value = $matches[1..2]
                $ini[$section][$name] = $value
            }
        }
        Write-Verbose "$($MyInvocation.MyCommand.Name):: Finished Processing file: $path"
        Return $ini
    }

    End { Write-Verbose "$($MyInvocation.MyCommand.Name):: Function ended" }
}

Function Test-CaseSensitivity ($Server, $Catalog, $User, $Password)
{
    [string]$file = Join-Path (Get-Location) 'Scripts\Tools\TestDatabaseCaseSensitivity.sql'
	[string]$output = Join-Path (Get-Location) 'TestDatabaseCaseSensitivity.log'
    [string]$cmd = Get-Key $Setup 'Database' 'CmdClient'
    $ExitCode = Execute-Command $cmd @( '-b', `
            '-l', '30', `
            '-d', $Catalog, `
            '-S', $Server, `
            '-U', $User, `
            '-P', $Password, `
            '-i', $file, `
            '-o', $output
    ) -silent
    if ($ExitCode -ne 0) {
            Write-Error "sqlcmd problems: Database might be case sensitive. More error information can be found in $output." -Category InvalidOperation
            Exit 1
    }
    else {
        Write-Host "Database $Catalog is case insensitive."
    }
    return ($ExitCode -eq 0)
}

Function Check-OMVersion ($DatabaseType, $OMType, $User, $Password, $ConnectionString, $ExpVersion, $Secondary)
{
    $builder = New-Object System.Data.Odbc.OdbcConnectionStringBuilder
    $builder.uid = $User
    $builder.pwd = [string]$Password
    [string]$connstr=$ConnectionString+';'+$builder.ConnectionString+';'
 
    [boolean]$foundVersion = $False;
    [int]$Major = 0
    [int]$Minor = 0
    [int]$Revision = 0
    [string]$TableName = ''  
    [string]$ColumnPrefix = ''  
    if ($OMType -eq 'repository') {
        $TableName = 'MR_DATABASE_VERSION'
        $ColumnPrefix = 'DBV'
    } elseif ($OMType -eq 'runtime') {
        $TableName = 'ODIN_DATABASE_VERSION'
        $ColumnPrefix = 'ODV'
    }
    else {
        Write-Error ("Unsupported Output Management database: $OMType")  
        Exit 1
    }  
    
    [string]$MajorKey = $ColumnPrefix+'_MAJOR_VERSION'
    [string]$MinorKey = $ColumnPrefix+'_MINOR_VERSION'
    [string]$RevisionKey = $ColumnPrefix+'_REVISION'
    
    $Conn = $Null
    $Cmd = $Null
    if ($DatabaseType -eq 'sqlserver') {
        $Conn = New-object Data.SqlClient.SqlConnection
        $Conn.ConnectionString = $connstr
        $Conn.Open()
        $Cmd = New-object system.Data.SqlClient.SqlCommand("SELECT * FROM $TableName",$Conn)
    } elseif ($DatabaseType -eq 'oracle') {
        # The OracleClient library is not loaded by default
        Add-Type -AssemblyName System.Data.OracleClient
        $Conn = New-object Data.OracleClient.OracleConnection
        $Conn.ConnectionString = $connstr
        $Conn.Open()
        $Cmd = New-object system.Data.OracleClient.OracleCommand("SELECT * FROM $TableName",$Conn)
    } else {
        Write-Error ('Trying to connect to an unsupported database type: ' + $DatabaseType)
        Exit 1
    }
    
    try {
        $reader = $Cmd.ExecuteReader()
        while ($reader.Read()) {
            $Major = $reader[$MajorKey]
            $Minor = $reader[$MinorKey]
            $Revision = $reader[$RevisionKey]
            $foundVersion = $True
        }
    }
    catch [System.Management.Automation.MethodInvocationException] {
        # This exception type will also occur if there were connection issues, but these
        # will have been caught by the earlier check-database step, so assume this indicates
        # a missing table.
        Write-Host "The $OMType database version table does not exist"
        $foundVersion = $False
    }
    catch {
        [string]$ErrorMessage = $Error[0].Exception.Message
        Write-Error ("Error while checking Database Version for Output Management $OMType : $ErrorMessage")  
        Exit 1
    }
    finally {
        $Conn.Close()
    }
    
    if ($Secondary) {
        if ($foundVersion){
            [string]$RetrievedVersion = "$Major.$Minor.$Revision"
            
            # -----------------------------------------------------------------------------------------------
            # Workaround for TFS 1214352 [5.2 output management databases actually have version 5.1.1].
            # This workaround is specific for 5.2. This piece of code MUST be updated for the next release.
            # -----------------------------------------------------------------------------------------------
            # We must check if the referred primary database has the same version as the secondary which
            # is being installed. For the workaround, we assume that the secondary is version 5.2, and we
            # therefore check if the primary has version 5.1.1. This check will succeed when the primary is
            # 5.2, and we will not actually reach this point when the primary was 5.1.1 because the database
            # is named differently in 5.1.1.
            # -----------------------------------------------------------------------------------------------
            
            if ('5.1.1' -eq $RetrievedVersion) {
                Write-Host "Connected to existing Output Management $OMType database with version: 5.2"
            }
            else {
                Write-Warning "The Output Management $OMType database content is of an incompatible version: $RetrievedVersion"
                Return $False
            }
        }
        else {
            Write-Warning ("The Output Management $OMType database has no content. A secondary Output Management can only be installed after installation of the primary Output Management has been completed")
            Return $False
        }
    }
    elseif ($foundVersion) {
        Write-Warning ("The Output Management $OMType database already has content. A primary Output Management can only be installed on an empty database")
        Return $False
    }
    
    Return $True
}

Function Test-DBPermissions ($DatabaseType, $ConnectionString, $User, $Password, $packageITPServerBinDir, $outputManagementType = "None")
{
    $checkdatabaseName = ''
    if ($outputManagementType -eq 'None') {
        $checkdatabaseName = 'checkdatabase.exe'
    } else {
        $checkdatabaseName = 'checkdatabase32.exe'
    }
    
    $checkdatabase = Join-Path $packageITPServerBinDir $checkdatabaseName
    if (-not (Test-Path $checkdatabase)) {
		Write-Error "checkdatabase.exe not found at `"$checkdatabase`""
		exit 2;
	}

    # Add the credentials to the connection string  
    $builder = New-Object System.Data.Odbc.OdbcConnectionStringBuilder
    $builder.uid = $User
    $builder.pwd = [string]$Password
    [string]$connstr=$ConnectionString+';'+$builder.ConnectionString+';'
    
    $Parameters = @(
        "/Permissions",
        "`"/DatabaseType=$DatabaseType`"",
        "`"/ConnectionString=$connstr`""
    )
    
    if ($outputManagementType -ne "None") {
        $Parameters += "/outputmanagement"
    }
    if ($outputManagementType -eq "Runtime") {
        $Parameters += "/runtime"
    } elseif ($outputManagementType -eq "Repository") {
        $Parameters += "/repository"
    }
    
    $ExitCode = Execute-Command $checkdatabase $Parameters -silent
    if ($ExitCode -eq 0) {
            Write-Host "Permissions for '$ConnectionString' for user '$User' checked successfully."
    }
    elseif ($ExitCode -eq 1){
            Write-Error "checkdatabase problems: Cannot check permissions with '$ConnectionString' for user '$User'." -Category InvalidOperation
            Exit 1
    }
    elseif ($ExitCode -eq 3){
            Write-Error "Insufficient Permissions for '$ConnectionString' for user '$User'." -Category InvalidOperation
            Exit 1
	}
    elseif ($ExitCode -eq 4){
            Write-Error "Insufficient table space for user '$User'." -Category InvalidOperation
            Exit 1
	}
    else {
            Write-Error "checkdatabase problems: Error while checking permissions with '$ConnectionString' for user '$User'." -Category InvalidOperation
            Exit 1
    }
    $True
}

Function Test-DBDefaultSchema ($DatabaseType, $ConnectionString, $User, $Password, $packageITPServerBinDir)
{
	$checkdatabase = Join-Path $packageITPServerBinDir 'checkdatabase.exe'
	if (-not (Test-Path $checkdatabase)) {
        Write-Error "checkdatabase.exe not found at `"$checkdatabase`""
        exit 2;
	}

	# Add the credentials to the connection string.
        $builder = New-Object System.Data.Odbc.OdbcConnectionStringBuilder
        $builder.uid = $User
        $builder.pwd = [string]$Password
        [string]$connstr=$ConnectionString+';'+$builder.ConnectionString+';'
	$ExitCode = Execute-Command $checkdatabase @( `
			"/Schema",
			"`"/DatabaseType=$DatabaseType`"",
			"`"/ConnectionString=$connstr`""
	) -silent
    if ($ExitCode -eq 0) {
            Write-Host "Default Schema for '$ConnectionString' for user '$User' checked successfully."
    }
    elseif ($ExitCode -eq 1){
            Write-Error "checkdatabase problems: Cannot check default scheam with '$ConnectionString' for user '$User'." -Category InvalidOperation
            Exit 1
    }
    elseif ($ExitCode -eq 3){
            Write-Error "Expected default Schema for '$ConnectionString' for user '$User' not found." -Category InvalidOperation
            Exit 1
	}
    else {
            Write-Error "checkdatabase problems: Error while checking default Schema with '$ConnectionString' for user '$User'." -Category InvalidOperation
            Exit 1
    }
    $True
}

Function Get-ScriptDirectory {
    Split-Path $script:MyInvocation.MyCommand.Path
}

Function ConvertRepositoryDatabase{
    param (
        [Parameter(Mandatory=$true)][string]$RepositoryInstallDir,
        [Parameter(Mandatory=$true)][string]$ItpRepIni,
        [Parameter(Mandatory=$false)][AllowEmptyString()][string]$OverrideServerPort=''
    )
    
    If(Test-Path $RepositoryInstallDir) {
        If(Test-Path $ItpRepIni) {
            Write-Host "CVC: converting database"
            Push-Location ($RepositoryInstallDir)
                $CVCParams = @('/cfg=' + $ItpRepIni)
                If($OverrideServerPort -ne '') {
                    $CVCParams += ('/server.port=' + $OverrideServerPort)
                }
                
                $ExitCode = Execute-Command '.\cvc.exe' $CVCParams
                if ($ExitCode -ne 0) {
                    Pop-Location
                    Write-Error "cvc problems (config)" -Category InvalidOperation
                    exit 1
                }
            Pop-Location
        }
        else {
            Write-Error "Cannot find itprep.ini: $ItpRepIni" -Category InvalidOperation
            exit 1
        }
    }
    else {
        Write-Error "Cannot find directory: $RepositoryInstallDir" -Category InvalidOperation
        exit 1
    }
}

Function ImportMdkFile{
    param (
        [Parameter(Mandatory=$true)][string]$RepositoryInstallDir,
        [Parameter(Mandatory=$true)][string]$ItpRepIni,
        [Parameter(Mandatory=$true)][string]$MdkFile,
        [Parameter(Mandatory=$false)][AllowEmptyString()][string]$OverrideServerPort=''
    )
    
    If(Test-Path $MdkFile) {
        If(Test-Path $RepositoryInstallDir) {
            If(Test-Path $ItpRepIni) {
                Write-Host "repimport: loading mdk file '$MdkFile'"
                Push-Location ($RepositoryInstallDir)
                    $ImportParams = @(
                        "/cfg=$ItpRepIni",
                        "/file=$MdkFile",
                        '/merge',
                        '/trusted'
                    )
                    
                    If($OverrideServerPort -ne '') {
                        $ImportParams += "/server.port=$OverrideServerPort"
                    }
                    
                    $ExitCode = Execute-Command '.\repimport.exe' $ImportParams
                    if ($ExitCode -ne 0) {
                        Pop-Location
                        Write-Error "cvc problems ($MdkFile)" -Category InvalidOperation
                        exit 1
                    }
                Pop-Location
            }
            else {
                Write-Error "Cannot find itprep.ini: $ItpRepIni" -Category InvalidOperation
                exit 1
            }
        }
        else {
            Write-Error "Cannot find directory: $RepositoryInstallDir" -Category InvalidOperation
            exit 1
        }
    }
    else {
        Write-Error "CVC: File '$MdkFile' not found"
        exit 1
    }
}

Function GetValidationId {
    [CmdletBinding()]
    param (
        [Parameter(Mandatory=$true)][string]$InstanceVersion,
        [Parameter(Mandatory=$true)][string]$InstanceName
    )
    
    [string]$registryRoot = ''
    [string]$validationId = ''
    
    if (Test-Path 'HKLM:\SOFTWARE\Wow6432Node') {
        $registryRoot = "HKLM:\SOFTWARE\Wow6432Node\Aia\Package\$InstanceVersion\$InstanceName"
    }
    else {
        $registryRoot = "HKLM:\SOFTWARE\Aia\Package\$InstanceVersion\$InstanceName"
    }

    if (Test-Path $registryRoot) {
        $validationId = (Get-ItemProperty $registryRoot).ValidationId
    }
    else {
        Write-Error "Could not find validation id for instance $InstanceVersion\$InstanceName"
    }
    
    return $validationId
}

Function Get-InstanceName{
    <#
    .Synopsis
        Get a CCM instance name from a validation id

    .Description
        Returns the name of the CCM instance name corresponding to a specific validation id
         
    .Inputs
        System.String

    .Outputs
        System.String

    .Parameter ValidationId
        Specifies the validation id to match.

    .Example
        $name = Get-InstanceName -InstanceId '663e5451-ed60-411b-9e66-304822e7d36f'
    #>

    [CmdletBinding()]
    param (
        [Parameter(Mandatory=$true)][string]$InstanceVersion,
        [Parameter(Mandatory=$true)][string]$InstanceId
    )
    
    [string]$registryRoot = ''
    [string]$instanceName = ''

    if (Test-Path 'HKLM:\SOFTWARE\Wow6432Node') {
        $registryRoot = "HKLM:\SOFTWARE\Wow6432Node\Aia\Package\$instanceVersion"
    }
    else {
        $registryRoot = "HKLM:\SOFTWARE\Aia\Package\$instanceVersion"
    }
    
    if (Test-Path $registryRoot) {
        Get-ChildItem -Path $registryRoot | ForEach-Object { 
            if ((Get-ItemProperty Registry::$_).InstanceId -eq $InstanceId) {
                $instanceName = (Get-ItemProperty Registry::$_).PSChildName
            }
        }
    }
    else {
        Write-Warning "Could not find instance name for instance id $InstanceId"
    }
    
    return $instanceName
}

Function Get-TomcatServices {
    param (
        [Parameter(Mandatory=$true)][Hashtable]$Configuration,
        [Parameter(Mandatory=$true)][string]$Version
    )

    [string[]]$tomcatServices = @()
    
    if (($Version -eq '4.4' -and (IsSet-Key $Configuration 'ITPMDKRepositoryServer' 'Install')) `
         -or (IsSet-Key $Configuration 'ITPDesignerForWeb' 'Install')) {
        $tomcatServices += Get-Key $Configuration 'WebDesigner' 'ServiceName'
    }
    
    if ((IsSet-Key $Configuration 'ITPOnlineJava' 'Install') -or `
        (IsSet-Key $Configuration 'ContractManager' 'Install') -or `
        (IsSet-Key $Configuration 'ITPLogServer' 'Install') -or `
        (IsSet-Key $Configuration 'ITPi' 'Install')) {
        [string]$tomcatServiceToBeStopped = ''  
        if($Version -eq '4.4'){
            $tomcatServiceToBeStopped = Get-Key $Configuration 'Webserver' 'Service'
        } else {
            $tomcatServiceToBeStopped = Get-Key $Configuration 'CCMRuntime' 'ServiceName'
        }
        
        $tomcatServices += $tomcatServiceToBeStopped
    }
    
    return $tomcatServices
}

Function Stop-CCM {
    param (
        [Parameter(Mandatory=$true)][string]$Version,
        [string[]][AllowEmptyCollection()]$TomcatServices,
        [hashtable][AllowEmptyCollection()]$OutputManagementServices,
        [switch]$DisableServices
    )
    
    [string]$programsPath = "*\CCM\Programs\$Version*"
    
	Write-Host "Stopping CCM $Version"
   
    # Do not stop any Output Management services here, as they require a specific start/stop order and force flag
    $runningServices = Get-WmiObject win32_service | ?{$_.State -eq 'Running'} | select Name, PathName | ? { -not ($_.PathName -match '[\\/]Output Management[\\/]') }
    if ($runningServices -ne $null -and $runningServices -ne @()) {
        foreach($service in $runningServices) {
            if($service.PathName -like $programsPath) {
                $result = Stop-Service -name $service.Name.replace('[', '?').replace(']', '?')
                Write-Host ('Stopped Service '+ $service.Name)
                if($DisableServices){
                    $result = Set-Service -name $service.Name -StartupType 'Disabled'
                    Write-Host ('Disabled Service '+ $service.Name)
                }
            }
        }
    }
    
    if ($TomcatServices -ne $null -and $TomcatServices -ne @()) {
        foreach ($tomcatService in $TomcatServices) {
            if (Get-Service -Name $tomcatService -ErrorAction SilentlyContinue) {
                $service = Get-Service -Name $tomcatService
                if ($service.Status -eq 'Running') {
                    $result = Stop-Service -name $service.Name.replace('[', '?').replace(']', '?')
                    Write-Host ('Stopped Service '+ $service.Name)
                    if($DisableServices){
                        $result = Set-Service -name $service.Name -StartupType 'Disabled'
                        Write-Host ('Disabled Service '+ $service.Name)
                    }
                }
            }
        }
    }
    
    if ($OutputManagementServices -ne $Null -and $OutputManagementServices.Count -gt 0) {
        # Stop all except the Core service first, because all other services depend on it
        foreach ($OutputManagementServiceIdentifier in ($OutputManagementServices.Keys | ? { $_ -ne 'Core' })) {
            $OutputManagementServiceName = $OutputManagementServices[$OutputManagementServiceIdentifier]["Name"]
            if (Get-Service -Name $OutputManagementServiceName -ErrorAction SilentlyContinue) {
                $service = Get-Service -Name $OutputManagementServiceName
                if ($service.Status -eq 'Running') {
                    $result = Stop-Service -name $service.Name.replace('[', '?').replace(']', '?')
                    Write-Host ('Stopped Service '+ $service.Name)
                    if($DisableServices){
                        $result = Set-Service -name $service.Name -StartupType 'Disabled'
                        Write-Host ('Disabled Service '+ $service.Name)
                    }
                }
            }
        }
        
        # Then stop the Core service
        if ($OutputManagementServices.Keys -contains 'Core') {
            $CoreServiceName = $OutputManagementServices["Core"]["Name"]
            if (Get-Service -Name $CoreServiceName -ErrorAction SilentlyContinue) {
                $service = Get-Service -Name $CoreServiceName
                if ($service.Status -eq 'Running') {
                    $result = Stop-Service -name $service.Name.replace('[', '?').replace(']', '?') -force
                    Write-Host ('Stopped Service '+ $service.Name)
                    if($DisableServices){
                        $result = Set-Service -name $service.Name -StartupType 'Disabled'
                        Write-Host ('Disabled Service '+ $service.Name)
                    }
                }
            }
        }
    }
}

Function Start-CCM {
    param (
        [Parameter(Mandatory=$true)][string]$Version,        
        [string[]][AllowEmptyCollection()]$TomcatServices,
        [hashtable][AllowEmptyCollection()]$OutputManagementServices,
        [switch]$EnableServices,
        [switch]$TomcatServicesOnly
    )

    [string]$programsPath = "*\CCM\Programs\$Version\*"
    
    Write-Host "Starting CCM $Version"

    if ($TomcatServicesOnly) {
        Write-Host "Starting Tomcat services only"
    } else {
        # Do not start any Output Management services here, as they require a specific start/stop order and force flag
        $runningServices = Get-WmiObject win32_service | ?{-Not ($_.State -eq 'Running')} | select Name, PathName | ? { -not ($_.PathName -match '[\\/]Output Management[\\/]') }
        if ($runningServices -ne $null -and $runningServices -ne @()) {
            foreach($service in $runningServices) {
                if($service.PathName -like $programsPath) {            
                    if($EnableServices){
                        $result = Set-Service -name $service.Name -StartupType 'Automatic'
                        Write-Host ('Enabled Service '+ $service.Name)
                    }
                    $result = Start-Service -name $service.Name.replace('[', '?').replace(']', '?')
                    Write-Host ('Started Service '+ $service.Name)
                }
            }
        }
    }
    
    if ($TomcatServices -ne $null -and $TomcatServices -ne @()) {
        foreach ($tomcatService in $TomcatServices){
            $service = Get-Service -Name $tomcatService
            if($EnableServices){
                $result = Set-Service -name $service.Name -StartupType 'Automatic'
                Write-Host ('Enabled Service '+ $service.Name)
            }
            if ($service.Status -ne 'Running') {
                $result = Start-Service -name $service.Name.replace('[', '?').replace(']', '?')
                Write-Host ('Started Service '+ $service.Name)
            } else {
                Write-Host ('Service ' + $service.Name + ' was already running')
            }
        }
    }
    
    if ($OutputManagementServices -ne $Null -and $OutputManagementServices.Count -gt 0) {
        # Start only the Core service first, because all other services depend on it
        if ($OutputManagementServices.Keys -contains 'Core') {
            $CoreServiceName = $OutputManagementServices["Core"]["Name"]
            if (Get-Service -Name $CoreServiceName -ErrorAction SilentlyContinue) {
                $service = Get-Service -Name $CoreServiceName
                if($EnableServices){
                    $result = Set-Service -name $service.Name -StartupType 'Automatic'
                    SetServiceDelayedStart -serviceName $service.Name
                    Write-Host ('Enabled Service '+ $service.Name)
                }
                $result = Start-Service -name $service.Name.replace('[', '?').replace(']', '?')
                Write-Host ('Started Service '+ $service.Name)
            }
        }
        
        # Then start the other services
        foreach ($OutputManagementServiceIdentifier in ($OutputManagementServices.Keys | ? { $_ -ne 'Core' })) {
            $OutputManagementServiceName = $OutputManagementServices[$OutputManagementServiceIdentifier]["Name"]
            if (Get-Service -Name $OutputManagementServiceName -ErrorAction SilentlyContinue) {
                $service = Get-Service -Name $OutputManagementServiceName
                if($EnableServices){
                    $result = Set-Service -name $service.Name -StartupType 'Automatic'
                    SetServiceDelayedStart -serviceName $service.Name
                    Write-Host ('Enabled Service '+ $service.Name)
                }
                $result = Start-Service -name $service.Name.replace('[', '?').replace(']', '?')
                Write-Host ('Started Service '+ $service.Name)
            }
        }
    }
}

function Get-Installed-Versions {
    param (
        [Parameter(Mandatory=$true)][string]$packageRegistryURI
    )

    [string[]]$versions = @();

    # Test 4.4 style registry structure
    $versionRegistryNode = (Join-Path $packageRegistryURI 'Version')
    if(Test-Path $versionRegistryNode) {
        $items = Get-ChildItem $versionRegistryNode
        foreach($item in $items) {
            $versions += $item.PSChildName;
        }
    }

    # Test 4.5 style registry structure
    if(Test-Path $packageRegistryURI) {
        $items = Get-ChildItem $packageRegistryURI
        foreach($item in $items) {
            # check whether the node name matches a version number
            if($item.PSChildName -match '^[0-9]+(\.[0-9]+)+$') {
                $versions += $item.PSChildName;
            }
        }
    }

    return $versions;
}

Function SetValidationId {
    [CmdletBinding()]
    param (
        [Parameter(Mandatory=$true)][string]$restApiClient,
        [Parameter(Mandatory=$true)][string]$instanceName,
        [Parameter(Mandatory=$true)][string]$ccmVersion
    )
    
    [string]$validationId = GetValidationId -InstanceName $instanceName -instanceVersion $ccmVersion
    
    if (Test-Path $restApiClient) {
        Write-Host "Setting designer instance validationId"
        $ExitCode = Execute-Command $restApiClient @( `
            '/trusted', `
            '/call', 'SetValidationId', `
            '/validationid', $validationId
        ) -silent
        if ($ExitCode -ne 0) {
           Write-Warning "Designer set validationId: unable to set validationId '$validationId'."
        }
    }
    else {
       Write-Warning "Designer set validationId: unable to set validationId '$validationId'. '$restApiClient' could not be found."
    }
}

Function ParseInstanceNumber {
    [CmdletBinding()]
    param (
        [Parameter(Mandatory=$true)]$instanceNumberString
    )
    
    if ($instanceNumberString.GetType().Name -ne 'String') {
        Throw "Parameter is not parsable"
    }
    
    [int]$instanceNumber = [Int32]::Parse($instanceNumberString)
    
    if (($instanceNumber -le 0) -or ($instanceNumber -ge 100)) {
        Throw "Number is not in range"
    }
    
    return $instanceNumber
}

function CheckDotNetVersion([string]$minVersion)
{
    #https://msdn.microsoft.com/en-us/library/hh925568

    try {
    [int]$releaseKey = (Get-ItemProperty 'HKLM:\SOFTWARE\Microsoft\NET Framework Setup\NDP\v4\Full\').Release
    
    [string]$version = ""
    
    if ($releaseKey -ge 394802) {
       $version =  '4.6.2'
       Write-Host "Found .NET version $version or later"
    }
    elseif (($releaseKey -ge 394254)) {
        $version =  '4.6.1'
        Write-Host "Found .NET version $version"
    }
    elseif (($releaseKey -ge 393295)) {
        $version =  '4.6'
        Write-Host "Found .NET version $version"
    }
    elseif (($releaseKey -ge 379893)) {
        $version =  '4.5.2'
        Write-Host "Found .NET version $version"
    }
    elseif (($releaseKey -ge 378675)) {
        $version =  '4.5.1'
        Write-Host "Found .NET version $version"
    }
    elseif (($releaseKey -ge 378389)) {
        $version =  '4.5'
        Write-Host "Found .NET version $version"
    }
    else {
        Write-Error "CCM requires .NET Framework 4.5.1 or higher. Please install it and run this installer again."
        Exit 1
    }
    
    if([System.Version]$version -lt [System.Version]$minVersion) {
        Write-Error "At least .NET $minVersion is required. Please install it and run this installer again."
        Exit 1
    }
    } catch {
        Write-Error "CCM requires .NET Framework 4.5.1 or higher. Please install it and run this installer again."
        Exit 1

    }
}

Function GetLanguageFile {
    [CmdletBinding()]
    param(
        [Parameter(Mandatory=$true)][string]$fileName,
        [Parameter(Mandatory=$true)][string]$srcLocation,
        [Parameter(Mandatory=$true)][string]$relativePath,
        [Parameter(Mandatory=$true)][string]$resultDir
    )
    
    $fullPath = ""
    if($relativePath -ne '') {
        $fullPath = Join-Path $srcLocation $relativePath
    }
    else {
        $fullPath = $srcLocation
    }
    $fileToCopy = (Join-Path $fullPath $fileName)
    
    if(Test-Path $fileToCopy){
        Write-Host "Retrieve '$fileToCopy'"
        $fullResultDir = (Join-Path $resultDir $relativePath)
        if(-not (Test-Path $fullResultDir)) {
            New-Item -ItemType directory -Path $fullResultDir | Out-Null
        }
        Copy-Item $fileToCopy $fullResultDir
    }
    else {
        Write-Error "Could not find file: '$fileToCopy'"
        Exit 1
    }
}

Function AddLanguageFile {
    [CmdletBinding()]
    param(
        [Parameter(Mandatory=$true)][string]$fileName,
        [Parameter(Mandatory=$true)][string]$srcLocation,
        [Parameter(Mandatory=$true)][string]$relativePath,
        [Parameter(Mandatory=$true)][string]$resultDir,
        [Parameter(Mandatory=$true)][bool]$checkOnly
    )
    
    $fullPath = ""
    if($relativePath -ne '') {
        $fullPath = Join-Path $srcLocation $relativePath
    }
    else {
        $fullPath = $srcLocation
    }
    $fileToCopy = (Join-Path $fullPath $fileName)
    
    if(Test-Path $fileToCopy){
        if (-not $checkOnly) {
            $fullResultDir = (Join-Path $resultDir $relativePath)
            Write-Host "Copy '$fileToCopy' to '$fullResultDir'"
            if(-not (Test-Path $fullResultDir)) {
                Write-Error "Destination directory '$fullResultDir' does not exist"
            }
            Copy-Item $fileToCopy $fullResultDir
        }
    }
    else {
        Write-Error "Could not find file: '$fileToCopy'"
        Exit 1
    }
}

Function RemoveLanguageFile {
    [CmdletBinding()]
    param(
        [Parameter(Mandatory=$true)][string]$fileName,
        [Parameter(Mandatory=$true)][string]$srcLocation,
        [Parameter(Mandatory=$true)][string]$relativePath   
    )
    
    $fullPath = ""
    if($relativePath -ne '') {
        $fullPath = Join-Path $srcLocation $relativePath
    }
    else {
        $fullPath = $srcLocation
    }
    $fileToRemove = (Join-Path $fullPath $fileName)
    
    if(Test-Path $fileToRemove){
            Write-Host "Remove '$fileToRemove'"
            Remove-Item $fileToRemove
    }
}

Function StandardLanguage {
    [CmdletBinding()]
    param(
        [Parameter(Mandatory=$true)][string]$languageCode,
        [Parameter(Mandatory=$false)][switch]$silent = $false
    )
    [string[]]$standardLanguages = @('de', 'en', 'es', 'fr', 'it', 'ja', 'nl', 'pt-br')

    foreach ($language in $standardLanguages) {
        if($language -eq $languageCode) {
            if (-Not $silent) {
			    Write-Warning 'This language pack is part of the standard languages.'
            }
			$true
        }
    }
    
    $false
}

# Check if the destination instance is active
function CheckInstanceStatus {
    [CmdletBinding()]
    param (
        [Parameter(Mandatory=$true)][string]$manageCM,
        [Parameter(Mandatory=$true)][string]$user,
        [Parameter(Mandatory=$true)][string]$password,
        [Parameter(Mandatory=$true)][string]$instanceNumber,
        [Parameter(Mandatory=$true)][string]$compareStatus
    )
    
    if (Test-Path $manageCM) {
        $ExitCode = 0
        
        # Export CM configuration
        [string[]]$stdOutRedirect = ''
        #Get the list of instances and their status
        $ExitCode = Execute-Command $manageCM @( '/listinstance', `
            "/user=$user", `
            "/password=$password", `
            "/outputlist" `
        ) $false ([ref]$stdOutRedirect) -silent

        if ($ExitCode -ne 0) {
            Throw 'Failed to retrieve the list of instances from the Contract Manager.'
        }
        
        if ($stdOutRedirect.length -gt 0) {
            foreach ($line in $stdOutRedirect){
                [string[]]$parts = $line.split("`t")
                if ($parts.length -ge 3){
                    [string]$instance = $parts[1]
                    [string]$status = $parts[2].Trim()
                    if ($instance -eq $instanceNumber){
                        write-host "Found instance $instance = $status."
                        return ($status.ToLower().CompareTo($compareStatus) -eq 0)
                    }
                } 
            }
        }
    }
    else {
        Throw "Management tool $manageCM could not be found."
    }

    return $false
}

Function GetActiveConfiguration {
    Param(
        [Parameter(Mandatory=$true)][ref]$deployIni
    )
        
    # Find existing package deployments in the registry
    if(Test-Path 'HKLM:\SOFTWARE\Wow6432Node') {
        $packageRegistryURI = 'HKLM:\SOFTWARE\Wow6432Node\Aia\Package'
    } else {
        $packageRegistryURI = 'HKLM:\SOFTWARE\Aia\Package'
    }

    [string[]]$installedVersions = Get-Installed-Versions $packageRegistryURI

    if (($installedVersions -eq $null) -or ($installedVersions.length -eq 0)) {
        Write-Error 'No installed CCM version found. Use install script instead of upgrade to install CCM'
        Exit 1
    }

    # Determine the active version
    [string]$activeVersion = ""

    if (Test-Path $packageRegistryURI) {
        $property = Get-ItemProperty -Path $packageRegistryURI -Name 'ActiveVersion' -ErrorAction SilentlyContinue
        if ($property -ne $null) {
            $activeVersion = $property.ActiveVersion
        } elseif (($installedVersions -contains '4.4') -and ($installedVersions.length -eq 1)) {
            $activeVersion = '4.4'
        }
    }

    if (([string]::IsNullOrEmpty($activeVersion)) -or (-not ($installedVersions -contains $activeVersion))) {
        Write-Error 'Cannot determine active CCM installation on this machine'
        Exit 1
    } 

    if($activeVersion -eq '4.4') {
        $deploymentConfigLocation = (Get-ItemProperty -literalPath (Join-Path (Join-Path $packageRegistryURI 'Version') $activeVersion)).DeploymentConfiguration
    } else {
        $deploymentConfigLocation = (Get-ItemProperty -literalPath (Join-Path $packageRegistryURI $activeVersion)).DeploymentConfiguration
    }
    if ([string]::IsNullOrEmpty($deploymentConfigLocation) -or (-not (Test-Path $deploymentConfigLocation))) {
        Write-Error ('Deployment configuration not found at "'+ $deploymentConfigLocation +'"')
        Exit 2
    }

    # read existing configuration parameters from deploy.ini
    if (-not (Get-IniFile $deploymentConfigLocation $deployIni)) {
        Exit 2
    }
}

function CopyComposerUILanguagePack {
    param (
        [Parameter(Mandatory=$true)][string]$inputDir,
        [Parameter(Mandatory=$true)][string]$destPath,
        [Parameter(Mandatory=$false)][switch]$checkOnly = $false
    )
        
    #ComposerUI
    $srcDir = Join-Path $inputDir 'ComposerUI'
    AddLanguageFile "$languageCode.js"      $srcDir '\' $destPath $checkOnly
}

function CopyTinyMCELanguagePack {
    param (
        [Parameter(Mandatory=$true)][string]$inputDir,
        [Parameter(Mandatory=$true)][string]$destPath,
        [Parameter(Mandatory=$false)][switch]$checkOnly = $false
    )
    #TinyMCE standard
    $srcDir = Join-Path $inputDir 'TinyMCEStandard'
    AddLanguageFile "$languageCode.js"      $srcDir 'langs' $destPath $checkOnly
    AddLanguageFile "$languageCode`_dlg.js" $srcDir 'plugins\advhr\langs' $destPath $checkOnly 
    AddLanguageFile "$languageCode`_dlg.js" $srcDir 'plugins\advimage\langs' $destPath $checkOnly
    AddLanguageFile "$languageCode`_dlg.js" $srcDir 'plugins\advlink\langs' $destPath $checkOnly
    AddLanguageFile "$languageCode`_dlg.js" $srcDir 'plugins\emotions\langs' $destPath $checkOnly
    AddLanguageFile "$languageCode`_dlg.js" $srcDir 'plugins\fullpage\langs' $destPath $checkOnly
    AddLanguageFile "$languageCode`_dlg.js" $srcDir 'plugins\media\langs' $destPath $checkOnly
    AddLanguageFile "$languageCode`_dlg.js" $srcDir 'plugins\paste\langs' $destPath $checkOnly
    AddLanguageFile "$languageCode`_dlg.js" $srcDir 'plugins\searchreplace\langs' $destPath $checkOnly
    AddLanguageFile "$languageCode`_dlg.js" $srcDir 'plugins\style\langs' $destPath $checkOnly
    AddLanguageFile "$languageCode`_dlg.js" $srcDir 'plugins\table\langs' $destPath $checkOnly
    AddLanguageFile "$languageCode`_dlg.js" $srcDir 'plugins\template\langs' $destPath $checkOnly
    AddLanguageFile "$languageCode`_dlg.js" $srcDir 'plugins\xhtmlxtras\langs' $destPath $checkOnly
    AddLanguageFile "$languageCode.js"      $srcDir 'themes\advanced\langs' $destPath $checkOnly
    AddLanguageFile "$languageCode`_dlg.js" $srcDir 'themes\advanced\langs' $destPath $checkOnly
    AddLanguageFile "$languageCode.js"      $srcDir 'themes\simple\langs' $destPath $checkOnly
        
    #TinyMCE plugins
    $srcDir = Join-Path $inputDir 'TinyMCEPlugins'
    AddLanguageFile "$languageCode.js"      $srcDir 'plugins\itptextblockeditor\langs' $destPath $checkOnly
    AddLanguageFile "$languageCode`_dlg.js" $srcDir 'plugins\itptextblockeditor\langs' $destPath $checkOnly
    AddLanguageFile "$languageCode.js"      $srcDir 'plugins\itpfieldformat\langs' $destPath $checkOnly
    AddLanguageFile "$languageCode`_dlg.js" $srcDir 'plugins\itpfieldformat\langs' $destPath $checkOnly
    AddLanguageFile "$languageCode.js"      $srcDir 'plugins\itpmetadata\langs' $destPath $checkOnly
    AddLanguageFile "$languageCode`_dlg.js" $srcDir 'plugins\itpmetadata\langs' $destPath $checkOnly
    AddLanguageFile "$languageCode.js"      $srcDir 'plugins\itpvisualchars\langs' $destPath $checkOnly
}

function RemoveComposerUILanguagePack {
    param (
        [Parameter(Mandatory=$true)][string]$targetDir
    )
        
    #ComposerUI
    RemoveLanguageFile "$languageCode.js"      $targetDir '\'
}

function RemoveTinyMCELanguagePack {
    param (
        [Parameter(Mandatory=$true)][string]$targetDir
    )
    
    #TinyMCE standard
    RemoveLanguageFile "$languageCode.js"      $targetDir 'langs'
    RemoveLanguageFile "$languageCode`_dlg.js" $targetDir 'plugins\advhr\langs'
    RemoveLanguageFile "$languageCode`_dlg.js" $targetDir 'plugins\advimage\langs'
    RemoveLanguageFile "$languageCode`_dlg.js" $targetDir 'plugins\advlink\langs'
    RemoveLanguageFile "$languageCode`_dlg.js" $targetDir 'plugins\emotions\langs'
    RemoveLanguageFile "$languageCode`_dlg.js" $targetDir 'plugins\fullpage\langs'
    RemoveLanguageFile "$languageCode`_dlg.js" $targetDir 'plugins\media\langs'
    RemoveLanguageFile "$languageCode`_dlg.js" $targetDir 'plugins\paste\langs'
    RemoveLanguageFile "$languageCode`_dlg.js" $targetDir 'plugins\searchreplace\langs'
    RemoveLanguageFile "$languageCode`_dlg.js" $targetDir 'plugins\style\langs'
    RemoveLanguageFile "$languageCode`_dlg.js" $targetDir 'plugins\table\langs'
    RemoveLanguageFile "$languageCode`_dlg.js" $targetDir 'plugins\template\langs'
    RemoveLanguageFile "$languageCode`_dlg.js" $targetDir 'plugins\xhtmlxtras\langs'
    RemoveLanguageFile "$languageCode.js"      $targetDir 'themes\advanced\langs'
    RemoveLanguageFile "$languageCode`_dlg.js" $targetDir 'themes\advanced\langs'
    RemoveLanguageFile "$languageCode.js"      $targetDir 'themes\simple\langs'
        
    #TinyMCE plugins
    RemoveLanguageFile "$languageCode.js"      $targetDir 'plugins\itptextblockeditor\langs'
    RemoveLanguageFile "$languageCode`_dlg.js" $targetDir 'plugins\itptextblockeditor\langs'
    RemoveLanguageFile "$languageCode.js"      $targetDir 'plugins\itpfieldformat\langs'
    RemoveLanguageFile "$languageCode`_dlg.js" $targetDir 'plugins\itpfieldformat\langs'
    RemoveLanguageFile "$languageCode.js"      $targetDir 'plugins\itpmetadata\langs'
    RemoveLanguageFile "$languageCode`_dlg.js" $targetDir 'plugins\itpmetadata\langs'
    RemoveLanguageFile "$languageCode.js"      $targetDir 'plugins\itpvisualchars\langs'
}

function ListLanguagePacks {
    param (
        [Parameter(Mandatory=$true)][string]$targetDir,
        [Parameter(Mandatory=$false)][string]$destinationFile
    )

    [bool]$writeToFile = -Not [string]::IsNullOrEmpty($destinationFile)
    [string]$header1 = 'Language Pack Language Codes'
    [string]$header2 = '----------------------------'
	Write-Host($header1)
    Write-Host($header2)
    if ($writeToFile) {
        try {
         $header1 | Out-File $destinationFile -Encoding 'UTF8' -append
         $header2 | Out-File $destinationFile -Encoding 'UTF8' -append
        }
        Catch [Exception] {
           Write-Warning "Error while writing results to output file '$destinationFile'"
           $writeToFile = $false
        }
    }
	
    Get-ChildItem -Path $targetDir -Filter *.js -Name| ForEach-Object {
        $language = [System.IO.Path]::GetFileNameWithoutExtension($_)
        if (StandardLanguage $language -silent) {
             $language += '	(standard language)'
        }

        Write-Host($language)
        if ($writeToFile)
        {
             try {
                 $language | Out-File $destinationFile -Encoding 'UTF8' -append
             }
             Catch [Exception] {
                 Write-Warning "Error while writing results to output file '$destinationFile'"
                 $writeToFile = $false
            }
        }
    }	
}

function Get-StrictModeVersion {
    # Gets PowerShell strict validation version
    # Does not use the latest version, to avoid deployment breaking completely if a hitherto unknown version is released
    Write-Output 2.0
}

function InstallPFXCertificate {
    # Installs a .pfx certificate in the given store, and grants the given user access to the private key, if needed.
    # Based off https://stackoverflow.com/a/31175117
    # Note that attempting to set private key access rights on any store except the Personal may result in null-reference
    # exceptions in the Windows Cert Manager, amongst other problems.
    
    param (
        [Parameter(Mandatory=$true)][System.Security.Cryptography.X509Certificates.X509Certificate2]$cert,
        [Parameter(Mandatory=$true)][string]$storeName,
        [Parameter(Mandatory=$true)][string]$storeLocation,
        [Parameter(Mandatory=$false)][string]$user = ""
    )
    
    # Normalize the user, if needed. The API cannot resolve '.\' as the local domain.
    $NormalizedUser = $user
    if ($user.StartsWith(".\")) {
        $NormalizedUser = $user.SubString(2)
    }
    
    # Open the certificate store, and add the cert.
    $Store = New-Object System.Security.Cryptography.X509Certificates.X509Store($storeName, $storeLocation)
    $Store.Open("ReadWrite")
    $Store.Add($cert)
    
    # If the user is set, add private key rights.
    if ($NormalizedUser -ne "") {
        # Create a new CSP object based on the existing certificate provider and key name.
        $CSP = New-Object System.Security.Cryptography.CspParameters($cert.PrivateKey.CspKeyContainerInfo.ProviderType, $cert.PrivateKey.CspKeyContainerInfo.ProviderName, $cert.PrivateKey.CspKeyContainerInfo.KeyContainerName)

        # Set the flags and key security based on the existing cert.
        $CSP.Flags = "UseExistingKey","UseMachineKeyStore"
        $CSP.CryptoKeySecurity = $cert.PrivateKey.CspKeyContainerInfo.CryptoKeySecurity
        $CSP.KeyNumber = $cert.PrivateKey.CspKeyContainerInfo.KeyNumber

        # Create the new access rule.
        $Access = New-Object System.Security.AccessControl.CryptoKeyAccessRule($NormalizedUser, "FullControl", "Allow")
        
        # Add the access rule to the CSP object.
        $CSP.CryptoKeySecurity.AddAccessRule($Access)

        # Create a new CryptoServiceProvider object, which updates the key with the CSP information created above.
        $RSA2 = New-Object System.Security.Cryptography.RSACryptoServiceProvider($CSP)
    }
    
    # Close the store.
    $Store.Close()
}

function UninstallPFXCertificate {
    # Uninstalls a .pfx certificate from the given store.
    # Based off https://stackoverflow.com/a/31175117
    
    param (
        [Parameter(Mandatory=$true)][string]$certFile,
        [Parameter(Mandatory=$true)][string]$password,
        [Parameter(Mandatory=$true)][string]$storeName,
        [Parameter(Mandatory=$true)][string]$storeLocation
    )
    
    # Load the certificate file.
    $Cert = New-Object System.Security.Cryptography.X509Certificates.X509Certificate2
    $Cert.Import($certFile, $password, 'MachineKeySet')
    
    # Open the certificate store, remove the cert, and close the store again.
    $Store = New-Object System.Security.Cryptography.X509Certificates.X509Store($storeName, $storeLocation)
    $Store.Open("ReadWrite")
    $Store.Remove($Cert)
    $Store.Close()
}

function AddOutputManagementRepositoryDatabase {
    # Creates the tables/schema/etc for an Output Management Repository database, and configures the appropriate alias.
    # Assumes the database exists, and that the user has appropriate rights (does not re-check).
    
    param (
        [Parameter(Mandatory=$true)][string]$databaseType,
        [Parameter(Mandatory=$true)][string]$connectionString,
        [Parameter(Mandatory=$true)][string]$database,
        [Parameter(Mandatory=$true)][string]$user,
        [Parameter(Mandatory=$true)][string]$password,
        [Parameter(Mandatory=$true)][string]$outputManagementBin,
        [Parameter(Mandatory=$true)][string]$aliasName
    )
    
    # Find the repository.config file.
    $RepositoryConfigFile = Join-Path $outputManagementBin "repository.config"
    if (-not (Test-Path $RepositoryConfigFile)) {
        Throw "Could not find 'repository.config' file to update."
    }
    
    # Make sure the alias name does not exist yet.
    $Found = $False
    Add-Type -AssemblyName System.Xml.Linq
    $XDoc = [Xml.Linq.XDocument]::Load($RepositoryConfigFile)
    $XRoot = $XDoc.Root
    $XSpace = $XRoot.Name.Namespace
    $XAliasList = $XRoot.Element($XSpace + "aliaslist")
    $XAliases = [Xml.Linq.XElement[]]$XAliasList.Elements()
    ForEach ($XAlias in $XAliases) {
        $XNameAttr = $XAlias.Attribute("name")
        if ($XNameAttr -ne $Null -and $XNameAttr.Value -eq $aliasName) {
            $Found = $True
        }
    }
    
    if ($Found) {
        Throw "Requested Output Management Repository alias $aliasName already exists."
    }
    
    # Get the encoded password string.
    $EncodedPassword = EncodeOutputManagementPassword $password $outputManagementBin
    
    # Prepare some values for the configuration and connection.
    $DBTypeValue = ""
    $OdbcConnectionString = New-Object Data.Odbc.OdbcConnectionStringBuilder $connectionString
    $OdbcConnectionString.Add("User ID", $user)
    
    if ($databaseType -eq 'sqlserver') {
        $OdbcConnectionString.Add("Initial Catalog", $database)
        $DBTypeValue = "SQLMS"
    } elseif ($databaseType -eq 'oracle') {
        $DBTypeValue = "ORAMS"
    } else {
        Throw "Output Management Repository database type $databaseType is not supported."
    }
    
    $DBConnectionStringValue = $OdbcConnectionString.ConnectionString
    
    # Fill the database.
    [string]$DatabaseTool = Join-Path $outputManagementBin "DatabaseCreate.exe"
    [ref]$StdOut = @()
    $exitCode = Execute-Command $DatabaseTool @( `
        ('/connectionstring=' + $DBConnectionStringValue), `
        ('/databasetype=' + $databaseType), `
        '/repository', `
        ('/password=' + $password) `
        ) -silent -stdout ([ref]$StdOut)

    if ($exitCode -ne 0) {
        $StdOut.Value | Write-Host
        Throw "Failed to create the Output Management Repository database."
    }

    # Configure the alias in the repository.config XML file.
    $XNewAlias = New-Object Xml.Linq.XElement ($XSpace + "alias")
    $XNewAliasNameAttr = New-Object Xml.Linq.XAttribute "name",$aliasName
    $XNewAlias.Add($XNewAliasNameAttr)
    $XNewAliasDBTypeAttr = New-Object Xml.Linq.XAttribute "dbtype",$DBTypeValue
    $XNewAlias.Add($XNewAliasDBTypeAttr)
    $XNewAliasDBConnectionStringAttr = New-Object Xml.Linq.XAttribute "connectionstring",$DBConnectionStringValue
    $XNewAlias.Add($XNewAliasDBConnectionStringAttr)
    $XNewAliasPasswordAttr = New-Object Xml.Linq.XAttribute "code",$EncodedPassword
    $XNewAlias.Add($XNewAliasPasswordAttr)
    $XAliasList.Add($XNewAlias)
    $XDoc.Save($RepositoryConfigFile)
}

function AddOutputManagementRuntimeDatabase {
    # Creates the tables/schema/etc for an Output Management Runtime database, and configures the appropriate alias.
    # Assumes the database exists, and that the user has appropriate rights (does not re-check).
    # Does not create the alias.
    
    param (
        [Parameter(Mandatory=$true)][string]$databaseType,
        [Parameter(Mandatory=$true)][string]$connectionString,
        [Parameter(Mandatory=$true)][string]$database,
        [Parameter(Mandatory=$true)][string]$user,
        [Parameter(Mandatory=$true)][string]$password,
        [Parameter(Mandatory=$true)][string]$outputManagementBin,
        [Parameter(Mandatory=$true)][string]$aliasID,
        [Parameter(Mandatory=$true)][string]$systemID
    )
    
    # Get the encoded password string.
    $EncodedPassword = EncodeOutputManagementPassword $password $outputManagementBin
    
    # Get the default SQL template queries from dbalias.config.
    $DBAliasFile = Join-Path $outputManagementBin "dbalias.config"
    Add-Type -AssemblyName System.Xml.Linq
    $XDoc = [Xml.Linq.XDocument]::Load($DBAliasFile)
    $XRoot = $XDoc.Root
    $XSpace = $XRoot.Name.Namespace
    $XNativeElement = $XRoot.Element($XSpace + "native$databaseType")
    if ($XNativeElement -eq $Null) {
        Throw "Failed to retrieve default template queries from dbalias.config for database type $databaseType"
    }
    
    $XTemplateAttribute = $XNativeElement.Attribute("sqltemplate")
    $XFieldTemplateAttribute = $XNativeElement.Attribute("sqlfieldtemplate")
    if ($XTemplateAttribute -eq $Null -or $XFieldTemplateAttribute -eq $Null) {
        Throw "Failed to retrieve default template query attribute values from dbalias.config for database type $databaseType"
    }
    
    $SQLTemplate = $XTemplateAttribute.Value
    $SQLFieldTemplate = $XFieldTemplateAttribute.Value
    
    # Prepare some values for the configuration and connection.
    $DBTypeValue = 0
    $OdbcConnectionString = New-Object Data.Odbc.OdbcConnectionStringBuilder $connectionString
    $OdbcConnectionString.Add("User ID", $user)
    
    if ($databaseType -eq 'sqlserver') {
        $OdbcConnectionString.Add("Initial Catalog", $database)
        $DBTypeValue = 1
    } elseif ($databaseType -eq 'oracle') {
        $DBTypeValue = 2
    } else {
        Throw "Output Management Runtime database type $databaseType is not supported."
    }
    
    $DBConnectionStringValue = $OdbcConnectionString.ConnectionString
    
    # Fill the database.
    [string]$DatabaseTool = Join-Path $outputManagementBin "DatabaseCreate.exe"
    [ref]$StdOut = @()
    $exitCode = Execute-Command $DatabaseTool @( `
        ('/connectionstring=' + $DBConnectionStringValue), `
        ('/databasetype=' + $databaseType), `
        '/runtime', `
        ('/password=' + $password) `
        ) -silent -stdout ([ref]$StdOut)

    if ($exitCode -ne 0) {
        $StdOut.Value | Write-Host
        Throw "Failed to create the Output Management Runtime database."
    }
    
    # Configure the database alias.
    $ObjectManager = Join-Path $outputManagementBin 'RepositoryObject.exe'
    
    [ref]$StdOut = @()
    $exitCode = Execute-Command $ObjectManager @( `
        '/update', `
        ('/id=' + $aliasID), `
        ('/system=' + $systemID), `
        ('/property:dbtype=' + $DBTypeValue), `
        ('/property:encryptedpassword=' + $EncodedPassword), `
        ('/property:connectionstring=' + $DBConnectionStringValue), `
        ('/property:sqltemplate=' + $SQLTemplate), `
        ('/property:sqlfieldtemplate=' + $SQLFieldTemplate) `
        ) -silent -stdout ([ref]$StdOut)
    
    if ($exitCode -ne 0) {
        $StdOut.Value | Write-Host
        Throw "Failed to configure the Output Management Runtime database alias '$($aliasID)'."
    }
}

function AddOutputManagementSystem {
    # Creates a new Output Management system.
    
    param (
        [Parameter(Mandatory=$true)][string]$id,
        [Parameter(Mandatory=$true)][string]$name,
        [Parameter(Mandatory=$true)][string]$repositoryAlias,
        [Parameter(Mandatory=$true)][string]$outputManagementBin
    )
    
    $ObjectManager = Join-Path $outputManagementBin "RepositoryObject.exe"
    
    [ref]$StdOut = @()
    $exitCode = Execute-Command $ObjectManager @( `
        '/create', `
        ('/name=' + $name), `
        ('/repalias=' + $repositoryAlias), `
        ('/system=' + $id) `
        ) -stdout ([ref]$StdOut)
    
    if ($exitCode -ne 0) {
        $StdOut.Value | Write-Host
        Throw "Failed to create the Output Management system '$($id)'."
    }
}

function EncodeOutputManagementPassword {
    # Encrypts a password (or other string) using the Output Management tools

    param (
        [Parameter(Mandatory=$true)][string]$password,
        [Parameter(Mandatory=$true)][string]$outputManagementBin
    )
    
    # Get the encoded password string.
    $EncoderCLI = Join-Path $outputManagementBin "EncoderCLI.exe"
    $EncodedPassword = ""
    [ref]$StdOut = @()
    $exitCode = Execute-Command $EncoderCLI @("/value=$password") -silent -stdout ([ref]$StdOut)
            
    if ($exitCode -ne 0) {
        $StdOut.Value | Write-Host
        Throw "Failed to encode the password."
    }
    
    if ($StdOut.Value -ne $Null -and $StdOut.Value.GetType().IsArray -and $StdOut.Value.Length -gt 0) {
        $EncodedPassword = $StdOut.Value[0]
    } elseif ($StdOut.Value -ne $Null -and $StdOut.Value.GetType().Name -eq "String") {
        $EncodedPassword = $StdOut.Value
    } else {
        Throw "Failed to retrieve encoded password from encoder output."
    }
    
    Write-Output $EncodedPassword
}

function SetServiceDelayedStart {
    # Sets a service to the Delayed Automatic Start startup type

    param (
        [Parameter(Mandatory=$true)][string]$serviceName
    )
    
    [ref]$StdOut = @()
    $exitCode = Execute-Command "sc.exe" @( `
        'config', `
        $serviceName, `
        'start=', `
        'delayed-auto' `
        ) -stdout ([ref]$StdOut)
    
    if ($exitCode -ne 0) {
        $StdOut.Value | Write-Host
        Throw "Failed to set service '$($serviceName)' to Automatic (Delayed Start)."
    }
}

function FindLegacyOutputManagement {
    <# Locates and collects information about a legacy (5.0 or 5.1) Output Management installation, from before
       it was integrated into the package.
       
       Will retrieve all information possibly required for an upgrade or re-activation.
       
       Collects the following information, in a hash table:
       - 'Install' - boolean, whether or not there is a legacy Output Management installation.
       - 'InstallType' - string, one of 'Full', 'StudioOnly' and 'ServerOnly'.
       - 'InstallDir' - string, location.
       - 'InstallDirEnvVar' - string, name of environment variable for InstallDir.
       - 'Services' - hash table, identifier (includes 'Core' and 'Output' predefined services) to another hash table:
         - 'Name' - string, name of the service.
         - 'Executable' - string, executable of the service.
         (Note: service identifier is assumed to be the name of the executable, minus the '.exe'.)
       - 'Repositories' - hash table, name to another hash table:
         - 'Type' - string, type of the database; either 'sqlserver' or 'oracle'.
         - 'ConnectionString' - string, connection string of the database.
    #>
    
    # Prep the return values
    [bool]$Install = $False
    [string]$InstallType = ''
    [string]$InstallDir = ''
    [string]$InstallDirEnvVar = 'CcmEpDir' # Constant across 5.0 and 5.1
    [hashtable]$Services = @{}
    [hashtable]$Repositories = @{}
    
    # Determine whether or not legacy Output Management is installed from its environment variable
    $EnvValue = [Environment]::GetEnvironmentVariable($InstallDirEnvVar, "Machine")
    if ($EnvValue -ne $Null) {
        # A legacy Output Management installation exists
        $Install = $True
        $InstallDir = $EnvValue
        
        # Cut off the trailing backslash; it's not relevant for the package
        if ($InstallDir.EndsWith("\")) {
            $InstallDir = $InstallDir.SubString(0, $InstallDir.Length - 1)
        }
        
        if (-not (Test-Path $InstallDir)) {
            Throw "Could not find CCM Output Management; expected at '$InstallDir'."
        }
        
        # Determine its installation type by looking for the Studio and Core Service executables
        if (-not (Test-Path (Join-Path $InstallDir "CCM_EP_Core.exe"))) {
            $InstallType = 'StudioOnly'
        } elseif (-not (Test-Path (Join-Path $InstallDir "CCMe.Studio.exe"))) {
            $InstallType = 'ServerOnly'
        } else {
            $InstallType = 'Full'
        }
        
        # Determine all the services belonging to the installation, if needed
        if ($InstallType -ne 'StudioOnly') {
            # Get all executable paths of all services
            $ExistingServices = @{}
            Get-WmiObject Win32_Service | ? { $_.PathName -ne $Null } | % { $ExistingServices[$_.Name] = $_.PathName }
            
            # Use resolved location comparison to determine whether the executable of a service is located in the Output Management folder
            # This doesn't work across network locations, symbolic links or other types of junction, but those scenarios are not creatable by the user anyway
            $InstallDirResolved = (Get-Item $InstallDir).FullName
            foreach ($ServiceName in $ExistingServices.Keys) {
                # Chop off trailing and leading quotation marks; the services we're interested in have no arguments anyway
                $ServicePath = $ExistingServices[$ServiceName]
                if ($ServicePath.StartsWith('"')) {
                    $ServicePath = $ServicePath.SubString(1, $ServicePath.Length - 2)
                }
                
                # Use a try/catch as a guard against invalid path characters, which Test-Path will error on
                $ValidPath = $True
                try {
                    Test-Path $ServicePath -ErrorAction Stop | Out-Null
                } catch {
                    $ValidPath = $False
                }
                
                if ($ValidPath) {
                    $ServiceDirectory = [IO.Path]::GetDirectoryName($ServicePath)
                    if ((Get-Item $ServiceDirectory).FullName -eq $InstallDirResolved) {
                        # This appears to be an Output Management service; store it
                        $Identifier = ''
                        if ($ServiceName -eq "CCM_EP_Core_SVC") {
                            $Identifier = 'Core'
                        } elseif ($ServiceName -eq "CCM_EP_Output_SVC") {
                            $Identifier = 'Output'
                        } else {
                            $Identifier = [IO.Path]::GetFilenameWithoutExtension($ServicePath)
                            
                            # Perform some normalization on the identifier, to make sure it's usable in a newer version as well
                            $NewIdentifier = ''
                            for ($Index = 0; $Index -lt $Identifier.Length -and $Index -lt 100; $Index++) {
                                if (-not ([Char]::IsControl($Identifier, $Index))) {
                                    $NewIdentifier += $Identifier[$Index]
                                }
                            }
                            $Identifier = $NewIdentifier
                            
                            # Make sure the identifier is unique, in case the old version had a custom service called 'Core.exe' or 'Output.exe', or a truncate made it non-unique
                            $Sequence = 2
                            while ((@('Core', 'Output') -contains $Identifier) -or ($Services.Keys -contains $Identifier) -or ($Identifier -eq '')) {
                                $Identifier = $Identifier + $Sequence++
                            }
                        }
                        
                        $Services[$Identifier] = @{'Name' = $ServiceName; 'Executable' = ([IO.Path]::GetFilename($ServicePath))}
                    }
                }
            }
            
            # A Core service is absolutely required; it it wasn't found, error out
            if (-not ($Services.Keys -contains 'Core')) {
                Throw "Could not find the Output Management Core service."
            }
        }
        
        # Determine all the Repository aliases belonging to the installation
        $RepositoryConfigFile = Join-Path $InstallDir "repository.config"
        if (-not (Test-Path $RepositoryConfigFile)) {
            Throw "Could not find 'repository.config' file at '$RepositoryConfigFile'."
        }

        Add-Type -AssemblyName System.Xml.Linq
        $XDoc = [Xml.Linq.XDocument]::Load($RepositoryConfigFile)
        $XRoot = $XDoc.Root
        $XSpace = $XRoot.Name.Namespace
        $XAliasList = $XRoot.Element($XSpace + "aliaslist")
        $XAliases = [Xml.Linq.XElement[]]$XAliasList.Elements()
        ForEach ($XAlias in $XAliases) {
            $XNameAttr = $XAlias.Attribute("name")
            if ($XNameAttr -eq $Null) {
                Throw 'Invalid repository alias configuration; missing name attribute.'
            }
            
            $XTypeAttr = $XAlias.Attribute("dbtype")
            if ($XTypeAttr -eq $Null) {
                Throw 'Invalid repository alias configuration; missing dbtype attribute.'
            }
            
            $XConnectionStringAttr = $XAlias.Attribute("connectionstring")
            if ($XConnectionStringAttr -eq $Null) {
                Throw 'Invalid repository alias configuration; missing connectionstring attribute.'
            }
            
            [string]$AliasName = $XNameAttr.Value
            [string]$AliasType = $XTypeAttr.Value
            [string]$AliasConnectionString = $XConnectionStringAttr.Value
            
            if ($AliasType -eq 'SQLMS') {
                $AliasType = 'sqlserver'
            } elseif ($AliasType -eq 'ORAMS') {
                $AliasType = 'oracle'
            } else {
                Throw "Unknown repository alias type '$AliasType'."
            }
            
            $Repositories[$AliasName] = @{'Type' = $AliasType; 'ConnectionString' = $AliasConnectionString}
        }
    }
    
    # Construct and return the results
    Write-Output @{'Install' = $Install; 'InstallType' = $InstallType; 'InstallDir' = $InstallDir; 'InstallDirEnvVar' = $InstallDirEnvVar; 'Services' = $Services; 'Repositories' = $Repositories}
}

function GetUniqueOutputManagementSystemID {
    # Finds a unique Output Management System ID from the given binary directory. Reads the repository.config
    # file to determine a unique value from the given initial.
    
    param (
        [Parameter(Mandatory=$true)][string]$initialValue,
        [Parameter(Mandatory=$true)][string]$outputManagementBin
    )
    
    # Read all current system IDs from the repository.config file
    [string]$RepositoryConfig = Join-Path $outputManagementBin 'repository.config'
    if (-not (Test-Path $RepositoryConfig)) {
        Throw "Could not find config file '$RepositoryConfig'."
    }
    
    [string[]]$ExistingIDs = @()
    
    Add-Type -AssemblyName System.Xml.Linq
    $XDoc = [Xml.Linq.XDocument]::Load($RepositoryConfig)
    $XRoot = $XDoc.Root
    $XSpace = $XRoot.Name.Namespace
    $XSystemList = $XRoot.Element($XSpace + "systemlist")
    $XSystems = [Xml.Linq.XElement[]]$XSystemList.Elements()
    ForEach ($XSystem in $XSystems) {
        $XOIDAttr = $XSystem.Attribute("oid")
        if ($XOIDAttr -eq $Null) {
            Throw 'Invalid system configuration; missing oid attribute.'
        }
        
        $ExistingIDs += $XOIDAttr.Value
    }
    
    # Find the next available ID from the initial value
    $NewSystemID = $initialValue
    $Sequence = 2
    while ($ExistingIDs -contains $NewSystemID) {
        $NewSystemID = $initialValue + $Sequence++
    }
    
    Write-Output $NewSystemID
}

function GetOutputManagementExportedNames {
    # Finds all the names of the objects in a given Output Management export content.xml file.
    
    param (
        [Parameter(Mandatory=$true)][string]$contentFile
    )
    
    if (-not (Test-Path $contentFile)) {
        Throw "Target file '$contentFile' does not exist."
    }
    
    [string[]]$Names = @()
    
    Add-Type -AssemblyName System.Xml.Linq
    $XDoc = [Xml.Linq.XDocument]::Load($contentFile)
    $XRoot = $XDoc.Root
    $XSpace = $XRoot.Name.Namespace
    $XItems = [Xml.Linq.XElement[]]$XRoot.Elements()
    ForEach ($XITem in $XItems) {
        $XNameAttr = $XItem.Attribute("name")
        if ($XNameAttr -eq $Null) {
            Throw 'Invalid content.xml file; missing name attribute.'
        }
        
        $Names += $XNameAttr.Value
    }
    
    Write-Output $Names
}

function FindPackageConfiguration {
    # Finds the location of the currently active package configuration file. Will throw an error
    # if no CCM versions are installed or active, or if the file does not exist.
    
    [string]$packageRegistryURI = ''
    if(Test-Path 'HKLM:\SOFTWARE\Wow6432Node') {
        $packageRegistryURI = 'HKLM:\SOFTWARE\Wow6432Node\Aia\Package'
    } else {
        $packageRegistryURI = 'HKLM:\SOFTWARE\Aia\Package'
    }

    [string[]]$installedVersions = Get-Installed-Versions $packageRegistryURI
    if (($installedVersions -eq $null) -or ($installedVersions.length -eq 0)) {
        Throw 'No installed CCM versions found on this machine.'
    }

    [string]$activeVersion = ''
    if (Test-Path $packageRegistryURI) {
        $property = Get-ItemProperty -Path $packageRegistryURI -Name 'ActiveVersion' -ErrorAction SilentlyContinue
        if ($property -ne $null) {
            $activeVersion = $property.ActiveVersion
        } elseif (($installedVersions -contains '4.4') -and ($installedVersions.length -eq 1)) {
            $activeVersion = '4.4'
        }
    }

    if([string]::IsNullOrEmpty($activeVersion)) {
        Throw 'No active CCM installation found on this machine.'
    }

    $packageRegistryURI = Join-Path $packageRegistryURI $activeVersion
    [string]$deploymentConfigLocation = (Get-ItemProperty -literalPath $packageRegistryURI).DeploymentConfiguration
    
    if (-not (Test-Path $deploymentConfigLocation)) {
        Throw ('Deployment configuration not found at "'+ $deploymentConfigLocation +'"')
    }
    
    Write-Output $deploymentConfigLocation
}

function EnableLegacyOutputManagementExport {
    # Copies and configures the ContentExport tool into a given 5.0/5.1 Output Management directory, and configures it.
    # This function must be executed from the package directory.
    # Returns the path to the finished ContentExport tool.
    
    param (
        [Parameter(Mandatory=$true)][string]$BinDir,
        [Parameter(Mandatory=$true)][string]$BinDirEnvVar
    )

    # Get the Content Export tool from the Package
    [string]$SourceContentExport = Join-Path (Get-Location).Path 'Software\Output Management\ContentExport.exe'
    [string]$SourceContentExportConfig = $SourceContentExport + '.config'
    
    if (-not (Test-Path $SourceContentExport) -or -not (Test-Path $SourceContentExportConfig)) {
        Throw "Could not find source ContentExport files."
    }
    
    # Ensure the destination name is unique, in case there exists a custom service host called 'ContentExport.exe'
    [string]$DestinationName = 'ContentExport.exe'
    while ((Test-Path (Join-Path $BinDir $DestinationName)) -or (Test-Path (Join-Path $BinDir ($DestinationName + '.config')))) {
        $DestinationName = [Guid]::NewGuid().ToString() + '.exe'
    }
    
    $DestinationExe = Join-Path $BinDir $DestinationName
    $DestinationConfig = $DestinationExe + '.config'
    
    # Set the log folder of the temporary executable to the package directory
    $LogFolder = (Get-Location).Path
    
    # Copy the executable and its .config file into the binary folder, and configure it
    Copy-Item $SourceContentExport $DestinationExe
    Copy-Item $SourceContentExportConfig $DestinationConfig
    
    $result = (Get-Content $DestinationConfig) | Foreach-Object {
        $_  -replace '%EnvironmentVariable%', $BinDirEnvVar `
            -replace '%LogFolder%', $LogFolder `
    } | Set-Content $DestinationConfig -Encoding UTF8
    Show-Result $result
    
    Write-Output $DestinationExe
}

function IsServiceMarkedForDeletion {
    # Checks the Windows registry to see if the given service is marked for deletion.

    param (
        [Parameter(Mandatory=$true)][string]$ServiceName
    )
    
    $Result = $False
    $Property = Get-ItemProperty -Path (Join-Path 'HKLM:\SYSTEM\CurrentControlSet\Services' $ServiceName) -Name 'DeleteFlag' -ErrorAction SilentlyContinue
    if ($Property -ne $Null -and $Property.DeleteFlag -eq 1) {
        $Result = $True
    }
    
    Write-Output $Result
}

function StopMarkedForDeletionService {
    # Stops the given service in a marked-for-deletion-safe way. Assumes that the service exists.
    
    param (
        [Parameter(Mandatory=$true)][string]$ServiceName,
        [Parameter(Mandatory=$false)][int]$WaitIntervalSeconds = 2,
        [Parameter(Mandatory=$false)][int]$WarningSeconds = 15
    )
    
    $Service = Get-Service $ServiceName
    If ($Service.Status -eq 'Running') {
        $Service.Stop()
        $TimeSeconds = 0
        Do {
            Start-Sleep -Seconds $WaitIntervalSeconds
            $TimeSeconds += $WaitIntervalSeconds
            If ($TimeSeconds -gt $WarningSeconds) {
                Write-Warning "Waiting for marked-for-deletion service '$ServiceName' to finish stopping..."
            }
            
            $Service = Get-Service $ServiceName -ErrorAction SilentlyContinue
        } While ($Service -ne $Null -or $Service.Status -ne 'Stopped')
    }
}

function StopAndFilterMarkedForDeletionServices {
    # Finds, stops and filters marked-for-deletion services from the given hashtable.
    
    param (
        [Parameter(Mandatory=$true)][Hashtable]$Services
    )
    
    $KeysToRemove = @()
    ForEach ($ServiceIdentifier in $Services.Keys) {
        $ServiceName = $Services[$ServiceIdentifier]["Name"]
        If (IsServiceMarkedForDeletion $ServiceName) {
            Write-Host "Stopping marked-for-deletion service '$ServiceName'"
            StopMarkedForDeletionService $ServiceName
            $KeysToRemove += $ServiceIdentifier
        }
    }
    
    ForEach ($Key in $KeysToRemove) {
        $Services.Remove($Key)
    }
}

function FindCoreDPServices {
    # Finds all Core DP services with the given prefix. Extracts the numbers, then returns them as a map from the DP number to the service name, i.e:
    # @{ 1 = "ITPDP [core_01_5.1.1] #1"; 3 = "ITPDP [core_01_5.1.1] #3" }
    # Ignores marked-for-deletion services if requested. This may occur after removing DPs through the Core Administrator, when Windows has trouble catching up.
    
    param (
        [Parameter(Mandatory=$true)][string]$ServiceNamePrefix,
        [Parameter(Mandatory=$false)][bool]$SkipMarkedForDeletion = $False
    )
    
    # Loop over all service names
    [Hashtable]$Result = @{}
    ForEach ($ServiceName in (Get-Service | % { $_.Name })) {
        # If it starts with the right prefix, find the trailing '#N' immediately, and extract N
        If ($ServiceName.StartsWith($ServiceNamePrefix) -and $ServiceName -match '^.+#(\d+)$') {
            [int]$DPNumber = $Matches[1]
        
            # Check whether it's marked for deletion, if necessary
            If (-not $SkipMarkedForDeletion -or -not (IsServiceMarkedForDeletion ($ServiceName -replace '[\[\]]','?'))) {
                $Result[$DPNumber] = $ServiceName
            }
        }
    }
    
    Return $Result
}
# SIG # Begin signature block
# MIIbewYJKoZIhvcNAQcCoIIbbDCCG2gCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUO+ySVgG/V9J3pJHaRAFkCciy
# HSCgghauMIIDtzCCAp+gAwIBAgIQDOfg5RfYRv6P5WD8G/AwOTANBgkqhkiG9w0B
# AQUFADBlMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2VydCBBc3N1cmVk
# IElEIFJvb3QgQ0EwHhcNMDYxMTEwMDAwMDAwWhcNMzExMTEwMDAwMDAwWjBlMQsw
# CQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cu
# ZGlnaWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2VydCBBc3N1cmVkIElEIFJvb3Qg
# Q0EwggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQCtDhXO5EOAXLGH87dg
# +XESpa7cJpSIqvTO9SA5KFhgDPiA2qkVlTJhPLWxKISKityfCgyDF3qPkKyK53lT
# XDGEKvYPmDI2dsze3Tyoou9q+yHyUmHfnyDXH+Kx2f4YZNISW1/5WBg1vEfNoTb5
# a3/UsDg+wRvDjDPZ2C8Y/igPs6eD1sNuRMBhNZYW/lmci3Zt1/GiSw0r/wty2p5g
# 0I6QNcZ4VYcgoc/lbQrISXwxmDNsIumH0DJaoroTghHtORedmTpyoeb6pNnVFzF1
# roV9Iq4/AUaG9ih5yLHa5FcXxH4cDrC0kqZWs72yl+2qp/C3xag/lRbQ/6GW6whf
# GHdPAgMBAAGjYzBhMA4GA1UdDwEB/wQEAwIBhjAPBgNVHRMBAf8EBTADAQH/MB0G
# A1UdDgQWBBRF66Kv9JLLgjEtUYunpyGd823IDzAfBgNVHSMEGDAWgBRF66Kv9JLL
# gjEtUYunpyGd823IDzANBgkqhkiG9w0BAQUFAAOCAQEAog683+Lt8ONyc3pklL/3
# cmbYMuRCdWKuh+vy1dneVrOfzM4UKLkNl2BcEkxY5NM9g0lFWJc1aRqoR+pWxnmr
# EthngYTffwk8lOa4JiwgvT2zKIn3X/8i4peEH+ll74fg38FnSbNd67IJKusm7Xi+
# fT8r87cmNW1fiQG2SVufAQWbqz0lwcy2f8Lxb4bG+mRo64EtlOtCt/qMHt1i8b5Q
# Z7dsvfPxH2sMNgcWfzd8qVttevESRmCD1ycEvkvOl77DZypoEd+A5wwzZr8TDRRu
# 838fYxAe+o0bJW1sj6W3YQGx0qMmoRBxna3iw/nDmVG3KwcIzi7mULKn+gpFL6Lw
# 8jCCA+4wggNXoAMCAQICEH6T6/t8xk5Z6kuad9QG/DswDQYJKoZIhvcNAQEFBQAw
# gYsxCzAJBgNVBAYTAlpBMRUwEwYDVQQIEwxXZXN0ZXJuIENhcGUxFDASBgNVBAcT
# C0R1cmJhbnZpbGxlMQ8wDQYDVQQKEwZUaGF3dGUxHTAbBgNVBAsTFFRoYXd0ZSBD
# ZXJ0aWZpY2F0aW9uMR8wHQYDVQQDExZUaGF3dGUgVGltZXN0YW1waW5nIENBMB4X
# DTEyMTIyMTAwMDAwMFoXDTIwMTIzMDIzNTk1OVowXjELMAkGA1UEBhMCVVMxHTAb
# BgNVBAoTFFN5bWFudGVjIENvcnBvcmF0aW9uMTAwLgYDVQQDEydTeW1hbnRlYyBU
# aW1lIFN0YW1waW5nIFNlcnZpY2VzIENBIC0gRzIwggEiMA0GCSqGSIb3DQEBAQUA
# A4IBDwAwggEKAoIBAQCxrLNJVEuXHBIK2CV5kSJXKm/cuCbEQ3Nrwr8uUFr7FMJ2
# jkMBJUO0oeJF9Oi3e8N0zCLXtJQAAvdN7b+0t0Qka81fRTvRRM5DEnMXgotptCvL
# mR6schsmTXEfsTHd+1FhAlOmqvVJLAV4RaUvic7nmef+jOJXPz3GktxK+Hsz5HkK
# +/B1iEGc/8UDUZmq12yfk2mHZSmDhcJgFMTIyTsU2sCB8B8NdN6SIqvK9/t0fCfm
# 90obf6fDni2uiuqm5qonFn1h95hxEbziUKFL5V365Q6nLJ+qZSDT2JboyHylTkhE
# /xniRAeSC9dohIBdanhkRc1gRn5UwRN8xXnxycFxAgMBAAGjgfowgfcwHQYDVR0O
# BBYEFF+a9W5czMx0mtTdfe8/2+xMgC7dMDIGCCsGAQUFBwEBBCYwJDAiBggrBgEF
# BQcwAYYWaHR0cDovL29jc3AudGhhd3RlLmNvbTASBgNVHRMBAf8ECDAGAQH/AgEA
# MD8GA1UdHwQ4MDYwNKAyoDCGLmh0dHA6Ly9jcmwudGhhd3RlLmNvbS9UaGF3dGVU
# aW1lc3RhbXBpbmdDQS5jcmwwEwYDVR0lBAwwCgYIKwYBBQUHAwgwDgYDVR0PAQH/
# BAQDAgEGMCgGA1UdEQQhMB+kHTAbMRkwFwYDVQQDExBUaW1lU3RhbXAtMjA0OC0x
# MA0GCSqGSIb3DQEBBQUAA4GBAAMJm495739ZMKrvaLX64wkdu0+CBl03X6ZSnxaN
# 6hySCURu9W3rWHww6PlpjSNzCxJvR6muORH4KrGbsBrDjutZlgCtzgxNstAxpghc
# Knr84nodV0yoZRjpeUBiJZZux8c3aoMhCI5B6t3ZVz8dd0mHKhYGXqY4aiISo1EZ
# g362MIIEozCCA4ugAwIBAgIQDs/0OMj+vzVuBNhqmBsaUDANBgkqhkiG9w0BAQUF
# ADBeMQswCQYDVQQGEwJVUzEdMBsGA1UEChMUU3ltYW50ZWMgQ29ycG9yYXRpb24x
# MDAuBgNVBAMTJ1N5bWFudGVjIFRpbWUgU3RhbXBpbmcgU2VydmljZXMgQ0EgLSBH
# MjAeFw0xMjEwMTgwMDAwMDBaFw0yMDEyMjkyMzU5NTlaMGIxCzAJBgNVBAYTAlVT
# MR0wGwYDVQQKExRTeW1hbnRlYyBDb3Jwb3JhdGlvbjE0MDIGA1UEAxMrU3ltYW50
# ZWMgVGltZSBTdGFtcGluZyBTZXJ2aWNlcyBTaWduZXIgLSBHNDCCASIwDQYJKoZI
# hvcNAQEBBQADggEPADCCAQoCggEBAKJjCzlEuLsjp0RJuw7/ofBhClOTsJjbrSwP
# SsVu/4Y8U1UPFc4EPyv9qZaW2b5heQtbyUyGduXgQ0sile7CK0PBn9hotI5AT+6F
# OLkRxSPyZFjwFTJvTlehroikAtcqHs1L4d1j1ReJMluwXplaqJ0oUA4X7pbbYTtF
# UR3PElYLkkf8q672Zj1HrHBy55LnX80QucSDZJQZvSWA4ejSIqXQugJ6oXeTW2XD
# 7hd0vEGGKtwITIySjJEtnndEH2jWqHR32w5bMotWizO92WPISZ06xcXqMwvS8aMb
# 9Iu+2bNXizveBKd6IrIkri7HcMW+ToMmCPsLvalPmQjhEChyqs0CAwEAAaOCAVcw
# ggFTMAwGA1UdEwEB/wQCMAAwFgYDVR0lAQH/BAwwCgYIKwYBBQUHAwgwDgYDVR0P
# AQH/BAQDAgeAMHMGCCsGAQUFBwEBBGcwZTAqBggrBgEFBQcwAYYeaHR0cDovL3Rz
# LW9jc3Aud3Muc3ltYW50ZWMuY29tMDcGCCsGAQUFBzAChitodHRwOi8vdHMtYWlh
# LndzLnN5bWFudGVjLmNvbS90c3MtY2EtZzIuY2VyMDwGA1UdHwQ1MDMwMaAvoC2G
# K2h0dHA6Ly90cy1jcmwud3Muc3ltYW50ZWMuY29tL3Rzcy1jYS1nMi5jcmwwKAYD
# VR0RBCEwH6QdMBsxGTAXBgNVBAMTEFRpbWVTdGFtcC0yMDQ4LTIwHQYDVR0OBBYE
# FEbGaaMOShQe1UzaUmMXP142vA3mMB8GA1UdIwQYMBaAFF+a9W5czMx0mtTdfe8/
# 2+xMgC7dMA0GCSqGSIb3DQEBBQUAA4IBAQB4O7SRKgBM8I9iMDd4o4QnB28Yst4l
# 3KDUlAOqhk4ln5pAAxzdzuN5yyFoBtq2MrRtv/QsJmMz5ElkbQ3mw2cO9wWkNWx8
# iRbG6bLfsundIMZxD82VdNy2XN69Nx9DeOZ4tc0oBCCjqvFLxIgpkQ6A0RH83Vx2
# bk9eDkVGQW4NsOo4mrE62glxEPwcebSAe6xp9P2ctgwWK/F/Wwk9m1viFsoTgW0A
# LjgNqCmPLOGy9FqpAa8VnCwvSRvbIrvD/niUUcOGsYKIXfA9tFGheTMrLnu53CAJ
# E3Hrahlbz+ilMFcsiUk/uc9/yb8+ImhjU5q9aXSsxR08f5Lgw7wc2AR1MIIFIjCC
# BAqgAwIBAgIQBuAUqkL8vKnzQr5Tqe4gIjANBgkqhkiG9w0BAQsFADByMQswCQYD
# VQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGln
# aWNlcnQuY29tMTEwLwYDVQQDEyhEaWdpQ2VydCBTSEEyIEFzc3VyZWQgSUQgQ29k
# ZSBTaWduaW5nIENBMB4XDTE2MDQxNDAwMDAwMFoXDTE5MDYxOTEyMDAwMFowXzEL
# MAkGA1UEBhMCVVMxEzARBgNVBAgTCkNhbGlmb3JuaWExDzANBgNVBAcTBklydmlu
# ZTEUMBIGA1UEChMLS29mYXgsIEluYy4xFDASBgNVBAMTC0tvZmF4LCBJbmMuMIIB
# IjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAp1sCHr59YU9R9Hsy1loUNPur
# VuF0bcL0/9usV+BHWYSM1jAWwsnrzKmss7dVu4Ld6XeB+vjE7lFT4SNPNPbZguiS
# 1jVKD8Kk83JszhHIRIf6fecSChF56Nj9mwHz9N3j9QmeBQhxUaoBZSrbOsW+g25U
# HxLQtX4Uc7xZ+BVWyqWs31ZcENgz1rWM8FID4TBZoH6Kp23EGVAL06vgcBUegsB5
# XwIf1m2iPI02akjuB/ib5lrTufmYGJkRCyALh+I25QVaXXuSZ5um3EMdUfkG2OGh
# 5g/rd7CPz7H8j1UXJu/wgDxOsNY/vCDUP6SyWBKFyBSxQBSZzBku0Xio/aPLvQID
# AQABo4IBxTCCAcEwHwYDVR0jBBgwFoAUWsS5eyoKo6XqcQPAYPkt9mV1DlgwHQYD
# VR0OBBYEFNgH2nzkwJngFB8kbXfVxmrR9zf8MA4GA1UdDwEB/wQEAwIHgDATBgNV
# HSUEDDAKBggrBgEFBQcDAzB3BgNVHR8EcDBuMDWgM6Axhi9odHRwOi8vY3JsMy5k
# aWdpY2VydC5jb20vc2hhMi1hc3N1cmVkLWNzLWcxLmNybDA1oDOgMYYvaHR0cDov
# L2NybDQuZGlnaWNlcnQuY29tL3NoYTItYXNzdXJlZC1jcy1nMS5jcmwwTAYDVR0g
# BEUwQzA3BglghkgBhv1sAwEwKjAoBggrBgEFBQcCARYcaHR0cHM6Ly93d3cuZGln
# aWNlcnQuY29tL0NQUzAIBgZngQwBBAEwgYQGCCsGAQUFBwEBBHgwdjAkBggrBgEF
# BQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29tME4GCCsGAQUFBzAChkJodHRw
# Oi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRTSEEyQXNzdXJlZElEQ29k
# ZVNpZ25pbmdDQS5jcnQwDAYDVR0TAQH/BAIwADANBgkqhkiG9w0BAQsFAAOCAQEA
# 5Pxp9n7UG1jGtY6fI0DQ6HdIzewX7ffaahn1PaBLmRzk/Bo6qhz+FyGqB4Yalnpv
# OWZxqjKtXh7+sbZn7p59ypS55lBk1A6DBNSooOvFaEF/+edq7fO4PJZeNXAeT0zn
# TxQsfr2HaVtdThasWH86RTIBRIDHqbLeL8n1CVxjuxcBh6xRWlqNWlslwniixEVe
# jEaB4z/xkXT4lvNA7Eav7J2D8stRkSBfefwVpgCNDjf6oh5R8pEPYJR4Xo3YSleK
# sTyZgZCdUtsXzFOwVMOcjqhFBpZTO1PZj7IsJbWR67VmCB5rrWs059XS8ui2SCPA
# REKUxt2Yk9sguJqS4ph9SzCCBTAwggQYoAMCAQICEAQJGBtf1btmdVNDtW+VUAgw
# DQYJKoZIhvcNAQELBQAwZTELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0
# IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEkMCIGA1UEAxMbRGlnaUNl
# cnQgQXNzdXJlZCBJRCBSb290IENBMB4XDTEzMTAyMjEyMDAwMFoXDTI4MTAyMjEy
# MDAwMFowcjELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcG
# A1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTExMC8GA1UEAxMoRGlnaUNlcnQgU0hBMiBB
# c3N1cmVkIElEIENvZGUgU2lnbmluZyBDQTCCASIwDQYJKoZIhvcNAQEBBQADggEP
# ADCCAQoCggEBAPjTsxx/DhGvZ3cH0wsxSRnP0PtFmbE620T1f+Wondsy13Hqdp0F
# LreP+pJDwKX5idQ3Gde2qvCchqXYJawOeSg6funRZ9PG+yknx9N7I5TkkSOWkHeC
# +aGEI2YSVDNQdLEoJrskacLCUvIUZ4qJRdQtoaPpiCwgla4cSocI3wz14k1gGL6q
# xLKucDFmM3E+rHCiq85/6XzLkqHlOzEcz+ryCuRXu0q16XTmK/5sy350OTYNkO/k
# tU6kqepqCquE86xnTrXE94zRICUj6whkPlKWwfIPEvTFjg/BougsUfdzvL2FsWKD
# c0GCB+Q4i2pzINAPZHM8np+mM6n9Gd8lk9ECAwEAAaOCAc0wggHJMBIGA1UdEwEB
# /wQIMAYBAf8CAQAwDgYDVR0PAQH/BAQDAgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMD
# MHkGCCsGAQUFBwEBBG0wazAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNl
# cnQuY29tMEMGCCsGAQUFBzAChjdodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20v
# RGlnaUNlcnRBc3N1cmVkSURSb290Q0EuY3J0MIGBBgNVHR8EejB4MDqgOKA2hjRo
# dHRwOi8vY3JsNC5kaWdpY2VydC5jb20vRGlnaUNlcnRBc3N1cmVkSURSb290Q0Eu
# Y3JsMDqgOKA2hjRodHRwOi8vY3JsMy5kaWdpY2VydC5jb20vRGlnaUNlcnRBc3N1
# cmVkSURSb290Q0EuY3JsME8GA1UdIARIMEYwOAYKYIZIAYb9bAACBDAqMCgGCCsG
# AQUFBwIBFhxodHRwczovL3d3dy5kaWdpY2VydC5jb20vQ1BTMAoGCGCGSAGG/WwD
# MB0GA1UdDgQWBBRaxLl7KgqjpepxA8Bg+S32ZXUOWDAfBgNVHSMEGDAWgBRF66Kv
# 9JLLgjEtUYunpyGd823IDzANBgkqhkiG9w0BAQsFAAOCAQEAPuwNWiSz8yLRFcgs
# fCUpdqgdXRwtOhrE7zBh134LYP3DPQ/Er4v97yrfIFU3sOH20ZJ1D1G0bqWOWuJe
# JIFOEKTuP3GOYw4TS63XX0R58zYUBor3nEZOXP+QsRsHDpEV+7qvtVHCjSSuJMbH
# JyqhKSgaOnEoAjwukaPAJRHinBRHoXpoaK+bp1wgXNlxsQyPu6j4xRJon89Ay0BE
# pRPw5mQMJQhCMrI2iiQC/i9yfhzXSUWW6Fkd6fp0ZGuy62ZD2rOwjNXpDd32ASDO
# mTFjPQgaGLOBm0/GkxAG/AeB+ova+YJJ92JuoVP6EpQYhS6SkepobEQysmah5xik
# mmRR7zGCBDcwggQzAgEBMIGGMHIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdp
# Q2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xMTAvBgNVBAMTKERp
# Z2lDZXJ0IFNIQTIgQXNzdXJlZCBJRCBDb2RlIFNpZ25pbmcgQ0ECEAbgFKpC/Lyp
# 80K+U6nuICIwCQYFKw4DAhoFAKB4MBgGCisGAQQBgjcCAQwxCjAIoAKAAKECgAAw
# GQYJKoZIhvcNAQkDMQwGCisGAQQBgjcCAQQwHAYKKwYBBAGCNwIBCzEOMAwGCisG
# AQQBgjcCARUwIwYJKoZIhvcNAQkEMRYEFDFZ2dKgwnfbO1weSebrj7V+1yltMA0G
# CSqGSIb3DQEBAQUABIIBAHmUlgEc1o7hwMhYBCyWTEfb701dnj9Ur79kULb+ViRh
# L7V/rCBbRPRAqxRTmKRnTZ5wNdCreVYk/q1d7ZjhbqWkth6qsIwBXP08eNW9KCrV
# SsvHbklikLISNNktohIWsV3aWHpcoRuucKQdbqSfL4RvAQNyL0zDTehT+EcMm9I/
# gfxHqeSbuIa4mYnRtCRz1V0HTGYC2XfXIHEMNOZHVmFEDIATJii+0vdvNVKFlJsq
# cQPHZKIOguuBWntbYKSTvJlwT1RCBxBTdtxIyrO8aHASAlVfO2ZPyMv4ZJrWj5ff
# ch5VTjVumHgDTnXk1fhNyoiM6yR1ELz2iPv3qRlctmChggILMIICBwYJKoZIhvcN
# AQkGMYIB+DCCAfQCAQEwcjBeMQswCQYDVQQGEwJVUzEdMBsGA1UEChMUU3ltYW50
# ZWMgQ29ycG9yYXRpb24xMDAuBgNVBAMTJ1N5bWFudGVjIFRpbWUgU3RhbXBpbmcg
# U2VydmljZXMgQ0EgLSBHMgIQDs/0OMj+vzVuBNhqmBsaUDAJBgUrDgMCGgUAoF0w
# GAYJKoZIhvcNAQkDMQsGCSqGSIb3DQEHATAcBgkqhkiG9w0BCQUxDxcNMTkwMjE0
# MTU1MzQ4WjAjBgkqhkiG9w0BCQQxFgQUsvkCS4stgKKlSXVjH8SVbQnHsPowDQYJ
# KoZIhvcNAQEBBQAEggEAFSpD2iZ39o9s/1/irGa9mm8+J1W6GlJQxk5JJq2lZ9kA
# FgZ0wq/5/xMvkB5JN2Zsr6dsULvGPpWl/a/epuiVGXMvG8dInrf7+bfVbkYPqA7Y
# UvpsPdWPtcicd2Tgm8ctaN81wEdBGqVyAoiMwy1IzXGYVkQOhbaNFFB14q9BLpnO
# vqqreJ6qA7qmOWKc31IKvTOD6xrjHUdlzM+Sd987Jm3Nbgk60zMMotZJ5XrNolzI
# NotoqmVRCiSUPsEK6bStXWWoAAp8OZUZlQ1vVRSOpCXBds0vrFIJ9p7a/mgrO4Ew
# mI0wdTYFirdVGTl7E7Bp6NHkIrzLEMdM/Xr0nMFonA==
# SIG # End signature block
