﻿# =============================================================================================================
# INI file read and write.
# Origin: http://blogs.technet.com/b/heyscriptingguy/archive/2011/08/20/use-powershell-to-work-with-any-ini-file.aspx
# =============================================================================================================

Set-Variable -name CONFIGURATION_PART_GLOBAL         -value '<<global>>' -option readonly -force
Set-Variable -name CONFIGURATION_REJECT_UNKNOWN_KEYS -value $True        -option readonly -force
Set-Variable -name CONFIGURATION_INCLUDE_COMMENTS    -value $False       -option readonly -force

Set-Variable -name CONFIGURATION_REJECT_EMPTY_VALUES -value $True        -option readonly -force
Set-Variable -name CONFIGURATION_EMPTY_VALUE         -value $True        -option readonly -force
Set-Variable -name CONFIGURATION_NULL_VALUE          -value $False        -option readonly -force

function Create-Section {
    <# Helper function for Load-IniFile #>
    [CmdletBinding()]
    param (
        [Parameter(Mandatory=$true)]$config,
        [Parameter(Mandatory=$true)][string]$section
    )
    if ($config[$section] -eq $null) {
        $s = New-Object PSObject -Property @{ Keys=@{}; Values=@{} }
        $config[$section] = $s
    }
}

function Load-IniFile {
    <#
    .Synopsis
        Load-IniFile - read ini file content and append content to existing configuration.
    .Arguments
        $iniFile            - File to read.
        [ref]$Configuration - File content.
        Return value        - True if content could be loaded without errors, false otherwise.
    .Desctiption
        Name/Value pairs:
        All name/value pairs in a section are stored in a hash table, all sections
        are included in a top level hash table.
        
        The name/value pair are stored in two seperate tables within the hash table entry
        for a section, one for the name part, 'Keys' -> [int] -> [string], and one for 
        the value part, 'Values' -> [string] -> [string]. The 'Keys' table preserves the
        original order.
        
        Comments:
        Optional, by setting the global variable CONFIGURATION_INCLUDE_COMMENTS to $True,
        it is possible to include comment lines in the configuration hash tables.
        Comment lines are recognisable as key names that start with a semicolon ';'
        followed by an integer. The comment lines can be distinguished from key/value
        pairs by the semicolon in the key part.
        
        Names without a value:
        Optional, by setting the global variable $CONFIGURATION_REJECT_EMPTY_VALUES to $False,
        it is possible to accept names without a value. The default value for such name/value
        pairs is $CONFIGURATION_EMPTY_VALUE.
        
        Value types.
        All name-value pairs from the .ini file are read as type [string].
        Converting known value types to [int] or [bool] is possible by using IsValid-Integer
        and IsValid-Bool.
        It is possible to include new name/value pairs where the value can be any type.
        (Note: Stick to [string], [int] and [bool] types if you intend to write the
        configuration back to an .ini file. Values are not serialised and any other type
        might violate the .ini structure unnoticed.)
     .Example
        Here is the content of an .ini file and its resulting hash tables.
        
        INI File.
        [Section1]
        key1 = value1
        key2 = value2
        1 = one
        
        [Section2]
        key1 = value1
        key2 = value2
        2 = two
        
        HASH Table.
        config['Section1'].Keys[0]      = 'key1'
        config['Section1'].Keys[1]      = 'key2'
        config['Section1'].Keys[2]      = '1'
        config['Section1'].Values['key1']='value1'
        config['Section1'].Values['key2']='value2'
        config['Section1'].Values['1']   ='one'
        config['Section2'].Keys[0]      = 'key1'
        config['Section2'].Keys[1]      = 'key2'
        config['Section2'].Keys[2]      = '2']
        config['Section2'].Values['key1']='value1'
        config['Section2'].Values['key2']='value2'
        config['Section2'].Values['2']   ='two'
        
     .Example
        How to acces the value for key 'key1' in section 'Section1'?
        $value1 = Get-Key $config 'Section1' 'key1'
        
        How to access the value for the first entry in section 'Section1'?
        $value1 = Get-Key $config 'Section1' ([int]1)
        (Note: the type of the index, an [int], forces Get-Key to take
        it as an indirection.)
        
        How to access the value for a key '1' in section 'Section1'?
        $value1 = Get-Key $config 'Section1' ([string]'1')
        (Note: the type of the index, a [string], forces Get-Key to take
        it as a key name.)
        
        How to loop over all values 'in sequence' of the original .ini file?
        for ([int]$i = 0; $i -lt $config[$section].$CONFIGURATION_KEYS.Length; $i++) {
            $name  = Get-Name $config $section $i
            $value = Get-Key $config $section $i
        }
    .See also
        Get-IniFile, Get-Name, Get-Key, IsValid-Integer, IsValid-Bool
    #>
    [CmdletBinding()]
    Param(
        [Parameter(Mandatory=$true)][string]$FilePath,
        [Parameter(Mandatory=$true)][ref]$Configuration
    )
    [bool]$Result = $false
    
    $config = $Configuration.Value
    if ($config -eq $null) { $config = @{} }
    if (-NOT (Test-Path $FilePath)) {
        Write-Error "Unable to locate file: `"$FilePath`""
    } else {
        Write-Host "Read ini file: `"$filePath`""
        $section = $CONFIGURATION_PART_GLOBAL
        Create-Section $config $section
        $IndexCount = $config[$section].Keys.Count
        $CommentCount = 0
        $Reject = $false
        switch -regex -file $FilePath {
            '^\[(.+)\]' { # Section
                $section = $matches[1].Trim()
                Create-Section $config $section
                $IndexCount = $config[$section].Keys.Count
            }
            '^(;.*)$' { # Comment
                if ($CONFIGURATION_INCLUDE_COMMENTS) {
                    $value = $matches[1].Trim()
                    $CommentCount++
                    $name = ';' + $IndexCount
                    $config[$section].Keys[$IndexCount] = $name
                    $IndexCount++
                    $config[$section].Values[$name] = $value
                }
            } 
            '^([^;].*?)\s*=(.*)$' { # Key=Value
                $name,$value = $matches[1..2]
                $name = $name.Trim()
                $config[$section].Keys[$IndexCount] = $name
                $IndexCount++
                $config[$section].Values[$name] = $value.Trim()
            }
            '^([^;][^][=]+)$' { # Key
                $name = $matches[1].Trim()
                if ($CONFIGURATION_REJECT_EMPTY_VALUES) {
                    Write-Error "content[$section][$name] has no value: configuration rejected."
                    $Reject = $True
                } else {
                    $config[$section].Keys[$IndexCount] = $name
                    $IndexCount++
                    $config[$section].Values[$name] = $CONFIGURATION_EMPTY_VALUE
                }
            }
        }
        $Result = -not $Reject
    }
    $Configuration.Value = $config
    return $Result
}

function Get-IniFile {
    <#
    .Synopsis
        Get-IniFile - read ini file content.
    .Description
        How to load the content from an ini file?
        [bool]$result = Get-IniFile "c:\path\to\file.ini" ([ref]$Configuration)
        
        How to access a section?
        $Configuration[$section]
        
        How to access a name/value pair?
        $value = $Configuration[$section].Keys[$name]
    .Arguments
        $FilePath    - File to read.
        [ref]$Config - Configuration from the file.
        Return value - True if content could be loaded without errors, false otherwise.
    .See also
        Load-IniFile
    #>
    [CmdletBinding()]
    Param(
        [Parameter(Mandatory=$true)][string]$FilePath,
        [Parameter(Mandatory=$true)][ref]$Config
    )
    $Config.Value = @{}
    return Load-IniFile $FilePath $Config
}

function Write--Value {
    <# Helper function for Out-IniFile #>
    [CmdletBinding()]
    param (
        [Parameter(Mandatory=$true)][string]$name,
        [Parameter(Mandatory=$true)][AllowEmptyString()][string]$value,
        $outFile
    )
    if ($name -match '^;[\d]+') {
        Add-Content -Path $outFile -Value $value -Encoding UTF8
    } else {
        Add-Content -Path $outFile -Value "$name = $value" -Encoding UTF8
    }
}

function Write--Content {
    <# Helper function for Out-IniFile #>
    [CmdletBinding()]
    param (
        [Parameter(Mandatory=$true)][Hashtable]$Config,
        [Parameter(Mandatory=$true)][Hashtable]$Exclude,
        [Parameter(Mandatory=$true)][string]$section,
        $outFile
    )
    
    if ($Config.ContainsKey($section)) {
        for ([int]$i = 0; $i -lt $config[$section].Keys.Count; $i++) {
            $name  = $config[$section].Keys[$i]
            if (-not (IsSet-Key $Exclude $section $name)) {
                Write--Value $name $config[$section].Values[$name] $outFile
            }
        }
        Add-Content -Path $outFile -Value '' -Encoding UTF8
    }
}

function Out-IniFile {
    <#
    .Synopsis
        Write .ini content to file with possibilities to omit keys
    .Description
        Note 1:
            Whitespace from original ini file lines is not preserved.
            Sections are written in alphabetic order.
            Keys are written in the original order within a section.
            (optional including comments)
            The original order of sections is not preserved.
            
        Note 2:
            Writing to an existing $FilePath result in errors.
            
    .Arguments
        $Config     - file content to write.
        $Exclude    - keys to omit from $Config in output
        $FilePath   - full file name to write to.
    #>
    [CmdletBinding()]
    Param(
        [Parameter(Mandatory=$true)][Hashtable]$Config,
        [Parameter(Mandatory=$false)][Hashtable]$Exclude = @{},
        [Parameter(Mandatory=$true)][string] $FilePath
    )
    if (Test-Path $FilePath) {
        Write-Host "Write existing ini file `"$FilePath`""
    } else {
        Write-Host "Write new ini file `"$FilePath`""
    }
    $outFile = New-Item -ItemType file -Path $FilePath -Force

    $section = $CONFIGURATION_PART_GLOBAL
    if ($Config.ContainsKey($section)) {
        Write--Content $Config $Exclude $section $outFile
        Add-Content -Path $outFile -Value '' -Encoding UTF8
    }
    foreach ($section in $Config.psbase.keys) {
        if ($section -ne $CONFIGURATION_PART_GLOBAL) {
            Add-Content -Path $outFile -Value "[$section]" -Encoding UTF8
            Write--Content $Config $Exclude $section $outFile
        }
    }
}

function Have-Key {
    <#
    .Synopsis
        Check availability of a part or a name in a part.
    .Description
        If name is null:
        Check if the part is available, return True if available.
        
        If name is not null:
        Check if a name is available in a part, return True if availabe.
    .Arguments
        Configuration: Configuration data, mandatory.
        Part:          String, part name, mandatory.
        Name:          String, name in part, optional, default: null
    .See also
        Have-GlobalKey, IsValid-Key
    #>
    [CmdletBinding()]
    param (
        [Parameter(Mandatory=$true)][Hashtable]$Config,
        [Parameter(Mandatory=$true)][string]$part,
        [string]$name = ''
    )
    return (($Config -ne $null) -and ($config.GetType().Name -eq 'Hashtable') -and `
            ($part -ne '') -and ($Config.ContainsKey($part)) -and `
            (($name -eq '') -or ($Config[$part].Values.ContainsKey($name))))
}

function Have-KeyList {
    <#
    .Synopsis
        Check availability of a list of names in a part
    .Description
        Check if a list of names is available in a part,
        return True if all are availabe.
    .Arguments
        Configuration: Configuration data, mandatory.
        Part:          String, part name, mandatory.
        Names:         String[], names in part
    .See also
        Have-GlobalKey, IsValid-Key
    #>
    param (
        [Parameter(Mandatory=$true)][Hashtable]$Config,
        [Parameter(Mandatory=$true)][string]$part,
        [Parameter(Mandatory=$true)][string[]]$names
    )
    
    [bool]$Result = Have-Key $Config $part
    if ($Result) {
        foreach ($name in $names) {
            if (-not (Have-Key $Config $part $name)) {
                Write-Warning "Name missing in configuration part: $part -> $name"
                $Result = $false
            }
        }
    } else {
        Write-Warning "Part missing in configuration: $part"
    }
    return $Result
}

Function Get-Key {
    <#
    .Synopsis
        Get part/key value from a configuration
    .Arguments
        $Configuration - ini file content
        $part          - part to look for.
        $name          - name to look for.
        $default       - (optional, default: $false)
                         default value (used if key is not available).
        Get-Key        - found value (by lookup or by default).
    .Description
        If part and name in that part is available, return the value with it.
        If part or name is not available return the value "False"
        
        Note 1: Use Have-Key to check if a name (or part) is available.
    #>
    [CmdletBinding()]
    Param (
        [Parameter(Mandatory=$true)][Hashtable]$Config,
        [Parameter(Mandatory=$true)][string]$part,
        [Parameter(Mandatory=$true)]$name,
        $default = $CONFIGURATION_NULL_VALUE
    )
    if (-NOT ($Config.ContainsKey($part))) {
        if ($CONFIGURATION_REJECT_UNKNOWN_KEYS) {
            $message = "Configuration[$part] is unknown."
            Write-Error $message
            throw $message
        } else {
            $result = $default
        }
    } else {
        $message = ''
        if ($name.GetType().Name -eq 'Int32') {
            if ((0 -le $name) -and ($name -lt $Config[$part].Keys.Count)) {
                $name = $Config[$part].Keys[$name]
            } else {
                $message = "Configuration[$part][[int]$name] is unknown."
                $name = ''
            }
        }

        if (($name -eq '') -or -not ($Config[$part].Values.ContainsKey($name))) {
            if ($CONFIGURATION_REJECT_UNKNOWN_KEYS) {
                if ($message -eq '') { $message = "Configuration[$part]['$name'] is unknown." }
                Write-Error $message
                throw $message
            } else {
                $result = $default
            }
        } else {
            $result = $Config[$part].Values[$name]
        }
    }
    #Write-Verbose "Get key: Configuration[$Part][$name]=$result"
    return $result
}

function Get-Name {
    <#
    .Synopsis
        Get name from a name/value pair based on its index
    .Arguments
        $Configuration - ini file content
        $part          - part to look for.
        $intex         - name to look for.
        Get-ame        - found name.
    .Description
        If part and index in that part is available, return the name with it.
        If part or name is not available return the value '' (empty string)
    #>
    [CmdletBinding()]
    Param (
        [Parameter(Mandatory=$true)][Hashtable]$Config,
        [Parameter(Mandatory=$true)][string]$part,
        [Parameter(Mandatory=$true)][int]$index
    )
    
    $result = ''
    if (-NOT ($Config.ContainsKey($part))) {
        if ($CONFIGURATION_REJECT_UNKNOWN_KEYS) {
            $message = "Configuration[$part] is unknown."
            Write-Error $message
            throw $message
        }
    } else {
        if ((0 -le $index) -and ($index -lt $Config[$part].Keys.Count)) {
            $result = $Config[$part].Keys[$index]
        }
    }
    #Write-Verbose "Get name: Configuration[$Part][$index]=$result"
    return $result
}

function Get-Count {
    <#
    .Synopsis
        Get the number of parts or the number of name/value pairs available.
    .Arguments
        $Configuration - ini file content
        $part          - (optional) part to look for.
    .Description
        If part is the empty string, return the number of parts available. (including the global part.)
        If a part is given, return the number of name/value pairs in that part.
    #>
    [CmdletBinding()]
    Param (
        [Parameter(Mandatory=$true)][Hashtable]$Config,
        [string]$part = ''
    )
    
    [int]$result = 0
    if ($part -eq '') {
        $Result = $Config.Count
    } elseif ($Config.ContainsKey($part)) {
        $Result = $Config[$part].Keys.Count
    }
    #Write-Verbose "Get count: Configuration[$Part]=$result"
    return $result
}

function Set-Key {
    <#
    .Synopsis
        Set a named value in a part, create if required.
    .Description
        Assign a new value to a name in a part.
        Create part and/or name if required.
    .Arguments
        Configuration: Configuration data, mandatory.
        Part:          String, part name, mandatory
        Name:          String, name in part, mandatory
        Value:         String, new value, mandatory
        Create:        Switch, create a new part!name value if true and part!name is not available.
        Return:        Boolean, true if successfull, false otherwise.
    .See also
        Set-GlobalKey
    #>
    [CmdletBinding()]
    param (
        [Parameter(Mandatory=$true)][Hashtable]$config,
        [Parameter(Mandatory=$true)][string]$part,
        [Parameter(Mandatory=$true)][string]$name,
        [Parameter(Mandatory=$true)]$value,
        [Boolean]$Create=$False
    )
    [Boolean]$result = $false
    if (($config -ne $null) -and ($part -ne '') -and ($name -ne '')) {
        $PartAvailable = $config.ContainsKey($part)
        if (-not $PartAvailable) {
            if ($Create) {
                #Write-Host "Create part: Configuration[$Part]"
                Create-Section $config $part
                $PartAvailable = $True
            } else {
                Write-Warning "Part not available, no create flag: config[$part][$name] = '$value'"
            }
        }
        if ($PartAvailable) {
            $NameAvailable = $config[$part].Values.ContainsKey($name)
            if ($NameAvailable) {
                $config[$part].Values[$name] = $value
                $result = $true
            }
            elseif ($Create) {
                [int]$index = $config[$part].Keys.Count
                $config[$part].Keys[$index] = $name
                $config[$part].Values[$name] = $value
                $result = $true
            } else {
                Write-Warning "Name not available, no create flag: config[$part][$name] = '$value'"
            }
        }
    }
    #Write-Verbose "Set key: Configuration[$Part][$name]=$value"
    return $result
}

function Copy-Key {
    <#
    .Synopsis
        Copies a key from one part to another
    .Description
        Copies a key from one part to another.
        Create part and/or name if required.
    .Arguments
        Configuration: Configuration data, mandatory.
        SourcePart:    String, part name, mandatory
        DestPart:      String, part name, mandatory 
        Name:          String, name in part, mandatory
        Return:        Boolean, true if key was copied.
    #>
    [CmdletBinding()]
    param (
        [Parameter(Mandatory=$true)][Hashtable]$config,
        [Parameter(Mandatory=$true)][string]$srcpart,
        [Parameter(Mandatory=$true)][string]$dstpart,
        [Parameter(Mandatory=$true)][string]$name
    )
    [Boolean]$result = $false
    if (Have-Key $config $srcpart $name) {
        $result = Set-Key $config $dstpart $name (Get-Key $config $srcpart $name) $true
    } elseif (Have-Key $config $dstpart $name) {
        # The key exists in the destination but not in the source.
        # It should be deleted in the destination, but we don't have code
        # for that yet. (Due to the index-based Keys substructure this is
        # not trivial.) As the function is currently only used to fill a
        # part that is guaranteed to be empty, we should not get here and
        # will panic if we do.
        Write-Error "Cannot clear key $name in $dstpart"
        Exit 1
    }
    return $result
}

function Have-GlobalKey {
    <#
    .Synopsis
        Check availability of a name in the global part.
    .Description
        Check if a name is available in a part, return True if availabe.
    .Arguments
        Configuration: Configuration data, mandatory.
        Name:          String, name in global part, mandatory.
    .See also
        Have-Key
    #>
    [CmdletBinding()]
    param (
        [Parameter(Mandatory=$true)][Hashtable]$Config,
        [Parameter(Mandatory=$true)][string]$name
    )
    return HaveKey $Config $CONFIGURATION_PART_GLOBAL $name
}

Function Get-GlobalKey {
    <#
    .Synopsis
        Get-Key - get key value from inifile
    .Arguments
        $Configuration - ini file content
        $section       - section to look for.
        $key           - key to look for.
        $default       - default value (used if key is not available).
        Get-GlobalKey  - found value (by lookup or by default).
    .See also
        Get-Key
    #>
    [CmdletBinding()]
    Param(
        [Parameter(Mandatory=$true)][Hashtable]$Config,
        [Parameter(Mandatory=$true)][string] $key,
        $default = ''
    )
    return Get-Key $Config $CONFIGURATION_PART_GLOBAL $key $default
}

function Set-GlobalKey {
    <#
    .Synopsis
        Set a named value in a part, create if required.
    .Description
        Assign a new value to a name in a part.
        Create part and/or name if required.
    .Arguments
        Configuration: Configuration data, mandatory.
        Name:          String, name in part, mandatory
        Value:         String, new value, mandatory
        Create:        Switch, create a new part!name value if true and part!name is not available.
        Return:        Boolean, true if successfull, false otherwise.
    .See also
        Set-Key
    #>
    [CmdletBinding()]
    param (
        [Parameter(Mandatory=$true)][Hashtable]$Config,
        [Parameter(Mandatory=$true)][string]$name,
        [Parameter(Mandatory=$true)]$value,
        [Boolean]$Create=$False
    )
    return Set-Key $Config $CONFIGURATION_PART_GLOBAL $name $value $Create
}

function Dump-IniFile {
    <#
    .Synopsis
        Show all confirutation values
    .Description
        Show all content in the configuration structure for all [string]keys.
    .Arguments
        $Config: Configuration data, mandatory.
    #>
    param (
        [Parameter(Mandatory=$true)][Hashtable]$Config
    )
    
    if ($Config -eq $null) {
        Write-Host 'Configuration: No configuration available.'
    } else {
        Write-Host "Configuration: count: $($Configuration.Count)"
        foreach ($part in $Config.keys) {
            Write-Host "Configuration[$part]: count: $($Config[$part].Count)"
            $config[$part].Values.GetEnumerator() | foreach {
                Write-Host "Configuration[$part]['$($name.key)'] = '$($value.value)'"
            }
            Write-Host ''
        }
    }
}

function IsValid-Part {
    <#
    .Synopsis
        Check part to be available.
    .Description
        Check a part on to be available.
    .Arguments
        Configuration: Configuration data, mandatory.
        part:          part in the configuration, mandatory.
        return value:  True is part is availabe.
    .See also
        IsValid-Key
    #>
    param (
        [Parameter(Mandatory=$true)][HashTable]$Config,
        [Parameter(Mandatory=$true)][string]$part
    )
    $result = Have-Key $Config $part
    if (-not $result) {
        Write-Warning "Configuration[$part] missing."
    }
    return $result
}

function IsValid-Key {
    <#
    .Synopsis
        Check part/key to have a value.
    .Description
        Check a part/key=value pair on to be available.
    .Arguments
        Configuration: Configuration data, mandatory.
        part:          part in the configuration, mandatory.
        name:          name in the part, mandatory.
        return value:  True is name is availabe and its value is not an empty string.
    .See also
        IsValid-Part, Have-Key, Dump-IniFile
    #>
    param (
        [Parameter(Mandatory=$true)][HashTable]$Config,
        [Parameter(Mandatory=$true)][string]$part,
        [Parameter(Mandatory=$true)][string]$name
    )
    $result = $false
    if (-not (Have-Key $Config $part $name)) {
        Write-Warning "Configuration[$part][$name] missing."
        return $false
    } else {
        $value = Get-Key $Config $part $name
        $result = ($value -ne '')
        if (-not $result) { Write-Warning "Configuration[$part]['$name'] = `"`""}
    }
    return $result
}

function IsValid-Integer {
    <#
    .Synopsis
        Parse and check that a string contains an integer.
    .Description
        Check that some integer in the configuration contains a string
        with an integer content. Convert that setting to an integer.
        Return true if acceptable, return false otherwise.
    #>
    param (
        [Parameter(Mandatory=$true)][HashTable]$Config,
        [Parameter(Mandatory=$true)][string]$part,
        [Parameter(Mandatory=$true)][string]$name
    )
    $entry = Get-Key $Config $part $name
    if ($entry.GetType().FullName -eq 'System.Int32') {
        return $result = $true
    } else {
        $entry = $entry.Trim()
        [int]$value=0
        $result = [Int32]::TryParse($entry, [ref]$value)
        
        # Must check $value.ToString() because 'entry' = '' parses to 'value' = '0'
        if ($Result) { $result = ($value.ToString() -eq $entry) }
        if ($Result) {
            Set-Key $Config $part $name $value
        } else {
            Write-Warning "Configuration $part $name = `"$entry`" not equal to integer value `"$value`""
        }
    }
    return $result
}

function IsValid-Bool {
    <#
    .Synopsis
        Parse and check that a string contains a boolean.
    .Description
        Check that some boolean in the configuration contains a string
        with a boolean content. Convert that setting to a boolean.
        Return true if acceptable, return false otherwise.
    #>
    param (
        [Parameter(Mandatory=$true)][HashTable]$Config,
        [Parameter(Mandatory=$true)][string]$part,
        [Parameter(Mandatory=$true)][string]$name
    )
    $result = $false
    
    $element = Get-Key $Config $part $name
    if(($element.GetType().FullName) -eq 'System.Boolean') {
        $result = $true
    }
    else {
        $entry = $element.Trim().ToLower()
        if ($entry -eq 'false') {
            set-Key $Config $part $name $false
            $result = $true
        } elseif ($entry -eq 'true') {
            set-Key $Config $part $name $true
            $result = $true
        } else {
            Write-Warning "Configuration $part $name = `"$entry`" not equal to a boolean value"
        }
    }
    return $result
}

function IsValid-IP {
    param (
        [Parameter(Mandatory=$true)][HashTable]$Config,
        [Parameter(Mandatory=$true)][string]$part,
        [Parameter(Mandatory=$true)][string]$name
    )
    
    [bool]$result = $false
    [string]$entry = Get-Key $Config $part $name
    $result = TryParse-IP $entry
    return $result
}

function IsSet-Key {
    <#
    .Synopsis
        Returns a boolean setting
    .Description
        Check that that the given key exists, is a Boolean, and is true.
    #>

    param (
        [Parameter(Mandatory=$true)][HashTable]$config,
        [Parameter(Mandatory=$true)][string]$part,
        [Parameter(Mandatory=$true)][string]$name
    )
    return (Have-Key $config $part $name) -and `
           (IsValid-Bool $config $part $name) -and `
           (Get-Key $config $part $name)
}

# SIG # Begin signature block
# MIIbewYJKoZIhvcNAQcCoIIbbDCCG2gCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUr25Md9HdcH3NjxCu8oetcsaV
# unagghauMIIDtzCCAp+gAwIBAgIQDOfg5RfYRv6P5WD8G/AwOTANBgkqhkiG9w0B
# AQUFADBlMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2VydCBBc3N1cmVk
# IElEIFJvb3QgQ0EwHhcNMDYxMTEwMDAwMDAwWhcNMzExMTEwMDAwMDAwWjBlMQsw
# CQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cu
# ZGlnaWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2VydCBBc3N1cmVkIElEIFJvb3Qg
# Q0EwggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQCtDhXO5EOAXLGH87dg
# +XESpa7cJpSIqvTO9SA5KFhgDPiA2qkVlTJhPLWxKISKityfCgyDF3qPkKyK53lT
# XDGEKvYPmDI2dsze3Tyoou9q+yHyUmHfnyDXH+Kx2f4YZNISW1/5WBg1vEfNoTb5
# a3/UsDg+wRvDjDPZ2C8Y/igPs6eD1sNuRMBhNZYW/lmci3Zt1/GiSw0r/wty2p5g
# 0I6QNcZ4VYcgoc/lbQrISXwxmDNsIumH0DJaoroTghHtORedmTpyoeb6pNnVFzF1
# roV9Iq4/AUaG9ih5yLHa5FcXxH4cDrC0kqZWs72yl+2qp/C3xag/lRbQ/6GW6whf
# GHdPAgMBAAGjYzBhMA4GA1UdDwEB/wQEAwIBhjAPBgNVHRMBAf8EBTADAQH/MB0G
# A1UdDgQWBBRF66Kv9JLLgjEtUYunpyGd823IDzAfBgNVHSMEGDAWgBRF66Kv9JLL
# gjEtUYunpyGd823IDzANBgkqhkiG9w0BAQUFAAOCAQEAog683+Lt8ONyc3pklL/3
# cmbYMuRCdWKuh+vy1dneVrOfzM4UKLkNl2BcEkxY5NM9g0lFWJc1aRqoR+pWxnmr
# EthngYTffwk8lOa4JiwgvT2zKIn3X/8i4peEH+ll74fg38FnSbNd67IJKusm7Xi+
# fT8r87cmNW1fiQG2SVufAQWbqz0lwcy2f8Lxb4bG+mRo64EtlOtCt/qMHt1i8b5Q
# Z7dsvfPxH2sMNgcWfzd8qVttevESRmCD1ycEvkvOl77DZypoEd+A5wwzZr8TDRRu
# 838fYxAe+o0bJW1sj6W3YQGx0qMmoRBxna3iw/nDmVG3KwcIzi7mULKn+gpFL6Lw
# 8jCCA+4wggNXoAMCAQICEH6T6/t8xk5Z6kuad9QG/DswDQYJKoZIhvcNAQEFBQAw
# gYsxCzAJBgNVBAYTAlpBMRUwEwYDVQQIEwxXZXN0ZXJuIENhcGUxFDASBgNVBAcT
# C0R1cmJhbnZpbGxlMQ8wDQYDVQQKEwZUaGF3dGUxHTAbBgNVBAsTFFRoYXd0ZSBD
# ZXJ0aWZpY2F0aW9uMR8wHQYDVQQDExZUaGF3dGUgVGltZXN0YW1waW5nIENBMB4X
# DTEyMTIyMTAwMDAwMFoXDTIwMTIzMDIzNTk1OVowXjELMAkGA1UEBhMCVVMxHTAb
# BgNVBAoTFFN5bWFudGVjIENvcnBvcmF0aW9uMTAwLgYDVQQDEydTeW1hbnRlYyBU
# aW1lIFN0YW1waW5nIFNlcnZpY2VzIENBIC0gRzIwggEiMA0GCSqGSIb3DQEBAQUA
# A4IBDwAwggEKAoIBAQCxrLNJVEuXHBIK2CV5kSJXKm/cuCbEQ3Nrwr8uUFr7FMJ2
# jkMBJUO0oeJF9Oi3e8N0zCLXtJQAAvdN7b+0t0Qka81fRTvRRM5DEnMXgotptCvL
# mR6schsmTXEfsTHd+1FhAlOmqvVJLAV4RaUvic7nmef+jOJXPz3GktxK+Hsz5HkK
# +/B1iEGc/8UDUZmq12yfk2mHZSmDhcJgFMTIyTsU2sCB8B8NdN6SIqvK9/t0fCfm
# 90obf6fDni2uiuqm5qonFn1h95hxEbziUKFL5V365Q6nLJ+qZSDT2JboyHylTkhE
# /xniRAeSC9dohIBdanhkRc1gRn5UwRN8xXnxycFxAgMBAAGjgfowgfcwHQYDVR0O
# BBYEFF+a9W5czMx0mtTdfe8/2+xMgC7dMDIGCCsGAQUFBwEBBCYwJDAiBggrBgEF
# BQcwAYYWaHR0cDovL29jc3AudGhhd3RlLmNvbTASBgNVHRMBAf8ECDAGAQH/AgEA
# MD8GA1UdHwQ4MDYwNKAyoDCGLmh0dHA6Ly9jcmwudGhhd3RlLmNvbS9UaGF3dGVU
# aW1lc3RhbXBpbmdDQS5jcmwwEwYDVR0lBAwwCgYIKwYBBQUHAwgwDgYDVR0PAQH/
# BAQDAgEGMCgGA1UdEQQhMB+kHTAbMRkwFwYDVQQDExBUaW1lU3RhbXAtMjA0OC0x
# MA0GCSqGSIb3DQEBBQUAA4GBAAMJm495739ZMKrvaLX64wkdu0+CBl03X6ZSnxaN
# 6hySCURu9W3rWHww6PlpjSNzCxJvR6muORH4KrGbsBrDjutZlgCtzgxNstAxpghc
# Knr84nodV0yoZRjpeUBiJZZux8c3aoMhCI5B6t3ZVz8dd0mHKhYGXqY4aiISo1EZ
# g362MIIEozCCA4ugAwIBAgIQDs/0OMj+vzVuBNhqmBsaUDANBgkqhkiG9w0BAQUF
# ADBeMQswCQYDVQQGEwJVUzEdMBsGA1UEChMUU3ltYW50ZWMgQ29ycG9yYXRpb24x
# MDAuBgNVBAMTJ1N5bWFudGVjIFRpbWUgU3RhbXBpbmcgU2VydmljZXMgQ0EgLSBH
# MjAeFw0xMjEwMTgwMDAwMDBaFw0yMDEyMjkyMzU5NTlaMGIxCzAJBgNVBAYTAlVT
# MR0wGwYDVQQKExRTeW1hbnRlYyBDb3Jwb3JhdGlvbjE0MDIGA1UEAxMrU3ltYW50
# ZWMgVGltZSBTdGFtcGluZyBTZXJ2aWNlcyBTaWduZXIgLSBHNDCCASIwDQYJKoZI
# hvcNAQEBBQADggEPADCCAQoCggEBAKJjCzlEuLsjp0RJuw7/ofBhClOTsJjbrSwP
# SsVu/4Y8U1UPFc4EPyv9qZaW2b5heQtbyUyGduXgQ0sile7CK0PBn9hotI5AT+6F
# OLkRxSPyZFjwFTJvTlehroikAtcqHs1L4d1j1ReJMluwXplaqJ0oUA4X7pbbYTtF
# UR3PElYLkkf8q672Zj1HrHBy55LnX80QucSDZJQZvSWA4ejSIqXQugJ6oXeTW2XD
# 7hd0vEGGKtwITIySjJEtnndEH2jWqHR32w5bMotWizO92WPISZ06xcXqMwvS8aMb
# 9Iu+2bNXizveBKd6IrIkri7HcMW+ToMmCPsLvalPmQjhEChyqs0CAwEAAaOCAVcw
# ggFTMAwGA1UdEwEB/wQCMAAwFgYDVR0lAQH/BAwwCgYIKwYBBQUHAwgwDgYDVR0P
# AQH/BAQDAgeAMHMGCCsGAQUFBwEBBGcwZTAqBggrBgEFBQcwAYYeaHR0cDovL3Rz
# LW9jc3Aud3Muc3ltYW50ZWMuY29tMDcGCCsGAQUFBzAChitodHRwOi8vdHMtYWlh
# LndzLnN5bWFudGVjLmNvbS90c3MtY2EtZzIuY2VyMDwGA1UdHwQ1MDMwMaAvoC2G
# K2h0dHA6Ly90cy1jcmwud3Muc3ltYW50ZWMuY29tL3Rzcy1jYS1nMi5jcmwwKAYD
# VR0RBCEwH6QdMBsxGTAXBgNVBAMTEFRpbWVTdGFtcC0yMDQ4LTIwHQYDVR0OBBYE
# FEbGaaMOShQe1UzaUmMXP142vA3mMB8GA1UdIwQYMBaAFF+a9W5czMx0mtTdfe8/
# 2+xMgC7dMA0GCSqGSIb3DQEBBQUAA4IBAQB4O7SRKgBM8I9iMDd4o4QnB28Yst4l
# 3KDUlAOqhk4ln5pAAxzdzuN5yyFoBtq2MrRtv/QsJmMz5ElkbQ3mw2cO9wWkNWx8
# iRbG6bLfsundIMZxD82VdNy2XN69Nx9DeOZ4tc0oBCCjqvFLxIgpkQ6A0RH83Vx2
# bk9eDkVGQW4NsOo4mrE62glxEPwcebSAe6xp9P2ctgwWK/F/Wwk9m1viFsoTgW0A
# LjgNqCmPLOGy9FqpAa8VnCwvSRvbIrvD/niUUcOGsYKIXfA9tFGheTMrLnu53CAJ
# E3Hrahlbz+ilMFcsiUk/uc9/yb8+ImhjU5q9aXSsxR08f5Lgw7wc2AR1MIIFIjCC
# BAqgAwIBAgIQBuAUqkL8vKnzQr5Tqe4gIjANBgkqhkiG9w0BAQsFADByMQswCQYD
# VQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGln
# aWNlcnQuY29tMTEwLwYDVQQDEyhEaWdpQ2VydCBTSEEyIEFzc3VyZWQgSUQgQ29k
# ZSBTaWduaW5nIENBMB4XDTE2MDQxNDAwMDAwMFoXDTE5MDYxOTEyMDAwMFowXzEL
# MAkGA1UEBhMCVVMxEzARBgNVBAgTCkNhbGlmb3JuaWExDzANBgNVBAcTBklydmlu
# ZTEUMBIGA1UEChMLS29mYXgsIEluYy4xFDASBgNVBAMTC0tvZmF4LCBJbmMuMIIB
# IjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAp1sCHr59YU9R9Hsy1loUNPur
# VuF0bcL0/9usV+BHWYSM1jAWwsnrzKmss7dVu4Ld6XeB+vjE7lFT4SNPNPbZguiS
# 1jVKD8Kk83JszhHIRIf6fecSChF56Nj9mwHz9N3j9QmeBQhxUaoBZSrbOsW+g25U
# HxLQtX4Uc7xZ+BVWyqWs31ZcENgz1rWM8FID4TBZoH6Kp23EGVAL06vgcBUegsB5
# XwIf1m2iPI02akjuB/ib5lrTufmYGJkRCyALh+I25QVaXXuSZ5um3EMdUfkG2OGh
# 5g/rd7CPz7H8j1UXJu/wgDxOsNY/vCDUP6SyWBKFyBSxQBSZzBku0Xio/aPLvQID
# AQABo4IBxTCCAcEwHwYDVR0jBBgwFoAUWsS5eyoKo6XqcQPAYPkt9mV1DlgwHQYD
# VR0OBBYEFNgH2nzkwJngFB8kbXfVxmrR9zf8MA4GA1UdDwEB/wQEAwIHgDATBgNV
# HSUEDDAKBggrBgEFBQcDAzB3BgNVHR8EcDBuMDWgM6Axhi9odHRwOi8vY3JsMy5k
# aWdpY2VydC5jb20vc2hhMi1hc3N1cmVkLWNzLWcxLmNybDA1oDOgMYYvaHR0cDov
# L2NybDQuZGlnaWNlcnQuY29tL3NoYTItYXNzdXJlZC1jcy1nMS5jcmwwTAYDVR0g
# BEUwQzA3BglghkgBhv1sAwEwKjAoBggrBgEFBQcCARYcaHR0cHM6Ly93d3cuZGln
# aWNlcnQuY29tL0NQUzAIBgZngQwBBAEwgYQGCCsGAQUFBwEBBHgwdjAkBggrBgEF
# BQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29tME4GCCsGAQUFBzAChkJodHRw
# Oi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRTSEEyQXNzdXJlZElEQ29k
# ZVNpZ25pbmdDQS5jcnQwDAYDVR0TAQH/BAIwADANBgkqhkiG9w0BAQsFAAOCAQEA
# 5Pxp9n7UG1jGtY6fI0DQ6HdIzewX7ffaahn1PaBLmRzk/Bo6qhz+FyGqB4Yalnpv
# OWZxqjKtXh7+sbZn7p59ypS55lBk1A6DBNSooOvFaEF/+edq7fO4PJZeNXAeT0zn
# TxQsfr2HaVtdThasWH86RTIBRIDHqbLeL8n1CVxjuxcBh6xRWlqNWlslwniixEVe
# jEaB4z/xkXT4lvNA7Eav7J2D8stRkSBfefwVpgCNDjf6oh5R8pEPYJR4Xo3YSleK
# sTyZgZCdUtsXzFOwVMOcjqhFBpZTO1PZj7IsJbWR67VmCB5rrWs059XS8ui2SCPA
# REKUxt2Yk9sguJqS4ph9SzCCBTAwggQYoAMCAQICEAQJGBtf1btmdVNDtW+VUAgw
# DQYJKoZIhvcNAQELBQAwZTELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0
# IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEkMCIGA1UEAxMbRGlnaUNl
# cnQgQXNzdXJlZCBJRCBSb290IENBMB4XDTEzMTAyMjEyMDAwMFoXDTI4MTAyMjEy
# MDAwMFowcjELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcG
# A1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTExMC8GA1UEAxMoRGlnaUNlcnQgU0hBMiBB
# c3N1cmVkIElEIENvZGUgU2lnbmluZyBDQTCCASIwDQYJKoZIhvcNAQEBBQADggEP
# ADCCAQoCggEBAPjTsxx/DhGvZ3cH0wsxSRnP0PtFmbE620T1f+Wondsy13Hqdp0F
# LreP+pJDwKX5idQ3Gde2qvCchqXYJawOeSg6funRZ9PG+yknx9N7I5TkkSOWkHeC
# +aGEI2YSVDNQdLEoJrskacLCUvIUZ4qJRdQtoaPpiCwgla4cSocI3wz14k1gGL6q
# xLKucDFmM3E+rHCiq85/6XzLkqHlOzEcz+ryCuRXu0q16XTmK/5sy350OTYNkO/k
# tU6kqepqCquE86xnTrXE94zRICUj6whkPlKWwfIPEvTFjg/BougsUfdzvL2FsWKD
# c0GCB+Q4i2pzINAPZHM8np+mM6n9Gd8lk9ECAwEAAaOCAc0wggHJMBIGA1UdEwEB
# /wQIMAYBAf8CAQAwDgYDVR0PAQH/BAQDAgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMD
# MHkGCCsGAQUFBwEBBG0wazAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNl
# cnQuY29tMEMGCCsGAQUFBzAChjdodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20v
# RGlnaUNlcnRBc3N1cmVkSURSb290Q0EuY3J0MIGBBgNVHR8EejB4MDqgOKA2hjRo
# dHRwOi8vY3JsNC5kaWdpY2VydC5jb20vRGlnaUNlcnRBc3N1cmVkSURSb290Q0Eu
# Y3JsMDqgOKA2hjRodHRwOi8vY3JsMy5kaWdpY2VydC5jb20vRGlnaUNlcnRBc3N1
# cmVkSURSb290Q0EuY3JsME8GA1UdIARIMEYwOAYKYIZIAYb9bAACBDAqMCgGCCsG
# AQUFBwIBFhxodHRwczovL3d3dy5kaWdpY2VydC5jb20vQ1BTMAoGCGCGSAGG/WwD
# MB0GA1UdDgQWBBRaxLl7KgqjpepxA8Bg+S32ZXUOWDAfBgNVHSMEGDAWgBRF66Kv
# 9JLLgjEtUYunpyGd823IDzANBgkqhkiG9w0BAQsFAAOCAQEAPuwNWiSz8yLRFcgs
# fCUpdqgdXRwtOhrE7zBh134LYP3DPQ/Er4v97yrfIFU3sOH20ZJ1D1G0bqWOWuJe
# JIFOEKTuP3GOYw4TS63XX0R58zYUBor3nEZOXP+QsRsHDpEV+7qvtVHCjSSuJMbH
# JyqhKSgaOnEoAjwukaPAJRHinBRHoXpoaK+bp1wgXNlxsQyPu6j4xRJon89Ay0BE
# pRPw5mQMJQhCMrI2iiQC/i9yfhzXSUWW6Fkd6fp0ZGuy62ZD2rOwjNXpDd32ASDO
# mTFjPQgaGLOBm0/GkxAG/AeB+ova+YJJ92JuoVP6EpQYhS6SkepobEQysmah5xik
# mmRR7zGCBDcwggQzAgEBMIGGMHIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdp
# Q2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xMTAvBgNVBAMTKERp
# Z2lDZXJ0IFNIQTIgQXNzdXJlZCBJRCBDb2RlIFNpZ25pbmcgQ0ECEAbgFKpC/Lyp
# 80K+U6nuICIwCQYFKw4DAhoFAKB4MBgGCisGAQQBgjcCAQwxCjAIoAKAAKECgAAw
# GQYJKoZIhvcNAQkDMQwGCisGAQQBgjcCAQQwHAYKKwYBBAGCNwIBCzEOMAwGCisG
# AQQBgjcCARUwIwYJKoZIhvcNAQkEMRYEFIiD1CnHUJBRGfLaoPmAbHZWKrYJMA0G
# CSqGSIb3DQEBAQUABIIBAJmjhnUJ7g84fE6VL3V8HnVXODsBuV6MG1BmuE/mTJ47
# FNJpa+XfAOgszuCjJGLw3Bse1TezKHpC8yhFjHvjduAgp+ounHbGWGA2Lyae1afZ
# 9e0xKdNWi+ibsMSOOwCorowFGwVOREjWcxgEB3YXOJmem3CIu8lxT2skRgjobQYE
# Gr0ifEmefrtLUDp/iJSzpHkhohdwaIntDW+glHYJkTXvGZQRbFBM/RaAIrZ14CN/
# ggSZPD+dmb2ppy4jm2ztwC/sxqK+k410eIawIPpxCZTjf2tKMNuMZFZiXwwMFLa1
# xRiioitG2U6gcJjiGXf5il6vEH7pYwWL8gUk80NFA8mhggILMIICBwYJKoZIhvcN
# AQkGMYIB+DCCAfQCAQEwcjBeMQswCQYDVQQGEwJVUzEdMBsGA1UEChMUU3ltYW50
# ZWMgQ29ycG9yYXRpb24xMDAuBgNVBAMTJ1N5bWFudGVjIFRpbWUgU3RhbXBpbmcg
# U2VydmljZXMgQ0EgLSBHMgIQDs/0OMj+vzVuBNhqmBsaUDAJBgUrDgMCGgUAoF0w
# GAYJKoZIhvcNAQkDMQsGCSqGSIb3DQEHATAcBgkqhkiG9w0BCQUxDxcNMTkwMjE0
# MTU1MzQ4WjAjBgkqhkiG9w0BCQQxFgQU0aYn28OnNItI0Q9nfaKu41qd+BUwDQYJ
# KoZIhvcNAQEBBQAEggEAevVILFsQh0PCF+cRkBXZwXAdQFHCmPbPkvZCB/82l6cn
# XuxfJo3+XPVRS4iyQojGbBpPco/XgH0h+FY4jx8J8J3HC3lTbwH1yN34RO3uhaAr
# r7v6HTCqM2knA3wQtmKw/BZ+A3kFIUggaRmchQ4RdTiGarKzLH+5nhJi07O1rUJq
# CZoRwnHnMvUUb84OqlVfJ+eudsZdNA0ThD7DhTMqTmXKHwAA+wb7pOFzQV8leSIN
# n+BPKZT1zgIvnNtD54X+ZRNxIblRWsDmIXzf2V2qsSaGZ96k4J7HHqagTdonlmAP
# bi6ZTaQSRjF+ZwejSbmpPIS7Qc80tc8JvUJwSPwtKg==
# SIG # End signature block
