﻿# =============================================================================================================
# Transcript functions
# =============================================================================================================

function Start-Trans([switch]$debug) {
    <#
    .Synopsis
        start transcription.
    .Description
        Starting a transcript file in an environment that does not Start-Transcript.
        To see debuging information startTrans -debug.
    .Notes
        source: http://blogs.technet.com/b/heyscriptingguy/archive/2010/09/25/create-a-transcript-of-commands-from-the-windows-powershell-ise.aspx
    #>
    # to see debuging information startTrans -debug 
    if ($debug) { $debugPreference = 'continue' }
    $dte = [dateTime]::Get_Today().DayOfWeek.tostring()
    write-debug $dte
    $dte = $dte + '_' + [dateTime]::now.hour
    write-debug $dte
    if(([datetime]::now.toLocalTIme()) -match 'AM') {
        Write-debug 'Inside if ...'
        $dte = $dte + '_AM'
        write-debug $dte
    } else {
        write-debug 'Inside else ...'
        $dte = $dte + '_PM'
        write-debug $dte
    }
    write-debug "Starting transcript ... c:\trans\$dte.txt"
    start-transcript -path "c:\trans\$dte.txt"
}

Function Get-logNameFromDate {
    <#
    .Synopsis
        Creates a log name from date
    .Description
        This script creates a log from a date. 
    .Example
        Get-logNameFromDate -path "c:\fso" -name "log"
        Creates a file name like c:\fso\log20100914-122019.Txt but does not
        create the file. It returns the file name to calling code.
    .Example
        Get-logNameFromDate -path "c:\fso" -name "log" -Create
        Creates a file name like c:\fso\log20100914-122019.Txt and
        create the file. It returns the file name to calling code.
    .Parameter path
        path to log file
    .Parameter name
        base name of log file
    .Parameter create
        switch that determines whether log file or only name is created
    .inputs
        [string]
    .outputs
        [string]
    .Notes
        source: http://blogs.technet.com/b/heyscriptingguy/archive/2010/09/25/create-a-transcript-of-commands-from-the-windows-powershell-ise.aspx
        
        NAME:  Get-LogNameFromDate
        AUTHOR: ed wilson, msft
        LASTEDIT: 09/10/2010 16:58:06
        KEYWORDS: parameter substitution, format specifier, string substitution
        HSG: WES-09-25-10
    .Link
        Http://www.ScriptingGuys.com
     #Requires -Version 2.0
     #>
    
    Param(
        [string]$path = '.',
        [string]$name = 'log',
        [switch]$Create
    )
    
    $logname = "{0}\{1}{2}.{3}" -f $path,$name, (Get-Date -Format yyyyMMdd-HHmmss),'log'
    if ($create) { New-Item -Path $logname -ItemType file }
    $logname
}

Function Start-iseTranscript {
    <#
    .Synopsis
        This captures output from a script to a created text file
    .Example
        Start-iseTranscript -logname "c:\fso\log.txt"
        Copies output from script to file named xxxxlog.txt in c:\fso folder
    .Parameter logname
        the name and path of the log file.
    .inputs
        [string]
    .outputs
        [io.file]
    .Notes
        source: http://blogs.technet.com/b/heyscriptingguy/archive/2010/09/25/create-a-transcript-of-commands-from-the-windows-powershell-ise.aspx
        
        To use the Start-ISETranscript function, add the call to the function
        at the bottom of your script. The script does not have to be saved or
        have a name. This is shown in the following image. One thing to keep
        in mind is that the entire contents of the output pane will be written
        to the log file. This means that if you run the script two or three
        times and do not clear the output pane, you could have the results from
        several scripts in your transcript log.
        
        With PowerShell 4.0, the Start-ISETranscript function did not work for me.
        I changed
        
            $psISE.CurrentPowerShellTab.Output.Text >> $logname to $psISE.CurrentPowerShellTab.ConsolePane.Text >> $logname
        
        and it now works correctly.
        
        NAME:  Start-iseTranscript
        AUTHOR: ed wilson, msft
        LASTEDIT: 09/10/2010 17:27:22
        KEYWORDS:
        HSG: WES-09-25-10
    .Link
        Http://www.ScriptingGuys.com
     #Requires -Version 2.0
     #>
    
    Param(
        [string]$logname = (Get-logNameFromDate -path '.' -name 'log' -Create)
    )
    
    $transcriptHeader = @"
**************************************
Windows PowerShell ISE Transcript Start
Start Time: $(get-date)
UserName: $env:username
UserDomain: $env:USERDNSDOMAIN
ComputerName: $env:COMPUTERNAME
Windows version: $((Get-WmiObject win32_operatingsystem).version)
**************************************
"@
    $transcriptHeader >> $logname
    $psISE.CurrentPowerShellTab.Output.Text >> $logname
}

function Start-HostTranscript() {
    <#
    .Synopsis
        This tries to detect a method to activate a Trascript action.
        Transcript functionality is depending on the host used.
        It is possible to force the use of Start-Transcript.
        Recognised hosts:
           ConsoleHost:
               Full transcript functionalty available.
           Windows PowerShell ISE Host:
               Limited transcript capabilities available.
           PowerGUIScriptEditorHost:
               Transcript capabilities unknown, handled as "Others".
           Others:
               Unknown transcript capabilities, assume none.
    .Example
        Start-HostTranscript -logname ".\log.txt"
        ...your code here.
        Stop-HostTranscript
        
        Copies output from script to file (if possible) or issues a warning.
    .Example
        Start-HostTranscript -logname ".\log.txt" $True
        ...your code here.
        Stop-HostTranscript
        
        Copies output from script to file (forced).
        This might trigger an exception when the host does not support
        Start-Transcript activations.
    .inputs
        [string]
        [boolean]
    .outputs
        [io.file]
    .Notes
        NAME:  Start-HostTranscript.
        ENVIRONMENT:  Uses 'HostTranscriptLog'.
        SEE ALSO:  Stop-HostTranscript.
        AUTHOR:  AS
        LASTEDIT:  23/01/2014 17:27:22
        KEYWORDS:
        .Link
     #Requires -Version 2.0
     #>
    
    Param(
        [string]$LogName = (Get-logNameFromDate -path '.' -name 'log' -Create),
        [switch]$Force   = $false
    )
    
    switch ($Host.Name) {
        'ConsoleHost' {
            Write-Host "Start Transcription to $logname"
            Start-Transcript -Path $logname | out-null
        }
        'Windows PowerShell ISE Host' {
            Clear-Host
            new-item -path env: -name hosttranscriptlog -value $logname -Force | out-null
            Write-Host "Transcript started, output file is $logname"
        }
        'PowerGUIScriptEditorHost' {
            $Host.UI.WriteErrorLine("WARNING: Transcript support by PowerGUI unknown, no log file created: $logname")
        }
        default {
            new-item -path env: -name hosttranscriptlog -value $Force -Force | out-null
            if ($Force) {
                Start-Transcript -Path $logname | out-null
            }
            else {
                $Host.UI.WriteErrorLine("WARNING: Transcript support by $($Host.Name) unknown, no log file created: $logname")
            }
        }
    }
}

function Stop-HostTranscript() {
    <#
        .Synopsis
        Tries to close a transcript action.
        .Example
        Start-HostTranscript -logname ".\log.txt"
        ...your code here.
        Stop-HostTranscript
        
        Copies output from script to file (if possible) or issues a warning.
        .inputs
        .outputs
        .Notes
        NAME:  StopTranscript
        GLOBALS:  Uses 'HostTranscriptLog'.
        SEE ALSO:  Start-HostTranscript and Start-Transcript for more details.
        AUTHOR:  AS
        LASTEDIT:  23/01/2014 17:27:22
        KEYWORDS:
        .Link
     #Requires -Version 2.0
     #>
    
    param (
        [string]$ClosingMessage = ''
    )
    
    if ($ClosingMessage.Length -ne '') { Write-Host $ClosingMessage }
    switch ($Host.Name) {
        'ConsoleHost' {
            Stop-Transcript | out-null
        }
        'Windows PowerShell ISE Host' {
            $HostTranscriptLog = (get-childitem -path env:hosttranscriptlog).Value
            Write-Host 'Windows PowerShell Transcript End'
            Start-iseTranscript $HostTranscriptLog
        }
        'PowerGUIScriptEditorHost' {
            # no action.
        }
        default {
            $HostTranscriptLog = (get-childitem -path env:hosttranscriptlog).Value
            if ($HostTranscriptLog -eq 'True') { Stop-Transcript | out-null}
        }
    }
}

# SIG # Begin signature block
# MIIbewYJKoZIhvcNAQcCoIIbbDCCG2gCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUDOTnzwJoXf35xpOoc57a/XbL
# dZWgghauMIIDtzCCAp+gAwIBAgIQDOfg5RfYRv6P5WD8G/AwOTANBgkqhkiG9w0B
# AQUFADBlMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2VydCBBc3N1cmVk
# IElEIFJvb3QgQ0EwHhcNMDYxMTEwMDAwMDAwWhcNMzExMTEwMDAwMDAwWjBlMQsw
# CQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cu
# ZGlnaWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2VydCBBc3N1cmVkIElEIFJvb3Qg
# Q0EwggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQCtDhXO5EOAXLGH87dg
# +XESpa7cJpSIqvTO9SA5KFhgDPiA2qkVlTJhPLWxKISKityfCgyDF3qPkKyK53lT
# XDGEKvYPmDI2dsze3Tyoou9q+yHyUmHfnyDXH+Kx2f4YZNISW1/5WBg1vEfNoTb5
# a3/UsDg+wRvDjDPZ2C8Y/igPs6eD1sNuRMBhNZYW/lmci3Zt1/GiSw0r/wty2p5g
# 0I6QNcZ4VYcgoc/lbQrISXwxmDNsIumH0DJaoroTghHtORedmTpyoeb6pNnVFzF1
# roV9Iq4/AUaG9ih5yLHa5FcXxH4cDrC0kqZWs72yl+2qp/C3xag/lRbQ/6GW6whf
# GHdPAgMBAAGjYzBhMA4GA1UdDwEB/wQEAwIBhjAPBgNVHRMBAf8EBTADAQH/MB0G
# A1UdDgQWBBRF66Kv9JLLgjEtUYunpyGd823IDzAfBgNVHSMEGDAWgBRF66Kv9JLL
# gjEtUYunpyGd823IDzANBgkqhkiG9w0BAQUFAAOCAQEAog683+Lt8ONyc3pklL/3
# cmbYMuRCdWKuh+vy1dneVrOfzM4UKLkNl2BcEkxY5NM9g0lFWJc1aRqoR+pWxnmr
# EthngYTffwk8lOa4JiwgvT2zKIn3X/8i4peEH+ll74fg38FnSbNd67IJKusm7Xi+
# fT8r87cmNW1fiQG2SVufAQWbqz0lwcy2f8Lxb4bG+mRo64EtlOtCt/qMHt1i8b5Q
# Z7dsvfPxH2sMNgcWfzd8qVttevESRmCD1ycEvkvOl77DZypoEd+A5wwzZr8TDRRu
# 838fYxAe+o0bJW1sj6W3YQGx0qMmoRBxna3iw/nDmVG3KwcIzi7mULKn+gpFL6Lw
# 8jCCA+4wggNXoAMCAQICEH6T6/t8xk5Z6kuad9QG/DswDQYJKoZIhvcNAQEFBQAw
# gYsxCzAJBgNVBAYTAlpBMRUwEwYDVQQIEwxXZXN0ZXJuIENhcGUxFDASBgNVBAcT
# C0R1cmJhbnZpbGxlMQ8wDQYDVQQKEwZUaGF3dGUxHTAbBgNVBAsTFFRoYXd0ZSBD
# ZXJ0aWZpY2F0aW9uMR8wHQYDVQQDExZUaGF3dGUgVGltZXN0YW1waW5nIENBMB4X
# DTEyMTIyMTAwMDAwMFoXDTIwMTIzMDIzNTk1OVowXjELMAkGA1UEBhMCVVMxHTAb
# BgNVBAoTFFN5bWFudGVjIENvcnBvcmF0aW9uMTAwLgYDVQQDEydTeW1hbnRlYyBU
# aW1lIFN0YW1waW5nIFNlcnZpY2VzIENBIC0gRzIwggEiMA0GCSqGSIb3DQEBAQUA
# A4IBDwAwggEKAoIBAQCxrLNJVEuXHBIK2CV5kSJXKm/cuCbEQ3Nrwr8uUFr7FMJ2
# jkMBJUO0oeJF9Oi3e8N0zCLXtJQAAvdN7b+0t0Qka81fRTvRRM5DEnMXgotptCvL
# mR6schsmTXEfsTHd+1FhAlOmqvVJLAV4RaUvic7nmef+jOJXPz3GktxK+Hsz5HkK
# +/B1iEGc/8UDUZmq12yfk2mHZSmDhcJgFMTIyTsU2sCB8B8NdN6SIqvK9/t0fCfm
# 90obf6fDni2uiuqm5qonFn1h95hxEbziUKFL5V365Q6nLJ+qZSDT2JboyHylTkhE
# /xniRAeSC9dohIBdanhkRc1gRn5UwRN8xXnxycFxAgMBAAGjgfowgfcwHQYDVR0O
# BBYEFF+a9W5czMx0mtTdfe8/2+xMgC7dMDIGCCsGAQUFBwEBBCYwJDAiBggrBgEF
# BQcwAYYWaHR0cDovL29jc3AudGhhd3RlLmNvbTASBgNVHRMBAf8ECDAGAQH/AgEA
# MD8GA1UdHwQ4MDYwNKAyoDCGLmh0dHA6Ly9jcmwudGhhd3RlLmNvbS9UaGF3dGVU
# aW1lc3RhbXBpbmdDQS5jcmwwEwYDVR0lBAwwCgYIKwYBBQUHAwgwDgYDVR0PAQH/
# BAQDAgEGMCgGA1UdEQQhMB+kHTAbMRkwFwYDVQQDExBUaW1lU3RhbXAtMjA0OC0x
# MA0GCSqGSIb3DQEBBQUAA4GBAAMJm495739ZMKrvaLX64wkdu0+CBl03X6ZSnxaN
# 6hySCURu9W3rWHww6PlpjSNzCxJvR6muORH4KrGbsBrDjutZlgCtzgxNstAxpghc
# Knr84nodV0yoZRjpeUBiJZZux8c3aoMhCI5B6t3ZVz8dd0mHKhYGXqY4aiISo1EZ
# g362MIIEozCCA4ugAwIBAgIQDs/0OMj+vzVuBNhqmBsaUDANBgkqhkiG9w0BAQUF
# ADBeMQswCQYDVQQGEwJVUzEdMBsGA1UEChMUU3ltYW50ZWMgQ29ycG9yYXRpb24x
# MDAuBgNVBAMTJ1N5bWFudGVjIFRpbWUgU3RhbXBpbmcgU2VydmljZXMgQ0EgLSBH
# MjAeFw0xMjEwMTgwMDAwMDBaFw0yMDEyMjkyMzU5NTlaMGIxCzAJBgNVBAYTAlVT
# MR0wGwYDVQQKExRTeW1hbnRlYyBDb3Jwb3JhdGlvbjE0MDIGA1UEAxMrU3ltYW50
# ZWMgVGltZSBTdGFtcGluZyBTZXJ2aWNlcyBTaWduZXIgLSBHNDCCASIwDQYJKoZI
# hvcNAQEBBQADggEPADCCAQoCggEBAKJjCzlEuLsjp0RJuw7/ofBhClOTsJjbrSwP
# SsVu/4Y8U1UPFc4EPyv9qZaW2b5heQtbyUyGduXgQ0sile7CK0PBn9hotI5AT+6F
# OLkRxSPyZFjwFTJvTlehroikAtcqHs1L4d1j1ReJMluwXplaqJ0oUA4X7pbbYTtF
# UR3PElYLkkf8q672Zj1HrHBy55LnX80QucSDZJQZvSWA4ejSIqXQugJ6oXeTW2XD
# 7hd0vEGGKtwITIySjJEtnndEH2jWqHR32w5bMotWizO92WPISZ06xcXqMwvS8aMb
# 9Iu+2bNXizveBKd6IrIkri7HcMW+ToMmCPsLvalPmQjhEChyqs0CAwEAAaOCAVcw
# ggFTMAwGA1UdEwEB/wQCMAAwFgYDVR0lAQH/BAwwCgYIKwYBBQUHAwgwDgYDVR0P
# AQH/BAQDAgeAMHMGCCsGAQUFBwEBBGcwZTAqBggrBgEFBQcwAYYeaHR0cDovL3Rz
# LW9jc3Aud3Muc3ltYW50ZWMuY29tMDcGCCsGAQUFBzAChitodHRwOi8vdHMtYWlh
# LndzLnN5bWFudGVjLmNvbS90c3MtY2EtZzIuY2VyMDwGA1UdHwQ1MDMwMaAvoC2G
# K2h0dHA6Ly90cy1jcmwud3Muc3ltYW50ZWMuY29tL3Rzcy1jYS1nMi5jcmwwKAYD
# VR0RBCEwH6QdMBsxGTAXBgNVBAMTEFRpbWVTdGFtcC0yMDQ4LTIwHQYDVR0OBBYE
# FEbGaaMOShQe1UzaUmMXP142vA3mMB8GA1UdIwQYMBaAFF+a9W5czMx0mtTdfe8/
# 2+xMgC7dMA0GCSqGSIb3DQEBBQUAA4IBAQB4O7SRKgBM8I9iMDd4o4QnB28Yst4l
# 3KDUlAOqhk4ln5pAAxzdzuN5yyFoBtq2MrRtv/QsJmMz5ElkbQ3mw2cO9wWkNWx8
# iRbG6bLfsundIMZxD82VdNy2XN69Nx9DeOZ4tc0oBCCjqvFLxIgpkQ6A0RH83Vx2
# bk9eDkVGQW4NsOo4mrE62glxEPwcebSAe6xp9P2ctgwWK/F/Wwk9m1viFsoTgW0A
# LjgNqCmPLOGy9FqpAa8VnCwvSRvbIrvD/niUUcOGsYKIXfA9tFGheTMrLnu53CAJ
# E3Hrahlbz+ilMFcsiUk/uc9/yb8+ImhjU5q9aXSsxR08f5Lgw7wc2AR1MIIFIjCC
# BAqgAwIBAgIQBuAUqkL8vKnzQr5Tqe4gIjANBgkqhkiG9w0BAQsFADByMQswCQYD
# VQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGln
# aWNlcnQuY29tMTEwLwYDVQQDEyhEaWdpQ2VydCBTSEEyIEFzc3VyZWQgSUQgQ29k
# ZSBTaWduaW5nIENBMB4XDTE2MDQxNDAwMDAwMFoXDTE5MDYxOTEyMDAwMFowXzEL
# MAkGA1UEBhMCVVMxEzARBgNVBAgTCkNhbGlmb3JuaWExDzANBgNVBAcTBklydmlu
# ZTEUMBIGA1UEChMLS29mYXgsIEluYy4xFDASBgNVBAMTC0tvZmF4LCBJbmMuMIIB
# IjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAp1sCHr59YU9R9Hsy1loUNPur
# VuF0bcL0/9usV+BHWYSM1jAWwsnrzKmss7dVu4Ld6XeB+vjE7lFT4SNPNPbZguiS
# 1jVKD8Kk83JszhHIRIf6fecSChF56Nj9mwHz9N3j9QmeBQhxUaoBZSrbOsW+g25U
# HxLQtX4Uc7xZ+BVWyqWs31ZcENgz1rWM8FID4TBZoH6Kp23EGVAL06vgcBUegsB5
# XwIf1m2iPI02akjuB/ib5lrTufmYGJkRCyALh+I25QVaXXuSZ5um3EMdUfkG2OGh
# 5g/rd7CPz7H8j1UXJu/wgDxOsNY/vCDUP6SyWBKFyBSxQBSZzBku0Xio/aPLvQID
# AQABo4IBxTCCAcEwHwYDVR0jBBgwFoAUWsS5eyoKo6XqcQPAYPkt9mV1DlgwHQYD
# VR0OBBYEFNgH2nzkwJngFB8kbXfVxmrR9zf8MA4GA1UdDwEB/wQEAwIHgDATBgNV
# HSUEDDAKBggrBgEFBQcDAzB3BgNVHR8EcDBuMDWgM6Axhi9odHRwOi8vY3JsMy5k
# aWdpY2VydC5jb20vc2hhMi1hc3N1cmVkLWNzLWcxLmNybDA1oDOgMYYvaHR0cDov
# L2NybDQuZGlnaWNlcnQuY29tL3NoYTItYXNzdXJlZC1jcy1nMS5jcmwwTAYDVR0g
# BEUwQzA3BglghkgBhv1sAwEwKjAoBggrBgEFBQcCARYcaHR0cHM6Ly93d3cuZGln
# aWNlcnQuY29tL0NQUzAIBgZngQwBBAEwgYQGCCsGAQUFBwEBBHgwdjAkBggrBgEF
# BQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29tME4GCCsGAQUFBzAChkJodHRw
# Oi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRTSEEyQXNzdXJlZElEQ29k
# ZVNpZ25pbmdDQS5jcnQwDAYDVR0TAQH/BAIwADANBgkqhkiG9w0BAQsFAAOCAQEA
# 5Pxp9n7UG1jGtY6fI0DQ6HdIzewX7ffaahn1PaBLmRzk/Bo6qhz+FyGqB4Yalnpv
# OWZxqjKtXh7+sbZn7p59ypS55lBk1A6DBNSooOvFaEF/+edq7fO4PJZeNXAeT0zn
# TxQsfr2HaVtdThasWH86RTIBRIDHqbLeL8n1CVxjuxcBh6xRWlqNWlslwniixEVe
# jEaB4z/xkXT4lvNA7Eav7J2D8stRkSBfefwVpgCNDjf6oh5R8pEPYJR4Xo3YSleK
# sTyZgZCdUtsXzFOwVMOcjqhFBpZTO1PZj7IsJbWR67VmCB5rrWs059XS8ui2SCPA
# REKUxt2Yk9sguJqS4ph9SzCCBTAwggQYoAMCAQICEAQJGBtf1btmdVNDtW+VUAgw
# DQYJKoZIhvcNAQELBQAwZTELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0
# IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEkMCIGA1UEAxMbRGlnaUNl
# cnQgQXNzdXJlZCBJRCBSb290IENBMB4XDTEzMTAyMjEyMDAwMFoXDTI4MTAyMjEy
# MDAwMFowcjELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcG
# A1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTExMC8GA1UEAxMoRGlnaUNlcnQgU0hBMiBB
# c3N1cmVkIElEIENvZGUgU2lnbmluZyBDQTCCASIwDQYJKoZIhvcNAQEBBQADggEP
# ADCCAQoCggEBAPjTsxx/DhGvZ3cH0wsxSRnP0PtFmbE620T1f+Wondsy13Hqdp0F
# LreP+pJDwKX5idQ3Gde2qvCchqXYJawOeSg6funRZ9PG+yknx9N7I5TkkSOWkHeC
# +aGEI2YSVDNQdLEoJrskacLCUvIUZ4qJRdQtoaPpiCwgla4cSocI3wz14k1gGL6q
# xLKucDFmM3E+rHCiq85/6XzLkqHlOzEcz+ryCuRXu0q16XTmK/5sy350OTYNkO/k
# tU6kqepqCquE86xnTrXE94zRICUj6whkPlKWwfIPEvTFjg/BougsUfdzvL2FsWKD
# c0GCB+Q4i2pzINAPZHM8np+mM6n9Gd8lk9ECAwEAAaOCAc0wggHJMBIGA1UdEwEB
# /wQIMAYBAf8CAQAwDgYDVR0PAQH/BAQDAgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMD
# MHkGCCsGAQUFBwEBBG0wazAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNl
# cnQuY29tMEMGCCsGAQUFBzAChjdodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20v
# RGlnaUNlcnRBc3N1cmVkSURSb290Q0EuY3J0MIGBBgNVHR8EejB4MDqgOKA2hjRo
# dHRwOi8vY3JsNC5kaWdpY2VydC5jb20vRGlnaUNlcnRBc3N1cmVkSURSb290Q0Eu
# Y3JsMDqgOKA2hjRodHRwOi8vY3JsMy5kaWdpY2VydC5jb20vRGlnaUNlcnRBc3N1
# cmVkSURSb290Q0EuY3JsME8GA1UdIARIMEYwOAYKYIZIAYb9bAACBDAqMCgGCCsG
# AQUFBwIBFhxodHRwczovL3d3dy5kaWdpY2VydC5jb20vQ1BTMAoGCGCGSAGG/WwD
# MB0GA1UdDgQWBBRaxLl7KgqjpepxA8Bg+S32ZXUOWDAfBgNVHSMEGDAWgBRF66Kv
# 9JLLgjEtUYunpyGd823IDzANBgkqhkiG9w0BAQsFAAOCAQEAPuwNWiSz8yLRFcgs
# fCUpdqgdXRwtOhrE7zBh134LYP3DPQ/Er4v97yrfIFU3sOH20ZJ1D1G0bqWOWuJe
# JIFOEKTuP3GOYw4TS63XX0R58zYUBor3nEZOXP+QsRsHDpEV+7qvtVHCjSSuJMbH
# JyqhKSgaOnEoAjwukaPAJRHinBRHoXpoaK+bp1wgXNlxsQyPu6j4xRJon89Ay0BE
# pRPw5mQMJQhCMrI2iiQC/i9yfhzXSUWW6Fkd6fp0ZGuy62ZD2rOwjNXpDd32ASDO
# mTFjPQgaGLOBm0/GkxAG/AeB+ova+YJJ92JuoVP6EpQYhS6SkepobEQysmah5xik
# mmRR7zGCBDcwggQzAgEBMIGGMHIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdp
# Q2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xMTAvBgNVBAMTKERp
# Z2lDZXJ0IFNIQTIgQXNzdXJlZCBJRCBDb2RlIFNpZ25pbmcgQ0ECEAbgFKpC/Lyp
# 80K+U6nuICIwCQYFKw4DAhoFAKB4MBgGCisGAQQBgjcCAQwxCjAIoAKAAKECgAAw
# GQYJKoZIhvcNAQkDMQwGCisGAQQBgjcCAQQwHAYKKwYBBAGCNwIBCzEOMAwGCisG
# AQQBgjcCARUwIwYJKoZIhvcNAQkEMRYEFOm4Tof7C2Z1mL6axYg/+ciq8PFXMA0G
# CSqGSIb3DQEBAQUABIIBAI6lP1jot/Xisxc6/JzNv6sQIDpdDoyEBS8klBiT1F/O
# XbLH/zNnMiS2rJfh9Ji8UWn2VRf2omG9BbByQ/+/ll/a17FfLuH9DO5uR57yjxpg
# yN0XZ5t0mARafrDpKIpd2BqCNFRbNb4euP0AZHBkoDdLgcMx/U8tCN8BUm0ZW395
# KiG3bKRB+A+5VZSDo/wrKphPu2ZjOSWJRupIaX+q1hTVN4tKj1KsK8vS68QPgkRW
# RWgnbRgCuDUazLT/EouFYSoiPWuuGUIeyWAmyHidCfuvJnZggj/y+3QGkPEGufzx
# DEKcI+sNFRmYcEbVyr0+cugPP4DyJc8E2njiSS+3kwmhggILMIICBwYJKoZIhvcN
# AQkGMYIB+DCCAfQCAQEwcjBeMQswCQYDVQQGEwJVUzEdMBsGA1UEChMUU3ltYW50
# ZWMgQ29ycG9yYXRpb24xMDAuBgNVBAMTJ1N5bWFudGVjIFRpbWUgU3RhbXBpbmcg
# U2VydmljZXMgQ0EgLSBHMgIQDs/0OMj+vzVuBNhqmBsaUDAJBgUrDgMCGgUAoF0w
# GAYJKoZIhvcNAQkDMQsGCSqGSIb3DQEHATAcBgkqhkiG9w0BCQUxDxcNMTkwMjE0
# MTU1MzQ5WjAjBgkqhkiG9w0BCQQxFgQUabOuIPwn/R/BNe7YpSRhKZi96U0wDQYJ
# KoZIhvcNAQEBBQAEggEAGxgWZDMrcjeFEh649gj/y4AiDZqfAsg9z0puyVmAtebn
# 5ScBddgB5cYIGcmFCcQYZgmdiyrZ+57fgQ2LXdayseneyf23IDgNyNRHK7KveyJv
# Z7GR71QXVRCFMi1WdvUlGUlPg0Axi4oZDXg1RidkDhJxA/VAv00/xsQKmA0KgH8U
# ooHvANDF2opIPinVUWMm66qZVR1MLCTY4avZnh6BLF35zGzavEa4IpMbPUezYrJe
# 7U+rvx8ZiOHGWnz/Ja9AmM/7rV5Vvsac5QVR5KucGvCPyqFrpBOqDzNKS8h78d3G
# WtEInn2HVU8wQPIouFjXDcKpl1TMzjlhZfyt1MjFgQ==
# SIG # End signature block
