################################################################################
# TODO: move the following code to a module.
################################################################################

# =============================================================================================================
# Reporting functions
# =============================================================================================================

function Write-Msg ($prefix, $FG, $BG, $msg) {
    <#
    .Synopsis
        Write message to host.
    .Description
        Write message in specified colours to host. (Internal use only.)
    #>
    $oldFG = $Host.UI.RawUI.ForegroundColor
    $oldBG = $Host.UI.RawUI.BackgroundColor
    $Host.UI.RawUI.ForegroundColor = $FG
    $Host.UI.RawUI.BackgroundColor = $BG
    Write-Host "$prefix : $msg"
    $Host.UI.RawUI.ForegroundColor = $oldFG
    $Host.UI.RawUI.BackgroundColor = $oldBG
}

function Write-Warn ($msg) {
    <#
    .Synopsis
        Write Warning message to host.
    .Description
        Write Warning message, red on white, to host
    #>
    Write-Msg 'WARNING' 'yellow' 'black' $msg
}

function Write-Err ($msg) {
    <#
    .Synopsis
        Write Error message to host.
    .Description
        Write Error message, red on black, to host
    #>
    Write-Msg 'ERROR' 'red' 'black' $msg
}

function Check-Argument ($name, $value) {
    <#
    .Synopsis
        Check an argument string on empty.
    .Description
        Check that an argument string is not empty.
        If empty terminate the process with exit code 3.
        Try to stop transcrip before exit.
    #>
    if ($value.Length -eq 0) {
        Write-Error "$Target : Missing argument: $name"
        Stop-HostTranscript
        exit 3
    }
}

function Write-StdErr {
    <#
    .SYNOPSIS
        Writes text to stderr when running in the regular PS console,
        to the host's error stream otherwise.

    .DESCRIPTION
        Writing to true stderr allows you to write a well-behaved CLI
        as a PS script that can be invoked from a batch file, for instance.

        Note that PS sends ALL its streams to *stdout* when invoked from cmd.exe.

        This function acts similarly to Write-Host in that it simply calls
        .ToString() on its input; to get the default output format, invoke
        it via a pipeline and precede with Out-String.
        
        More background:
        It seems that PS (observed on v3) sends all its streams to stdout
        when invoked from cmd.exe. Internally, PS appears to have at least 3 streams:
        output, error, and warning (try Write-Warning "I'll go unheard." 3> $null).

    .SEE
        http://stackoverflow.com/questions/4998173/how-do-i-write-to-standard-error-in-powershell
    #> 
    param ([PSObject] $InputObject)
    $outFunc = if ($Host.Name -eq 'ConsoleHost') { 
        [Console]::Error.WriteLine
    } else {
        $host.ui.WriteErrorLine
    }
    if ($InputObject) {
        [void] $outFunc.Invoke($InputObject.ToString())
    } else {
        [string[]] $lines = @()
        $Input | % { $lines += $_.ToString() }
        [void] $outFunc.Invoke($lines -join "`r`n")
    }
}

Function Show-Result ($result) {
    if (($result -ne $null) -and ($result -ne '')) {
        switch ($result.GetType().FullName) {
            'System.Object[]' {
                foreach ($r in $result) { Show-Result $r }
            }
            'System.String' {
                Write-Host $result
            }
            default {
                #Write-Host $result.ToString()
            }
        }
    }
}

# =============================================================================================================
# Command line argument parsing.
# =============================================================================================================

function Log-Arguments ($executable, $ArgumentList) {
    Write-Host ''
    Write-Host "$($executable):"
    for ([int]$i = 0; $i -lt $ArgumentList.Count; $i++) {
        if (-not ($ArgumentList[$i].GetType().Name -eq 'String')) {
            continue;
        }
        $argument = $($ArgumentList[$i])
        $endOfKey = $argument.IndexOf("=")
        if ($endOfKey -gt 0) {
            $key = $argument.substring(0,$endOfKey)     
            If(-not (($($key).ToLower().Contains('password')))){
                Write-Host "    arg[$i] = '$($ArgumentList[$i])'"
            }
            else {
                Write-Host "    arg[$i] = '$key=********'"
            }
        } else {
            Write-Host "    arg[$i] = (Unknown parameter type)"
        }
    }
    Write-Host ''
}

function Log-Done ($executable) {
    Write-Host "$($executable): Done."
}

function ParseExtend-Arguments {
    <#
    .Synopsis
        Parse command line arguments and adds these to an existing configuration structure.
    .Description
        Convert command line arguments to a configuration structure 
        using the following grammar (where 'configuration' is an unspecified
        structure containing all configuration details).
        Return the configuration structure when ready.
        
        As EBNF:
        start       : { configuration.empty() } commandline ;
        commandline :|argument commandline ;
        argument    : opt_ITPpart Name opt_Value    { configuration.add($1, $2, $3) } ;
        opt_ITPpart :                               { $0 = "<<global>>" }
                    | ITPpart '!'                   { $0 = ITPpart }
                    ;
        opt_Value   :                               { $0 = "True" }
                    | '=' Value                     { $0 = $1
                    ;
        
        As an regex.
        # Problem: PS uses the same delimiters for groups as for catch.
        #          resulting in multiple catches, causing problems wit
        #          (...)? 'group' and (...) 'catch this part of the match'.
        configuration.empty()
        foreach (arg in args) {
            if (arg -match "^(([^!=]*)!)?([^!=]+)(=(.*))?$") {
                $part,$name,$value = $matches[1..3]
                configuration.add($part,$name,$value)
            }
        }
    #>
    [CmdletBinding()]
    Param(
        [Parameter(Mandatory=$true)][Hashtable]$Extend,
        [Parameter(Mandatory=$true)][AllowEmptyCollection()][string[]]$Argumentlist,
        [switch]$TrustOverrides
    )
    
    # Problem: PS uses the same delimiters for groups as for catch.
    #          resulting in multiple catches.
    # Solved by using a regular expression for each distinctive pattern.
    $re_name            = '^([^!=]+)$'
    $re_name_value      = '^([^!=]+)=(.*)$'
    $re_part_name       = '^([^!=]+)!([^!=]+)$'
    $re_part_name_value = '^([^!=]+)!([^!=]+)=(.*)$'
    
    [bool]$errors = $False
    foreach ($arg in $ArgumentList) {
        if ($arg -match $re_name) {
            $part = $CONFIGURATION_PART_GLOBAL
            $name = $matches[1]
            $value = $True
        } elseif ($arg -match $re_name_value) {
            $part = $CONFIGURATION_PART_GLOBAL
            $name = $matches[1]
            $value = $matches[2]
        } elseif ($arg -match $re_part_name) {
            $part = $matches[1]
            $name = $matches[2]
            $value = $True
        } elseif ($arg -match $re_part_name_value) {
            $part = $matches[1]
            $name = $matches[2]
            $value = $matches[3]
        } else {
            $part = ''
            $name = ''
            $value = ''
        }
        if ($value -eq $Null -or ($value.GetType().Name -eq 'String' -and $value.Length -eq 0)) {
            $errors = $True
            Write-Error "Argument is missing a value: '$arg'"
        }
        else {
            if ($part.Length -eq 0) { $part = $CONFIGURATION_PART_GLOBAL }
            if ($value -eq $Null -or ($value.GetType().Name -eq 'String' -and $value.Length -eq 0)) { $value = $True }
            
            if (Have-Key $Extend $part $name) {
                $old = Get-Key $Extend $part $name
                if (-not $old.Equals($value)) {
                    if ($TrustOverrides) {
                        Write-Host "Overwriting configuration key [$part][$name]='$old' with '$value'"
                    } else {
                        Write-Warning "Overwriting configuration key [$part][$name]='$old' with '$value'"
                    }
                }
            }
            
            $dummy = Set-Key $Extend $part $name $value $True
            #$val = $Extend[$part].Values[$name]
            #Write-Host "Accepted Extend[$part][$name]=$val"
        }
    }
    if ($Errors) {
        Write-Error 'Errors on the command line, abort installation.'
        Stop-HostTranscript
        Exit 1
    }
    
}

function Parse-Arguments {
    <#
    .Synopsis
        Parse command line arguments and return a configuration structure.
    .Description
        Convert command line arguments to a configuration structure 
        using the following grammar (where 'configuration' is an unspecified
        structure containing all configuration details).
        Return the configuration structure when ready.
        
        As EBNF:
        start       : { configuration.empty() } commandline ;
        commandline :|argument commandline ;
        argument    : opt_ITPpart Name opt_Value    { configuration.add($1, $2, $3) } ;
        opt_ITPpart :                               { $0 = "<<global>>" }
                    | ITPpart '!'                   { $0 = ITPpart }
                    ;
        opt_Value   :                               { $0 = "True" }
                    | '=' Value                     { $0 = $1
                    ;
        
        As an regex.
        # Problem: PS uses the same delimiters for groups as for catch.
        #          resulting in multiple catches, causing problems wit
        #          (...)? 'group' and (...) 'catch this part of the match'.
        configuration.empty()
        foreach (arg in args) {
            if (arg -match "^(([^!=]*)!)?([^!=]+)(=(.*))?$") {
                $part,$name,$value = $matches[1..3]
                configuration.add($part,$name,$value)
            }
        }
    #>
    [CmdletBinding()]
    Param(
        [Parameter(Mandatory=$true)][AllowEmptyCollection()][string[]]$Argumentlist
    )   
    
    [Hashtable]$Configuration = @{}

    ParseExtend-Arguments -Extend $Configuration -Argumentlist $Argumentlist
    
    return $Configuration
}

# =============================================================================================================
# String handling
# =============================================================================================================

Function Parse-Bool {
    param(
        [Parameter(Mandatory=$true)][string]$value
    )

    if (($value -eq 'Yes') -or ($value -eq 'Y') -or ($value -eq 'True') -or ($value -eq '1')) {
        $TRUE
    } elseif (($value -eq 'No') -or ($value -eq 'N') -or ($value -eq 'False') -or ($value -eq '0')) {
        $FALSE
    } else {
        Write-Host "ERROR: Unable to parse as boolean: $value"
    }
}

function Parse-IP {
    <#
    .Synopsis
        Trim and parse string containing an IP number.
    .Description
        Parameters:
            $value      String conaining the IP number.
            $digits     Reference to an int[4] array for the results. (optional)
            $silent     Report no errors if the switch is used.
            Returns     True if the IP is valid, false otherwise.
        
        Note on the $silent switch, how to use. It is *not* a boolean at call time.
        Using the switch, be silent:
        
            Parse-IP 't.u.v.w' ... -silent
        
        Not using the switch, be verbose:
        
            Parse-IP 't.u.v.w' ...
    #>
    param (
        [Parameter(Mandatory=$true)][string]$value,
        [ref]$digits = $null,
        [switch]$silent
    )
    
    [Regex]$re = New-Object -TypeName Regex('^\s*(?<d1>[0-9]{1,3})\.(?<d2>[0-9]{1,3})\.(?<d3>[0-9]{1,3})\.(?<d4>[0-9]{1,3})\s*$')
    $match = $re.Match($value)
    
    [bool]$result = $false
    if (-not ($match.Success)) {
        if (-not $silent) { Write-Warning "Parse IP: Not an IP number: $value" }
    } else {
        [string]$d = ''
        $d = $match.Groups['d1']; [int]$d1 = $d
        $d = $match.Groups['d2']; [int]$d2 = $d
        $d = $match.Groups['d3']; [int]$d3 = $d
        $d = $match.Groups['d4']; [int]$d4 = $d
        $result = (-not ((255 -lt $d1) -or (255 -lt $d2) -or (255 -lt $d3) -or (255 -lt $d4)))
        if ($result) {
            if ($digits -ne $null) { $digits.Value = $d1,$d2,$d3,$d4 }
        } else {
            if (-not $silent) { Write-Warning "Parse IP: Not a valid IP number: $value" }
        }
    }
    return $result
}

function TryParse-IP {
    param (
        [Parameter(Mandatory=$true)][string]$value,
        [switch]$silent
    )
    
    [int[]]$d = @( -1, -1, -1, -1 )
    [bool]$result = $false
    if ($silent) {
        $result = Parse-IP $value ([ref]$d) -silent
    } else {
        $result = Parse-IP $value ([ref]$d)
    }
    return $result

}

# =============================================================================================================
# Credentials
# =============================================================================================================

Function Create-Credentials {
    param (
        [Parameter(Mandatory=$true)][string]$User,
        [Parameter(Mandatory=$true)]$Password
    )
    return (New-Object System.Management.Automation.PSCredential($User, $Password))
}

# =============================================================================================================
# Product version.
#
# This will read the contents of a file in the installation dir of a particular product that contains a 'clean'
# product version number of that product. This version number will be returned 'as is'. This file is typically
# named versionnr.txt
# =============================================================================================================

Function GetProductVersion($filePath) {
    if (Test-Path $filePath) {
        [string]$version = (Get-Content $filePath)
        
        if ($version.Length -eq 0)
        {
            Write-Error "Could not read the product version from file '$filePath'. File is empty."
            Exit 1        
        }
        
        $version = $version.Trim()
        
        if ($version.Length -eq 0)
        {
            Write-Error "Could not read the product version from file '$filePath'. File does not contain a version."
            Exit 1        
        }

        return $version
    } else {
        Write-Error "Could not read the product version from file '$filePath'. File does not exist."
        Exit 1
    }
}

Function MergeXml {
    param (
        [Parameter(Mandatory=$true)][string]$File,
        [Parameter(Mandatory=$true)][string]$AdditionalFile
    )
    
    [xml]$File1 = Get-Content $File
    [xml]$File2 = Get-Content $AdditionalFile

    ForEach ($XmlNode in $File2.DocumentElement.ChildNodes) {
        $result = $File1.DocumentElement.AppendChild($File1.ImportNode($XmlNode, $true))
    }
    ($File1).Save($File)
}

Function HasLogonAsService {
    param (
        [Parameter(Mandatory=$true)][string]$AccountName
    )
    [string]$AccountNameToCheck = $AccountName
    
    if($AccountName.StartsWith('.\')) {
        $AccountNameToCheck = $AccountName.substring(2)
    }
                
    Add-Type -Path '.\Scripts\Tools\LSAController.cs'

        return [LSAController.LSAWrapper]::HasLogonAsService($AccountNameToCheck)
}

# SIG # Begin signature block
# MIIbewYJKoZIhvcNAQcCoIIbbDCCG2gCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQU7UffSJwqy2DV+1Jgc4UO17wg
# IOqgghauMIIDtzCCAp+gAwIBAgIQDOfg5RfYRv6P5WD8G/AwOTANBgkqhkiG9w0B
# AQUFADBlMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2VydCBBc3N1cmVk
# IElEIFJvb3QgQ0EwHhcNMDYxMTEwMDAwMDAwWhcNMzExMTEwMDAwMDAwWjBlMQsw
# CQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cu
# ZGlnaWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2VydCBBc3N1cmVkIElEIFJvb3Qg
# Q0EwggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQCtDhXO5EOAXLGH87dg
# +XESpa7cJpSIqvTO9SA5KFhgDPiA2qkVlTJhPLWxKISKityfCgyDF3qPkKyK53lT
# XDGEKvYPmDI2dsze3Tyoou9q+yHyUmHfnyDXH+Kx2f4YZNISW1/5WBg1vEfNoTb5
# a3/UsDg+wRvDjDPZ2C8Y/igPs6eD1sNuRMBhNZYW/lmci3Zt1/GiSw0r/wty2p5g
# 0I6QNcZ4VYcgoc/lbQrISXwxmDNsIumH0DJaoroTghHtORedmTpyoeb6pNnVFzF1
# roV9Iq4/AUaG9ih5yLHa5FcXxH4cDrC0kqZWs72yl+2qp/C3xag/lRbQ/6GW6whf
# GHdPAgMBAAGjYzBhMA4GA1UdDwEB/wQEAwIBhjAPBgNVHRMBAf8EBTADAQH/MB0G
# A1UdDgQWBBRF66Kv9JLLgjEtUYunpyGd823IDzAfBgNVHSMEGDAWgBRF66Kv9JLL
# gjEtUYunpyGd823IDzANBgkqhkiG9w0BAQUFAAOCAQEAog683+Lt8ONyc3pklL/3
# cmbYMuRCdWKuh+vy1dneVrOfzM4UKLkNl2BcEkxY5NM9g0lFWJc1aRqoR+pWxnmr
# EthngYTffwk8lOa4JiwgvT2zKIn3X/8i4peEH+ll74fg38FnSbNd67IJKusm7Xi+
# fT8r87cmNW1fiQG2SVufAQWbqz0lwcy2f8Lxb4bG+mRo64EtlOtCt/qMHt1i8b5Q
# Z7dsvfPxH2sMNgcWfzd8qVttevESRmCD1ycEvkvOl77DZypoEd+A5wwzZr8TDRRu
# 838fYxAe+o0bJW1sj6W3YQGx0qMmoRBxna3iw/nDmVG3KwcIzi7mULKn+gpFL6Lw
# 8jCCA+4wggNXoAMCAQICEH6T6/t8xk5Z6kuad9QG/DswDQYJKoZIhvcNAQEFBQAw
# gYsxCzAJBgNVBAYTAlpBMRUwEwYDVQQIEwxXZXN0ZXJuIENhcGUxFDASBgNVBAcT
# C0R1cmJhbnZpbGxlMQ8wDQYDVQQKEwZUaGF3dGUxHTAbBgNVBAsTFFRoYXd0ZSBD
# ZXJ0aWZpY2F0aW9uMR8wHQYDVQQDExZUaGF3dGUgVGltZXN0YW1waW5nIENBMB4X
# DTEyMTIyMTAwMDAwMFoXDTIwMTIzMDIzNTk1OVowXjELMAkGA1UEBhMCVVMxHTAb
# BgNVBAoTFFN5bWFudGVjIENvcnBvcmF0aW9uMTAwLgYDVQQDEydTeW1hbnRlYyBU
# aW1lIFN0YW1waW5nIFNlcnZpY2VzIENBIC0gRzIwggEiMA0GCSqGSIb3DQEBAQUA
# A4IBDwAwggEKAoIBAQCxrLNJVEuXHBIK2CV5kSJXKm/cuCbEQ3Nrwr8uUFr7FMJ2
# jkMBJUO0oeJF9Oi3e8N0zCLXtJQAAvdN7b+0t0Qka81fRTvRRM5DEnMXgotptCvL
# mR6schsmTXEfsTHd+1FhAlOmqvVJLAV4RaUvic7nmef+jOJXPz3GktxK+Hsz5HkK
# +/B1iEGc/8UDUZmq12yfk2mHZSmDhcJgFMTIyTsU2sCB8B8NdN6SIqvK9/t0fCfm
# 90obf6fDni2uiuqm5qonFn1h95hxEbziUKFL5V365Q6nLJ+qZSDT2JboyHylTkhE
# /xniRAeSC9dohIBdanhkRc1gRn5UwRN8xXnxycFxAgMBAAGjgfowgfcwHQYDVR0O
# BBYEFF+a9W5czMx0mtTdfe8/2+xMgC7dMDIGCCsGAQUFBwEBBCYwJDAiBggrBgEF
# BQcwAYYWaHR0cDovL29jc3AudGhhd3RlLmNvbTASBgNVHRMBAf8ECDAGAQH/AgEA
# MD8GA1UdHwQ4MDYwNKAyoDCGLmh0dHA6Ly9jcmwudGhhd3RlLmNvbS9UaGF3dGVU
# aW1lc3RhbXBpbmdDQS5jcmwwEwYDVR0lBAwwCgYIKwYBBQUHAwgwDgYDVR0PAQH/
# BAQDAgEGMCgGA1UdEQQhMB+kHTAbMRkwFwYDVQQDExBUaW1lU3RhbXAtMjA0OC0x
# MA0GCSqGSIb3DQEBBQUAA4GBAAMJm495739ZMKrvaLX64wkdu0+CBl03X6ZSnxaN
# 6hySCURu9W3rWHww6PlpjSNzCxJvR6muORH4KrGbsBrDjutZlgCtzgxNstAxpghc
# Knr84nodV0yoZRjpeUBiJZZux8c3aoMhCI5B6t3ZVz8dd0mHKhYGXqY4aiISo1EZ
# g362MIIEozCCA4ugAwIBAgIQDs/0OMj+vzVuBNhqmBsaUDANBgkqhkiG9w0BAQUF
# ADBeMQswCQYDVQQGEwJVUzEdMBsGA1UEChMUU3ltYW50ZWMgQ29ycG9yYXRpb24x
# MDAuBgNVBAMTJ1N5bWFudGVjIFRpbWUgU3RhbXBpbmcgU2VydmljZXMgQ0EgLSBH
# MjAeFw0xMjEwMTgwMDAwMDBaFw0yMDEyMjkyMzU5NTlaMGIxCzAJBgNVBAYTAlVT
# MR0wGwYDVQQKExRTeW1hbnRlYyBDb3Jwb3JhdGlvbjE0MDIGA1UEAxMrU3ltYW50
# ZWMgVGltZSBTdGFtcGluZyBTZXJ2aWNlcyBTaWduZXIgLSBHNDCCASIwDQYJKoZI
# hvcNAQEBBQADggEPADCCAQoCggEBAKJjCzlEuLsjp0RJuw7/ofBhClOTsJjbrSwP
# SsVu/4Y8U1UPFc4EPyv9qZaW2b5heQtbyUyGduXgQ0sile7CK0PBn9hotI5AT+6F
# OLkRxSPyZFjwFTJvTlehroikAtcqHs1L4d1j1ReJMluwXplaqJ0oUA4X7pbbYTtF
# UR3PElYLkkf8q672Zj1HrHBy55LnX80QucSDZJQZvSWA4ejSIqXQugJ6oXeTW2XD
# 7hd0vEGGKtwITIySjJEtnndEH2jWqHR32w5bMotWizO92WPISZ06xcXqMwvS8aMb
# 9Iu+2bNXizveBKd6IrIkri7HcMW+ToMmCPsLvalPmQjhEChyqs0CAwEAAaOCAVcw
# ggFTMAwGA1UdEwEB/wQCMAAwFgYDVR0lAQH/BAwwCgYIKwYBBQUHAwgwDgYDVR0P
# AQH/BAQDAgeAMHMGCCsGAQUFBwEBBGcwZTAqBggrBgEFBQcwAYYeaHR0cDovL3Rz
# LW9jc3Aud3Muc3ltYW50ZWMuY29tMDcGCCsGAQUFBzAChitodHRwOi8vdHMtYWlh
# LndzLnN5bWFudGVjLmNvbS90c3MtY2EtZzIuY2VyMDwGA1UdHwQ1MDMwMaAvoC2G
# K2h0dHA6Ly90cy1jcmwud3Muc3ltYW50ZWMuY29tL3Rzcy1jYS1nMi5jcmwwKAYD
# VR0RBCEwH6QdMBsxGTAXBgNVBAMTEFRpbWVTdGFtcC0yMDQ4LTIwHQYDVR0OBBYE
# FEbGaaMOShQe1UzaUmMXP142vA3mMB8GA1UdIwQYMBaAFF+a9W5czMx0mtTdfe8/
# 2+xMgC7dMA0GCSqGSIb3DQEBBQUAA4IBAQB4O7SRKgBM8I9iMDd4o4QnB28Yst4l
# 3KDUlAOqhk4ln5pAAxzdzuN5yyFoBtq2MrRtv/QsJmMz5ElkbQ3mw2cO9wWkNWx8
# iRbG6bLfsundIMZxD82VdNy2XN69Nx9DeOZ4tc0oBCCjqvFLxIgpkQ6A0RH83Vx2
# bk9eDkVGQW4NsOo4mrE62glxEPwcebSAe6xp9P2ctgwWK/F/Wwk9m1viFsoTgW0A
# LjgNqCmPLOGy9FqpAa8VnCwvSRvbIrvD/niUUcOGsYKIXfA9tFGheTMrLnu53CAJ
# E3Hrahlbz+ilMFcsiUk/uc9/yb8+ImhjU5q9aXSsxR08f5Lgw7wc2AR1MIIFIjCC
# BAqgAwIBAgIQBuAUqkL8vKnzQr5Tqe4gIjANBgkqhkiG9w0BAQsFADByMQswCQYD
# VQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGln
# aWNlcnQuY29tMTEwLwYDVQQDEyhEaWdpQ2VydCBTSEEyIEFzc3VyZWQgSUQgQ29k
# ZSBTaWduaW5nIENBMB4XDTE2MDQxNDAwMDAwMFoXDTE5MDYxOTEyMDAwMFowXzEL
# MAkGA1UEBhMCVVMxEzARBgNVBAgTCkNhbGlmb3JuaWExDzANBgNVBAcTBklydmlu
# ZTEUMBIGA1UEChMLS29mYXgsIEluYy4xFDASBgNVBAMTC0tvZmF4LCBJbmMuMIIB
# IjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAp1sCHr59YU9R9Hsy1loUNPur
# VuF0bcL0/9usV+BHWYSM1jAWwsnrzKmss7dVu4Ld6XeB+vjE7lFT4SNPNPbZguiS
# 1jVKD8Kk83JszhHIRIf6fecSChF56Nj9mwHz9N3j9QmeBQhxUaoBZSrbOsW+g25U
# HxLQtX4Uc7xZ+BVWyqWs31ZcENgz1rWM8FID4TBZoH6Kp23EGVAL06vgcBUegsB5
# XwIf1m2iPI02akjuB/ib5lrTufmYGJkRCyALh+I25QVaXXuSZ5um3EMdUfkG2OGh
# 5g/rd7CPz7H8j1UXJu/wgDxOsNY/vCDUP6SyWBKFyBSxQBSZzBku0Xio/aPLvQID
# AQABo4IBxTCCAcEwHwYDVR0jBBgwFoAUWsS5eyoKo6XqcQPAYPkt9mV1DlgwHQYD
# VR0OBBYEFNgH2nzkwJngFB8kbXfVxmrR9zf8MA4GA1UdDwEB/wQEAwIHgDATBgNV
# HSUEDDAKBggrBgEFBQcDAzB3BgNVHR8EcDBuMDWgM6Axhi9odHRwOi8vY3JsMy5k
# aWdpY2VydC5jb20vc2hhMi1hc3N1cmVkLWNzLWcxLmNybDA1oDOgMYYvaHR0cDov
# L2NybDQuZGlnaWNlcnQuY29tL3NoYTItYXNzdXJlZC1jcy1nMS5jcmwwTAYDVR0g
# BEUwQzA3BglghkgBhv1sAwEwKjAoBggrBgEFBQcCARYcaHR0cHM6Ly93d3cuZGln
# aWNlcnQuY29tL0NQUzAIBgZngQwBBAEwgYQGCCsGAQUFBwEBBHgwdjAkBggrBgEF
# BQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29tME4GCCsGAQUFBzAChkJodHRw
# Oi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRTSEEyQXNzdXJlZElEQ29k
# ZVNpZ25pbmdDQS5jcnQwDAYDVR0TAQH/BAIwADANBgkqhkiG9w0BAQsFAAOCAQEA
# 5Pxp9n7UG1jGtY6fI0DQ6HdIzewX7ffaahn1PaBLmRzk/Bo6qhz+FyGqB4Yalnpv
# OWZxqjKtXh7+sbZn7p59ypS55lBk1A6DBNSooOvFaEF/+edq7fO4PJZeNXAeT0zn
# TxQsfr2HaVtdThasWH86RTIBRIDHqbLeL8n1CVxjuxcBh6xRWlqNWlslwniixEVe
# jEaB4z/xkXT4lvNA7Eav7J2D8stRkSBfefwVpgCNDjf6oh5R8pEPYJR4Xo3YSleK
# sTyZgZCdUtsXzFOwVMOcjqhFBpZTO1PZj7IsJbWR67VmCB5rrWs059XS8ui2SCPA
# REKUxt2Yk9sguJqS4ph9SzCCBTAwggQYoAMCAQICEAQJGBtf1btmdVNDtW+VUAgw
# DQYJKoZIhvcNAQELBQAwZTELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0
# IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEkMCIGA1UEAxMbRGlnaUNl
# cnQgQXNzdXJlZCBJRCBSb290IENBMB4XDTEzMTAyMjEyMDAwMFoXDTI4MTAyMjEy
# MDAwMFowcjELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcG
# A1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTExMC8GA1UEAxMoRGlnaUNlcnQgU0hBMiBB
# c3N1cmVkIElEIENvZGUgU2lnbmluZyBDQTCCASIwDQYJKoZIhvcNAQEBBQADggEP
# ADCCAQoCggEBAPjTsxx/DhGvZ3cH0wsxSRnP0PtFmbE620T1f+Wondsy13Hqdp0F
# LreP+pJDwKX5idQ3Gde2qvCchqXYJawOeSg6funRZ9PG+yknx9N7I5TkkSOWkHeC
# +aGEI2YSVDNQdLEoJrskacLCUvIUZ4qJRdQtoaPpiCwgla4cSocI3wz14k1gGL6q
# xLKucDFmM3E+rHCiq85/6XzLkqHlOzEcz+ryCuRXu0q16XTmK/5sy350OTYNkO/k
# tU6kqepqCquE86xnTrXE94zRICUj6whkPlKWwfIPEvTFjg/BougsUfdzvL2FsWKD
# c0GCB+Q4i2pzINAPZHM8np+mM6n9Gd8lk9ECAwEAAaOCAc0wggHJMBIGA1UdEwEB
# /wQIMAYBAf8CAQAwDgYDVR0PAQH/BAQDAgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMD
# MHkGCCsGAQUFBwEBBG0wazAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNl
# cnQuY29tMEMGCCsGAQUFBzAChjdodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20v
# RGlnaUNlcnRBc3N1cmVkSURSb290Q0EuY3J0MIGBBgNVHR8EejB4MDqgOKA2hjRo
# dHRwOi8vY3JsNC5kaWdpY2VydC5jb20vRGlnaUNlcnRBc3N1cmVkSURSb290Q0Eu
# Y3JsMDqgOKA2hjRodHRwOi8vY3JsMy5kaWdpY2VydC5jb20vRGlnaUNlcnRBc3N1
# cmVkSURSb290Q0EuY3JsME8GA1UdIARIMEYwOAYKYIZIAYb9bAACBDAqMCgGCCsG
# AQUFBwIBFhxodHRwczovL3d3dy5kaWdpY2VydC5jb20vQ1BTMAoGCGCGSAGG/WwD
# MB0GA1UdDgQWBBRaxLl7KgqjpepxA8Bg+S32ZXUOWDAfBgNVHSMEGDAWgBRF66Kv
# 9JLLgjEtUYunpyGd823IDzANBgkqhkiG9w0BAQsFAAOCAQEAPuwNWiSz8yLRFcgs
# fCUpdqgdXRwtOhrE7zBh134LYP3DPQ/Er4v97yrfIFU3sOH20ZJ1D1G0bqWOWuJe
# JIFOEKTuP3GOYw4TS63XX0R58zYUBor3nEZOXP+QsRsHDpEV+7qvtVHCjSSuJMbH
# JyqhKSgaOnEoAjwukaPAJRHinBRHoXpoaK+bp1wgXNlxsQyPu6j4xRJon89Ay0BE
# pRPw5mQMJQhCMrI2iiQC/i9yfhzXSUWW6Fkd6fp0ZGuy62ZD2rOwjNXpDd32ASDO
# mTFjPQgaGLOBm0/GkxAG/AeB+ova+YJJ92JuoVP6EpQYhS6SkepobEQysmah5xik
# mmRR7zGCBDcwggQzAgEBMIGGMHIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdp
# Q2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xMTAvBgNVBAMTKERp
# Z2lDZXJ0IFNIQTIgQXNzdXJlZCBJRCBDb2RlIFNpZ25pbmcgQ0ECEAbgFKpC/Lyp
# 80K+U6nuICIwCQYFKw4DAhoFAKB4MBgGCisGAQQBgjcCAQwxCjAIoAKAAKECgAAw
# GQYJKoZIhvcNAQkDMQwGCisGAQQBgjcCAQQwHAYKKwYBBAGCNwIBCzEOMAwGCisG
# AQQBgjcCARUwIwYJKoZIhvcNAQkEMRYEFIy9waT3KVPEaturGfe0O6Mcz/ELMA0G
# CSqGSIb3DQEBAQUABIIBAKEcoIICq8wuPZ9YiRhvHliGB5tz1yG1qXcKAuvXcApK
# rEHq6T7AbxxP1ay1zV4P7iVooqGUBE3MXD+X45+q/Eqe1xsRHavMdmnjy3kImhhH
# c9A7M+CnNSPu52qp7czKJ/U2zGmbUcpoNocfThGCrxPISF39JuIWOh8ovb6sqUTD
# LEmG44AjC6x2T+ZR+1ygMPHKBkGovsJSPzwv+JIdKm1Bf0zOimjXrOJS8Si2p1w0
# NOXa55pqo8RmfxAtxtYNRAksfvGmRb+y08wcZbNWMRKIpZGi1LQcMRY3oFlp7vNB
# ZrZH7eL7o+XB/h6JP4BhSpl0XIlvvZnZ/UMHli9LEYGhggILMIICBwYJKoZIhvcN
# AQkGMYIB+DCCAfQCAQEwcjBeMQswCQYDVQQGEwJVUzEdMBsGA1UEChMUU3ltYW50
# ZWMgQ29ycG9yYXRpb24xMDAuBgNVBAMTJ1N5bWFudGVjIFRpbWUgU3RhbXBpbmcg
# U2VydmljZXMgQ0EgLSBHMgIQDs/0OMj+vzVuBNhqmBsaUDAJBgUrDgMCGgUAoF0w
# GAYJKoZIhvcNAQkDMQsGCSqGSIb3DQEHATAcBgkqhkiG9w0BCQUxDxcNMTkwMjE0
# MTU1MzQ4WjAjBgkqhkiG9w0BCQQxFgQULs+C/99JQK6Sy9HHUgm8/sqh6IYwDQYJ
# KoZIhvcNAQEBBQAEggEAGuET1AFiIBlYsB+imx69FSTCo09J7333Rw0EgzOwS+g4
# JKeY7JkVzOhd2u5gEL9Ar/5M+Clqcn53BDyUc06ragUL0j0dQ4pv3+1bdst2LYwu
# d3ziM72ZjVjT5b+lILr1S0MGRTwmQXauxJAix3Ify3r5zyIe47YOvIMTnmzL71fS
# 591+zAopFO3ji/QcOfkTARAxOrAtq/rC0QzuKBYeH3xsZo8A2Vqz7eFgnMDDAlgr
# lWBOrJcffNL+SofiXWFTGhXrBKYqGosjMm5E6oVlvGQbzuxeTmmwJuXHSa3jfCIf
# 88VyA7mm23UgJINo3ZruLEORUITzEd6y1pYfQ5tLAA==
# SIG # End signature block
