# ============================================================================================================================================================
# Fix pack uninstallation script
# ============================================================================================================================================================

$ErrorActionPreference = 'Continue'
$BaseVersion = '5.2'
$FixpackName = '5.2.0.1'
$RestoreVersion = '<%RESTOREVERSION%>'
$SystemToDowngrade = '<%SYSTEMTODOWNGRADE%>'

. '.\Tools\ExecuteTools.ps1'
. '.\Tools\Functions.ps1'
. '.\Tools\IniTools.ps1'
. '.\Tools\PatchIni.ps1'
. '.\Tools\tools.ps1'
. '.\Tools\TranscriptTools.ps1'
. '.\Tools\FixpackFunctions.ps1'

Set-StrictMode -Version (Get-StrictModeVersion)

$Now = Get-Date -Format 'yyyyMMdd-HHmmss'
Start-HostTranscript (Join-Path (Split-Path -Parent $MyInvocation.MyCommand.Path) "Uninstall-FixPack-$FixPackName ($Now).log") | Out-Null

Try {

    # ------------------------------------------------------------------------------------------------------------------------------------------------------------
    # Check if a CCM installation exists that can be downgraded
    # ------------------------------------------------------------------------------------------------------------------------------------------------------------

    if (Test-Path 'HKLM:\SOFTWARE\Wow6432Node') {
        $packageRegistryURI = 'HKLM:\SOFTWARE\Wow6432Node\Aia\Package'
    } else {
        $packageRegistryURI = 'HKLM:\SOFTWARE\Aia\Package'
    }
    
    $versionHive = Join-Path $packageRegistryURI $BaseVersion

    $fpVersion = Get-ItemProperty -Path $versionHive -Name 'Version' -ErrorAction SilentlyContinue
    if ([string]::IsNullOrEmpty($fpVersion)) {
        Write-Error 'No fixpack installed'
        Exit 1
    }
    if ($fpVersion.Version -ne $FixPackName) {
        Write-Error 'Fixpack $UninstallFixpackVersion is superseded or not installed'
        Exit 1
    }
    
    # ------------------------------------------------------------------------------------------------------------------------------------------------------------
    # Read deployment parameters of existing CCM installation
    # ------------------------------------------------------------------------------------------------------------------------------------------------------------

    $deploymentConfigLocation = (Get-ItemProperty -literalPath $versionHive).DeploymentConfiguration
    if ([string]::IsNullOrEmpty($deploymentConfigLocation) -or (-not (Test-Path $deploymentConfigLocation))) {
        Write-Error ('Deployment configuration not found at "'+ $deploymentConfigLocation +'"')
        Exit 2
    }

    [ref]$deployIni = @{}
    if (-not (Get-IniFile $deploymentConfigLocation $deployIni)) {
        Exit 2
    }

    [string]$sourceDir                          = Split-Path -Parent $MyInvocation.MyCommand.Path
    [string]$backupDir                          = Join-Path (Get-Key $deployIni 'ITP' 'ProgramsDir') "Uninstall-FixPack-$FixPackName"
    
    # ------------------------------------------------------------------------------------------------------------------------------------------------------------
    # Start rollback (and stop CCM services. Tomcat Services and Output Management Services need not be stopped)
    # ------------------------------------------------------------------------------------------------------------------------------------------------------------

    [bool]$RegistryDowngraded = $False
    Stop-CCM -Version $BaseVersion 
    Start-Sleep -s 5
    
    
    # ------------------------------------------------------------------------------------------------------------------------------------------------------------
    # Rollback the (global, i.e. non instance) files that were backed up
    # ------------------------------------------------------------------------------------------------------------------------------------------------------------
    
    . .\RestoreFiles.ps1

    # ------------------------------------------------------------------------------------------------------------------------------------------------------------
    # Downgrade registry
    # ------------------------------------------------------------------------------------------------------------------------------------------------------------
    
    if ($RestoreVersion -eq $BaseVersion) {
        # If all fixpacks are removed, also remove the (sub-)version entry from the registry
        Remove-ItemProperty -Path $versionHive -Name 'Version'
    } else {
        # Else rollback registry to previous (fixpack) version
        CreateRegistryValue -Key $versionHive -Name 'Version' -Value $RestoreVersion | Out-Null
    }

    [bool]$RegistryDowngraded = $True
    
    # ------------------------------------------------------------------------------------------------------------------------------------------------------------
    # Done - success
    # ------------------------------------------------------------------------------------------------------------------------------------------------------------
    
    # Do not start CCM services again. We do not know if they were stopped originally or stopped because of the fixpack install
    # Installers should start the services themselves to ensure that they are started when intended.
    Write-Host 'Fix pack uninstalled successfully. Please start CCM services manually.'

} Catch {
    if ($RegistryDowngraded) {
        Write-Host -foregroundcolor Red 'Error starting CCM services after uninstall. Please start the services manually. The uninstall script does not need to be run again.'
        Write-Error $Error[0]
    } else {
        Write-Host -foregroundcolor Red 'Error during uninstall before reverting registry to $RestoreVersion. Please fix problem and run uninstall script again.'
        Write-Error $Error[0]
    }
    exit 1
} Finally {
    Stop-Transcript | Out-Null
}
# SIG # Begin signature block
# MIIbewYJKoZIhvcNAQcCoIIbbDCCG2gCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUD/B/ckX3gF44s6fHDawM899o
# Wk6gghauMIIDtzCCAp+gAwIBAgIQDOfg5RfYRv6P5WD8G/AwOTANBgkqhkiG9w0B
# AQUFADBlMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2VydCBBc3N1cmVk
# IElEIFJvb3QgQ0EwHhcNMDYxMTEwMDAwMDAwWhcNMzExMTEwMDAwMDAwWjBlMQsw
# CQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cu
# ZGlnaWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2VydCBBc3N1cmVkIElEIFJvb3Qg
# Q0EwggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQCtDhXO5EOAXLGH87dg
# +XESpa7cJpSIqvTO9SA5KFhgDPiA2qkVlTJhPLWxKISKityfCgyDF3qPkKyK53lT
# XDGEKvYPmDI2dsze3Tyoou9q+yHyUmHfnyDXH+Kx2f4YZNISW1/5WBg1vEfNoTb5
# a3/UsDg+wRvDjDPZ2C8Y/igPs6eD1sNuRMBhNZYW/lmci3Zt1/GiSw0r/wty2p5g
# 0I6QNcZ4VYcgoc/lbQrISXwxmDNsIumH0DJaoroTghHtORedmTpyoeb6pNnVFzF1
# roV9Iq4/AUaG9ih5yLHa5FcXxH4cDrC0kqZWs72yl+2qp/C3xag/lRbQ/6GW6whf
# GHdPAgMBAAGjYzBhMA4GA1UdDwEB/wQEAwIBhjAPBgNVHRMBAf8EBTADAQH/MB0G
# A1UdDgQWBBRF66Kv9JLLgjEtUYunpyGd823IDzAfBgNVHSMEGDAWgBRF66Kv9JLL
# gjEtUYunpyGd823IDzANBgkqhkiG9w0BAQUFAAOCAQEAog683+Lt8ONyc3pklL/3
# cmbYMuRCdWKuh+vy1dneVrOfzM4UKLkNl2BcEkxY5NM9g0lFWJc1aRqoR+pWxnmr
# EthngYTffwk8lOa4JiwgvT2zKIn3X/8i4peEH+ll74fg38FnSbNd67IJKusm7Xi+
# fT8r87cmNW1fiQG2SVufAQWbqz0lwcy2f8Lxb4bG+mRo64EtlOtCt/qMHt1i8b5Q
# Z7dsvfPxH2sMNgcWfzd8qVttevESRmCD1ycEvkvOl77DZypoEd+A5wwzZr8TDRRu
# 838fYxAe+o0bJW1sj6W3YQGx0qMmoRBxna3iw/nDmVG3KwcIzi7mULKn+gpFL6Lw
# 8jCCA+4wggNXoAMCAQICEH6T6/t8xk5Z6kuad9QG/DswDQYJKoZIhvcNAQEFBQAw
# gYsxCzAJBgNVBAYTAlpBMRUwEwYDVQQIEwxXZXN0ZXJuIENhcGUxFDASBgNVBAcT
# C0R1cmJhbnZpbGxlMQ8wDQYDVQQKEwZUaGF3dGUxHTAbBgNVBAsTFFRoYXd0ZSBD
# ZXJ0aWZpY2F0aW9uMR8wHQYDVQQDExZUaGF3dGUgVGltZXN0YW1waW5nIENBMB4X
# DTEyMTIyMTAwMDAwMFoXDTIwMTIzMDIzNTk1OVowXjELMAkGA1UEBhMCVVMxHTAb
# BgNVBAoTFFN5bWFudGVjIENvcnBvcmF0aW9uMTAwLgYDVQQDEydTeW1hbnRlYyBU
# aW1lIFN0YW1waW5nIFNlcnZpY2VzIENBIC0gRzIwggEiMA0GCSqGSIb3DQEBAQUA
# A4IBDwAwggEKAoIBAQCxrLNJVEuXHBIK2CV5kSJXKm/cuCbEQ3Nrwr8uUFr7FMJ2
# jkMBJUO0oeJF9Oi3e8N0zCLXtJQAAvdN7b+0t0Qka81fRTvRRM5DEnMXgotptCvL
# mR6schsmTXEfsTHd+1FhAlOmqvVJLAV4RaUvic7nmef+jOJXPz3GktxK+Hsz5HkK
# +/B1iEGc/8UDUZmq12yfk2mHZSmDhcJgFMTIyTsU2sCB8B8NdN6SIqvK9/t0fCfm
# 90obf6fDni2uiuqm5qonFn1h95hxEbziUKFL5V365Q6nLJ+qZSDT2JboyHylTkhE
# /xniRAeSC9dohIBdanhkRc1gRn5UwRN8xXnxycFxAgMBAAGjgfowgfcwHQYDVR0O
# BBYEFF+a9W5czMx0mtTdfe8/2+xMgC7dMDIGCCsGAQUFBwEBBCYwJDAiBggrBgEF
# BQcwAYYWaHR0cDovL29jc3AudGhhd3RlLmNvbTASBgNVHRMBAf8ECDAGAQH/AgEA
# MD8GA1UdHwQ4MDYwNKAyoDCGLmh0dHA6Ly9jcmwudGhhd3RlLmNvbS9UaGF3dGVU
# aW1lc3RhbXBpbmdDQS5jcmwwEwYDVR0lBAwwCgYIKwYBBQUHAwgwDgYDVR0PAQH/
# BAQDAgEGMCgGA1UdEQQhMB+kHTAbMRkwFwYDVQQDExBUaW1lU3RhbXAtMjA0OC0x
# MA0GCSqGSIb3DQEBBQUAA4GBAAMJm495739ZMKrvaLX64wkdu0+CBl03X6ZSnxaN
# 6hySCURu9W3rWHww6PlpjSNzCxJvR6muORH4KrGbsBrDjutZlgCtzgxNstAxpghc
# Knr84nodV0yoZRjpeUBiJZZux8c3aoMhCI5B6t3ZVz8dd0mHKhYGXqY4aiISo1EZ
# g362MIIEozCCA4ugAwIBAgIQDs/0OMj+vzVuBNhqmBsaUDANBgkqhkiG9w0BAQUF
# ADBeMQswCQYDVQQGEwJVUzEdMBsGA1UEChMUU3ltYW50ZWMgQ29ycG9yYXRpb24x
# MDAuBgNVBAMTJ1N5bWFudGVjIFRpbWUgU3RhbXBpbmcgU2VydmljZXMgQ0EgLSBH
# MjAeFw0xMjEwMTgwMDAwMDBaFw0yMDEyMjkyMzU5NTlaMGIxCzAJBgNVBAYTAlVT
# MR0wGwYDVQQKExRTeW1hbnRlYyBDb3Jwb3JhdGlvbjE0MDIGA1UEAxMrU3ltYW50
# ZWMgVGltZSBTdGFtcGluZyBTZXJ2aWNlcyBTaWduZXIgLSBHNDCCASIwDQYJKoZI
# hvcNAQEBBQADggEPADCCAQoCggEBAKJjCzlEuLsjp0RJuw7/ofBhClOTsJjbrSwP
# SsVu/4Y8U1UPFc4EPyv9qZaW2b5heQtbyUyGduXgQ0sile7CK0PBn9hotI5AT+6F
# OLkRxSPyZFjwFTJvTlehroikAtcqHs1L4d1j1ReJMluwXplaqJ0oUA4X7pbbYTtF
# UR3PElYLkkf8q672Zj1HrHBy55LnX80QucSDZJQZvSWA4ejSIqXQugJ6oXeTW2XD
# 7hd0vEGGKtwITIySjJEtnndEH2jWqHR32w5bMotWizO92WPISZ06xcXqMwvS8aMb
# 9Iu+2bNXizveBKd6IrIkri7HcMW+ToMmCPsLvalPmQjhEChyqs0CAwEAAaOCAVcw
# ggFTMAwGA1UdEwEB/wQCMAAwFgYDVR0lAQH/BAwwCgYIKwYBBQUHAwgwDgYDVR0P
# AQH/BAQDAgeAMHMGCCsGAQUFBwEBBGcwZTAqBggrBgEFBQcwAYYeaHR0cDovL3Rz
# LW9jc3Aud3Muc3ltYW50ZWMuY29tMDcGCCsGAQUFBzAChitodHRwOi8vdHMtYWlh
# LndzLnN5bWFudGVjLmNvbS90c3MtY2EtZzIuY2VyMDwGA1UdHwQ1MDMwMaAvoC2G
# K2h0dHA6Ly90cy1jcmwud3Muc3ltYW50ZWMuY29tL3Rzcy1jYS1nMi5jcmwwKAYD
# VR0RBCEwH6QdMBsxGTAXBgNVBAMTEFRpbWVTdGFtcC0yMDQ4LTIwHQYDVR0OBBYE
# FEbGaaMOShQe1UzaUmMXP142vA3mMB8GA1UdIwQYMBaAFF+a9W5czMx0mtTdfe8/
# 2+xMgC7dMA0GCSqGSIb3DQEBBQUAA4IBAQB4O7SRKgBM8I9iMDd4o4QnB28Yst4l
# 3KDUlAOqhk4ln5pAAxzdzuN5yyFoBtq2MrRtv/QsJmMz5ElkbQ3mw2cO9wWkNWx8
# iRbG6bLfsundIMZxD82VdNy2XN69Nx9DeOZ4tc0oBCCjqvFLxIgpkQ6A0RH83Vx2
# bk9eDkVGQW4NsOo4mrE62glxEPwcebSAe6xp9P2ctgwWK/F/Wwk9m1viFsoTgW0A
# LjgNqCmPLOGy9FqpAa8VnCwvSRvbIrvD/niUUcOGsYKIXfA9tFGheTMrLnu53CAJ
# E3Hrahlbz+ilMFcsiUk/uc9/yb8+ImhjU5q9aXSsxR08f5Lgw7wc2AR1MIIFIjCC
# BAqgAwIBAgIQBuAUqkL8vKnzQr5Tqe4gIjANBgkqhkiG9w0BAQsFADByMQswCQYD
# VQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGln
# aWNlcnQuY29tMTEwLwYDVQQDEyhEaWdpQ2VydCBTSEEyIEFzc3VyZWQgSUQgQ29k
# ZSBTaWduaW5nIENBMB4XDTE2MDQxNDAwMDAwMFoXDTE5MDYxOTEyMDAwMFowXzEL
# MAkGA1UEBhMCVVMxEzARBgNVBAgTCkNhbGlmb3JuaWExDzANBgNVBAcTBklydmlu
# ZTEUMBIGA1UEChMLS29mYXgsIEluYy4xFDASBgNVBAMTC0tvZmF4LCBJbmMuMIIB
# IjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAp1sCHr59YU9R9Hsy1loUNPur
# VuF0bcL0/9usV+BHWYSM1jAWwsnrzKmss7dVu4Ld6XeB+vjE7lFT4SNPNPbZguiS
# 1jVKD8Kk83JszhHIRIf6fecSChF56Nj9mwHz9N3j9QmeBQhxUaoBZSrbOsW+g25U
# HxLQtX4Uc7xZ+BVWyqWs31ZcENgz1rWM8FID4TBZoH6Kp23EGVAL06vgcBUegsB5
# XwIf1m2iPI02akjuB/ib5lrTufmYGJkRCyALh+I25QVaXXuSZ5um3EMdUfkG2OGh
# 5g/rd7CPz7H8j1UXJu/wgDxOsNY/vCDUP6SyWBKFyBSxQBSZzBku0Xio/aPLvQID
# AQABo4IBxTCCAcEwHwYDVR0jBBgwFoAUWsS5eyoKo6XqcQPAYPkt9mV1DlgwHQYD
# VR0OBBYEFNgH2nzkwJngFB8kbXfVxmrR9zf8MA4GA1UdDwEB/wQEAwIHgDATBgNV
# HSUEDDAKBggrBgEFBQcDAzB3BgNVHR8EcDBuMDWgM6Axhi9odHRwOi8vY3JsMy5k
# aWdpY2VydC5jb20vc2hhMi1hc3N1cmVkLWNzLWcxLmNybDA1oDOgMYYvaHR0cDov
# L2NybDQuZGlnaWNlcnQuY29tL3NoYTItYXNzdXJlZC1jcy1nMS5jcmwwTAYDVR0g
# BEUwQzA3BglghkgBhv1sAwEwKjAoBggrBgEFBQcCARYcaHR0cHM6Ly93d3cuZGln
# aWNlcnQuY29tL0NQUzAIBgZngQwBBAEwgYQGCCsGAQUFBwEBBHgwdjAkBggrBgEF
# BQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29tME4GCCsGAQUFBzAChkJodHRw
# Oi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRTSEEyQXNzdXJlZElEQ29k
# ZVNpZ25pbmdDQS5jcnQwDAYDVR0TAQH/BAIwADANBgkqhkiG9w0BAQsFAAOCAQEA
# 5Pxp9n7UG1jGtY6fI0DQ6HdIzewX7ffaahn1PaBLmRzk/Bo6qhz+FyGqB4Yalnpv
# OWZxqjKtXh7+sbZn7p59ypS55lBk1A6DBNSooOvFaEF/+edq7fO4PJZeNXAeT0zn
# TxQsfr2HaVtdThasWH86RTIBRIDHqbLeL8n1CVxjuxcBh6xRWlqNWlslwniixEVe
# jEaB4z/xkXT4lvNA7Eav7J2D8stRkSBfefwVpgCNDjf6oh5R8pEPYJR4Xo3YSleK
# sTyZgZCdUtsXzFOwVMOcjqhFBpZTO1PZj7IsJbWR67VmCB5rrWs059XS8ui2SCPA
# REKUxt2Yk9sguJqS4ph9SzCCBTAwggQYoAMCAQICEAQJGBtf1btmdVNDtW+VUAgw
# DQYJKoZIhvcNAQELBQAwZTELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0
# IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEkMCIGA1UEAxMbRGlnaUNl
# cnQgQXNzdXJlZCBJRCBSb290IENBMB4XDTEzMTAyMjEyMDAwMFoXDTI4MTAyMjEy
# MDAwMFowcjELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcG
# A1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTExMC8GA1UEAxMoRGlnaUNlcnQgU0hBMiBB
# c3N1cmVkIElEIENvZGUgU2lnbmluZyBDQTCCASIwDQYJKoZIhvcNAQEBBQADggEP
# ADCCAQoCggEBAPjTsxx/DhGvZ3cH0wsxSRnP0PtFmbE620T1f+Wondsy13Hqdp0F
# LreP+pJDwKX5idQ3Gde2qvCchqXYJawOeSg6funRZ9PG+yknx9N7I5TkkSOWkHeC
# +aGEI2YSVDNQdLEoJrskacLCUvIUZ4qJRdQtoaPpiCwgla4cSocI3wz14k1gGL6q
# xLKucDFmM3E+rHCiq85/6XzLkqHlOzEcz+ryCuRXu0q16XTmK/5sy350OTYNkO/k
# tU6kqepqCquE86xnTrXE94zRICUj6whkPlKWwfIPEvTFjg/BougsUfdzvL2FsWKD
# c0GCB+Q4i2pzINAPZHM8np+mM6n9Gd8lk9ECAwEAAaOCAc0wggHJMBIGA1UdEwEB
# /wQIMAYBAf8CAQAwDgYDVR0PAQH/BAQDAgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMD
# MHkGCCsGAQUFBwEBBG0wazAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNl
# cnQuY29tMEMGCCsGAQUFBzAChjdodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20v
# RGlnaUNlcnRBc3N1cmVkSURSb290Q0EuY3J0MIGBBgNVHR8EejB4MDqgOKA2hjRo
# dHRwOi8vY3JsNC5kaWdpY2VydC5jb20vRGlnaUNlcnRBc3N1cmVkSURSb290Q0Eu
# Y3JsMDqgOKA2hjRodHRwOi8vY3JsMy5kaWdpY2VydC5jb20vRGlnaUNlcnRBc3N1
# cmVkSURSb290Q0EuY3JsME8GA1UdIARIMEYwOAYKYIZIAYb9bAACBDAqMCgGCCsG
# AQUFBwIBFhxodHRwczovL3d3dy5kaWdpY2VydC5jb20vQ1BTMAoGCGCGSAGG/WwD
# MB0GA1UdDgQWBBRaxLl7KgqjpepxA8Bg+S32ZXUOWDAfBgNVHSMEGDAWgBRF66Kv
# 9JLLgjEtUYunpyGd823IDzANBgkqhkiG9w0BAQsFAAOCAQEAPuwNWiSz8yLRFcgs
# fCUpdqgdXRwtOhrE7zBh134LYP3DPQ/Er4v97yrfIFU3sOH20ZJ1D1G0bqWOWuJe
# JIFOEKTuP3GOYw4TS63XX0R58zYUBor3nEZOXP+QsRsHDpEV+7qvtVHCjSSuJMbH
# JyqhKSgaOnEoAjwukaPAJRHinBRHoXpoaK+bp1wgXNlxsQyPu6j4xRJon89Ay0BE
# pRPw5mQMJQhCMrI2iiQC/i9yfhzXSUWW6Fkd6fp0ZGuy62ZD2rOwjNXpDd32ASDO
# mTFjPQgaGLOBm0/GkxAG/AeB+ova+YJJ92JuoVP6EpQYhS6SkepobEQysmah5xik
# mmRR7zGCBDcwggQzAgEBMIGGMHIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdp
# Q2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xMTAvBgNVBAMTKERp
# Z2lDZXJ0IFNIQTIgQXNzdXJlZCBJRCBDb2RlIFNpZ25pbmcgQ0ECEAbgFKpC/Lyp
# 80K+U6nuICIwCQYFKw4DAhoFAKB4MBgGCisGAQQBgjcCAQwxCjAIoAKAAKECgAAw
# GQYJKoZIhvcNAQkDMQwGCisGAQQBgjcCAQQwHAYKKwYBBAGCNwIBCzEOMAwGCisG
# AQQBgjcCARUwIwYJKoZIhvcNAQkEMRYEFOt9KYNzFMcl9keRSitV3UVY6MOCMA0G
# CSqGSIb3DQEBAQUABIIBAAfz6HC1MF3VHBISZwqn9y1qvL0IU1jbsJ6M+KcZWeP9
# ScA1SumYyoFEuD7l6L8tp1ttP5vIdp9Tzpfv2fGGC6ZEKdZvOr087BWd9DEW9pOI
# hwVbrTxBtaLAugkPOK+LFLKgN/77FxljWQ0JBNHtIA4v2J5OvJQt86IR7F7wK7SY
# /3rUtcDbJCDJfMQwHbCAA+XtEQFyXlwyno8/EyfSBQtFNLA4CJBGhriXyq5klIcb
# n6PwEORF3GAbOpXXumX6E7FNaKM3bfiZczts/oCKuLlN2uKGa8kUSFuP5KUVF1PN
# E/po2oCvCVzUQWBOarwp4I4zJ7sc2i1AeLlgSHBF3MuhggILMIICBwYJKoZIhvcN
# AQkGMYIB+DCCAfQCAQEwcjBeMQswCQYDVQQGEwJVUzEdMBsGA1UEChMUU3ltYW50
# ZWMgQ29ycG9yYXRpb24xMDAuBgNVBAMTJ1N5bWFudGVjIFRpbWUgU3RhbXBpbmcg
# U2VydmljZXMgQ0EgLSBHMgIQDs/0OMj+vzVuBNhqmBsaUDAJBgUrDgMCGgUAoF0w
# GAYJKoZIhvcNAQkDMQsGCSqGSIb3DQEHATAcBgkqhkiG9w0BCQUxDxcNMTkwMjE0
# MTU1MzQ3WjAjBgkqhkiG9w0BCQQxFgQUHSWZNJs+IQdwDwIlqFyAgpXpUZwwDQYJ
# KoZIhvcNAQEBBQAEggEACmmTmSm4h1IxADw6RQd7YIEq8u4Kp9ccoNq8ca7tt29d
# vi+CEY199BulJwd4aikJ6fUnl2vX6/+JqJQMHjPGsLNod7YEF1HTT6GKRkfg3UKq
# JIP90GKMwSybjazTdxEMfFD8o6QjqIAy6hma3eCCVfJ1hKuVsKYQ5RkPekM/EeHK
# 9Y5lcV9KzuITU9J0C4f3jYYoaznPpmokMFt2ZtHWov0Mr/zxf9g0F1kDrbcdCLY2
# MuueaW1NxCjm1p0nC1RLjS8SzFyZ9FrJPDz7xw0JcRZgTMfOJZ2upl0BCCF7KZPw
# Zb7UQ/XY2tynITvhVLvoLeBztLftOPhMn5txtErUoQ==
# SIG # End signature block
